@@ -13,6 +13,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import functools
 import urllib
 from collections import defaultdict
 
@@ -468,9 +469,20 @@ class FakeIssue(object):
         return self._fake_pull_request.number
 
 
+@functools.total_ordering
 class FakeFile(object):
-    def __init__(self, filename):
+    def __init__(self, filename, previous_filename=None):
         self.filename = filename
+        if previous_filename is not None:
+            self.previous_filename = previous_filename
+
+    def __eq__(self, other):
+        return self.filename == other.filename
+
+    def __lt__(self, other):
+        return self.filename < other.filename
+
+    __hash__ = object.__hash__
 
 
 class FakePull(object):
@@ -482,8 +494,7 @@ class FakePull(object):
 
     def files(self):
         # Github lists max. 300 files of a PR in alphabetical order
-        return [FakeFile(fn)
-                for fn in sorted(self._fake_pull_request.files)][:300]
+        return sorted(self._fake_pull_request.files)[:300]
 
     def reviews(self):
         return self._fake_pull_request.reviews
@@ -641,8 +652,9 @@ class FakeGithubSession(object):
         if re.match(r'.*/app/installations/.*/access_tokens', url):
             expiry = (datetime.datetime.now(utc) + datetime.timedelta(
                 minutes=60)).replace(microsecond=0).isoformat()
+            install_id = url.split('/')[-2]
             data = {
-                'token': 'fake',
+                'token': 'token-%s' % install_id,
                 'expires_at': expiry,
             }
             return FakeResponse(data, 201)
@@ -767,6 +779,12 @@ class FakeGithubData(object):
         self.reports = []
         self.fail_check_run_creation = False
 
+    def __repr__(self):
+        return ("pull_requests:%s repos:%s reports:%s "
+                "fail_check_run_creation:%s" % (
+                    self.pull_requests, self.repos, self.reports,
+                    self.fail_check_run_creation))
+
 
 class FakeGithubClient(object):
 
