@@ -15,9 +15,9 @@
 
 import textwrap
 
-import zuul.rpcclient
+from zuul.model import PromoteEvent
 
-from tests.base import ZuulTestCase
+from tests.base import ZuulTestCase, simple_layout
 
 
 class TestGerritCircularDependencies(ZuulTestCase):
@@ -55,7 +55,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
         # already (without approvals), we need to clear the cache
         # first.
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         A.addApproval("Code-Review", 2)
         B.addApproval("Code-Review", 2)
@@ -108,7 +108,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
         # already (without approvals), we need to clear the cache
         # first.
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         A.addApproval("Code-Review", 2)
         B.addApproval("Code-Review", 2)
@@ -172,7 +172,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
         # already (without approvals), we need to clear the cache
         # first.
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         A.addApproval("Code-Review", 2)
         B.addApproval("Code-Review", 2)
@@ -180,7 +180,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
         self.fake_gerrit.addEvent(A.addApproval("Approved", 1))
         self.waitUntilSettled()
 
-        self.assertEqual(A.reported, 1)
+        self.assertEqual(A.reported, 2)
         self.assertEqual(B.reported, 1)
         self.assertEqual(A.data["status"], "NEW")
         self.assertEqual(B.data["status"], "NEW")
@@ -223,7 +223,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
         # already (without approvals), we need to clear the cache
         # first.
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         self.executor_server.hold_jobs_in_build = True
         A.addApproval("Code-Review", 2)
@@ -285,7 +285,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
         # already (without approvals), we need to clear the cache
         # first.
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         A.addApproval("Code-Review", 2)
         B.addApproval("Code-Review", 2)
@@ -398,7 +398,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
         # already (without approvals), we need to clear the cache
         # first.
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         A.addApproval("Code-Review", 2)
         B.addApproval("Code-Review", 2)
@@ -447,7 +447,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
         # already (without approvals), we need to clear the cache
         # first.
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         A.addApproval("Code-Review", 2)
         B.addApproval("Code-Review", 2)
@@ -464,6 +464,75 @@ class TestGerritCircularDependencies(ZuulTestCase):
         self.assertEqual(A.data["status"], "NEW")
         self.assertEqual(B.data["status"], "NEW")
 
+    def test_failing_cycle_behind_failing_change(self):
+        self.executor_server.hold_jobs_in_build = True
+        A = self.fake_gerrit.addFakeChange("org/project", "master", "A")
+        B = self.fake_gerrit.addFakeChange("org/project", "master", "B")
+        C = self.fake_gerrit.addFakeChange("org/project", "master", "C")
+        D = self.fake_gerrit.addFakeChange("org/project", "master", "D")
+        E = self.fake_gerrit.addFakeChange("org/project", "master", "E")
+
+        # C <-> D (via commit-depends)
+        C.data["commitMessage"] = "{}\n\nDepends-On: {}\n".format(
+            C.subject, D.data["url"]
+        )
+        D.data["commitMessage"] = "{}\n\nDepends-On: {}\n".format(
+            D.subject, C.data["url"]
+        )
+
+        A.addApproval("Code-Review", 2)
+        B.addApproval("Code-Review", 2)
+        C.addApproval("Code-Review", 2)
+        D.addApproval("Code-Review", 2)
+        E.addApproval("Code-Review", 2)
+
+        self.fake_gerrit.addEvent(A.addApproval("Approved", 1))
+        self.fake_gerrit.addEvent(B.addApproval("Approved", 1))
+        # Make sure we enqueue C as part of the circular dependency with D, so
+        # we end up with the following queue state: A, B, C, ...
+        C.addApproval("Approved", 1)
+        self.fake_gerrit.addEvent(D.addApproval("Approved", 1))
+        self.fake_gerrit.addEvent(E.addApproval("Approved", 1))
+        self.waitUntilSettled()
+
+        # Fail a job of the circular dependency
+        self.executor_server.failJob("project-job", D)
+        self.executor_server.release("project-job", change="4 1")
+
+        # Fail job for item B ahead of the circular dependency so that this
+        # causes a gate reset and item C and D are moved behind item A.
+        self.executor_server.failJob("project-job", B)
+        self.executor_server.release("project-job", change="2 1")
+        self.waitUntilSettled()
+
+        # Don't fail any other jobs
+        self.executor_server.fail_tests.clear()
+        self.executor_server.hold_jobs_in_build = False
+        self.executor_server.release()
+        self.waitUntilSettled()
+
+        self.assertEqual(A.data["status"], "MERGED")
+        self.assertEqual(B.data["status"], "NEW")
+        self.assertEqual(C.data["status"], "MERGED")
+        self.assertEqual(D.data["status"], "MERGED")
+        self.assertEqual(E.data["status"], "MERGED")
+
+        self.assertHistory([
+            dict(name="project-job", result="SUCCESS", changes="1,1"),
+            dict(name="project-job", result="FAILURE", changes="1,1 2,1"),
+            # First attempt of change C and D before gate reset due to change B
+            dict(name="project-job", result="FAILURE",
+                 changes="1,1 2,1 3,1 4,1"),
+            dict(name="project-job", result="FAILURE",
+                 changes="1,1 2,1 3,1 4,1"),
+            dict(name="project-job", result="ABORTED",
+                 changes="1,1 2,1 3,1 4,1 5,1"),
+            dict(name="project-job", result="SUCCESS", changes="1,1 3,1 4,1"),
+            dict(name="project-job", result="SUCCESS", changes="1,1 3,1 4,1"),
+            dict(name="project-job", result="SUCCESS",
+                 changes="1,1 3,1 4,1 5,1"),
+        ], ordered=False)
+
     def test_dependency_on_cycle_failure(self):
         A = self.fake_gerrit.addFakeChange("org/project", "master", "A")
         B = self.fake_gerrit.addFakeChange("org/project1", "master", "B")
@@ -535,7 +604,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
         # already (without approvals), we need to clear the cache
         # first.
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         A.addApproval("Code-Review", 2)
         B.addApproval("Code-Review", 2)
@@ -667,6 +736,12 @@ class TestGerritCircularDependencies(ZuulTestCase):
         self.assertEqual(A.data["status"], "NEW")
         self.assertEqual(B.data["status"], "NEW")
 
+        buildsets = {bs.change: bs for bs in
+                     self.scheds.first.connections.connections[
+                         'database'].getBuildsets()}
+        self.assertEqual(buildsets[2].result, 'MERGE_FAILURE')
+        self.assertEqual(buildsets[1].result, 'FAILURE')
+
     def test_cycle_reporting_partial_failure(self):
         A = self.fake_gerrit.addFakeChange("org/project", "master", "A")
         B = self.fake_gerrit.addFakeChange("org/project1", "master", "B")
@@ -901,7 +976,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
         self.assertEqual(A.patchsets[-1]["approvals"][0]["value"], "1")
 
     def test_cycle_merge_conflict(self):
-        self.gearman_server.hold_merge_jobs_in_queue = True
+        self.hold_merge_jobs_in_queue = True
         A = self.fake_gerrit.addFakeChange('org/project', 'master', 'A')
         B = self.fake_gerrit.addFakeChange('org/project', 'master', 'B')
 
@@ -923,12 +998,14 @@ class TestGerritCircularDependencies(ZuulTestCase):
         # We only want to have a merge failure for the first item in the queue
         tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
         items = tenant.layout.pipelines['gate'].getAllItems()
-        items[0].current_build_set.unable_to_merge = True
+        context = self.createZKContext()
+        items[0].current_build_set.updateAttributes(context,
+                                                    unable_to_merge=True)
 
         self.waitUntilSettled()
 
-        self.gearman_server.hold_merge_jobs_in_queue = False
-        self.gearman_server.release()
+        self.hold_merge_jobs_in_queue = False
+        self.merger_api.release()
         self.waitUntilSettled()
 
         self.assertEqual(A.reported, 0)
@@ -1034,7 +1111,8 @@ class TestGerritCircularDependencies(ZuulTestCase):
 
         vars_builds = [b for b in self.builds if b.name == "project-vars-job"]
         self.assertEqual(len(vars_builds), 1)
-        self.assertEqual(vars_builds[0].parameters["vars"]["test_var"], "pass")
+        self.assertEqual(vars_builds[0].job.combined_variables["test_var"],
+                         "pass")
 
         self.executor_server.release()
         self.waitUntilSettled()
@@ -1048,7 +1126,8 @@ class TestGerritCircularDependencies(ZuulTestCase):
 
         vars_builds = [b for b in self.builds if b.name == "project-vars-job"]
         self.assertEqual(len(vars_builds), 1)
-        self.assertEqual(vars_builds[0].parameters["vars"]["test_var"], "pass")
+        self.assertEqual(vars_builds[0].job.combined_variables["test_var"],
+                         "pass")
 
         self.executor_server.release()
         self.waitUntilSettled()
@@ -1062,7 +1141,8 @@ class TestGerritCircularDependencies(ZuulTestCase):
 
         vars_builds = [b for b in self.builds if b.name == "project-vars-job"]
         self.assertEqual(len(vars_builds), 1)
-        self.assertEqual(vars_builds[0].parameters["vars"]["test_var"], "pass")
+        self.assertEqual(vars_builds[0].job.combined_variables["test_var"],
+                         "pass")
 
         self.executor_server.release()
         self.waitUntilSettled()
@@ -1083,7 +1163,95 @@ class TestGerritCircularDependencies(ZuulTestCase):
         vars_builds = [b for b in self.builds if b.name == "project-vars-job"]
         self.assertEqual(len(vars_builds), 3)
         for build in vars_builds:
-            self.assertEqual(build.parameters["vars"]["test_var"], "pass")
+            self.assertEqual(build.job.combined_variables["test_var"],
+                             "pass")
+
+        self.executor_server.hold_jobs_in_build = False
+        self.executor_server.release()
+        self.waitUntilSettled()
+
+        self.assertEqual(A.reported, 3)
+        self.assertEqual(B.reported, 3)
+        self.assertEqual(C.reported, 3)
+        self.assertEqual(A.data["status"], "MERGED")
+        self.assertEqual(B.data["status"], "MERGED")
+        self.assertEqual(C.data["status"], "MERGED")
+
+    def test_bundle_id_in_zuul_var(self):
+        A = self.fake_gerrit.addFakeChange("org/project1", "master", "A")
+        B = self.fake_gerrit.addFakeChange("org/project1", "master", "B")
+        C = self.fake_gerrit.addFakeChange("org/project1", "master", "C")
+
+        # A <-> B (via commit-depends)
+        A.data["commitMessage"] = "{}\n\nDepends-On: {}\n".format(
+            A.subject, B.data["url"]
+        )
+        B.data["commitMessage"] = "{}\n\nDepends-On: {}\n".format(
+            B.subject, A.data["url"]
+        )
+
+        self.executor_server.hold_jobs_in_build = True
+
+        # bundle_id should be in check build of A
+        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+        var_zuul_items = self.builds[0].parameters["zuul"]["items"]
+        self.assertEqual(len(var_zuul_items), 2)
+        self.assertIn("bundle_id", var_zuul_items[0])
+        bundle_id_0 = var_zuul_items[0]["bundle_id"]
+        self.assertIn("bundle_id", var_zuul_items[1])
+        bundle_id_1 = var_zuul_items[1]["bundle_id"]
+        self.assertEqual(bundle_id_0, bundle_id_1)
+        self.executor_server.release()
+        self.waitUntilSettled()
+        self.assertEqual(len(A.patchsets[-1]["approvals"]), 1)
+        self.assertEqual(A.patchsets[-1]["approvals"][0]["type"], "Verified")
+        self.assertEqual(A.patchsets[-1]["approvals"][0]["value"], "1")
+
+        # bundle_id should be in check build of B
+        self.fake_gerrit.addEvent(B.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+        var_zuul_items = self.builds[0].parameters["zuul"]["items"]
+        self.assertEqual(len(var_zuul_items), 2)
+        self.assertIn("bundle_id", var_zuul_items[0])
+        bundle_id_0 = var_zuul_items[0]["bundle_id"]
+        self.assertIn("bundle_id", var_zuul_items[1])
+        bundle_id_1 = var_zuul_items[1]["bundle_id"]
+        self.assertEqual(bundle_id_0, bundle_id_1)
+        self.executor_server.release()
+        self.waitUntilSettled()
+        self.assertEqual(len(B.patchsets[-1]["approvals"]), 1)
+        self.assertEqual(B.patchsets[-1]["approvals"][0]["type"], "Verified")
+        self.assertEqual(B.patchsets[-1]["approvals"][0]["value"], "1")
+
+        # bundle_id should not be in check build of C
+        self.fake_gerrit.addEvent(C.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+        var_zuul_items = self.builds[0].parameters["zuul"]["items"]
+        self.assertEqual(len(var_zuul_items), 1)
+        self.assertNotIn("bundle_id", var_zuul_items[0])
+        self.executor_server.release()
+        self.waitUntilSettled()
+        self.assertEqual(len(C.patchsets[-1]["approvals"]), 1)
+        self.assertEqual(C.patchsets[-1]["approvals"][0]["type"], "Verified")
+        self.assertEqual(C.patchsets[-1]["approvals"][0]["value"], "1")
+
+        # bundle_id should be in gate jobs of A and B, but not in C
+        A.addApproval("Code-Review", 2)
+        B.addApproval("Code-Review", 2)
+        C.addApproval("Code-Review", 2)
+        self.fake_gerrit.addEvent(A.addApproval("Approved", 1))
+        self.fake_gerrit.addEvent(B.addApproval("Approved", 1))
+        self.fake_gerrit.addEvent(C.addApproval("Approved", 1))
+        self.waitUntilSettled()
+        var_zuul_items = self.builds[-1].parameters["zuul"]["items"]
+        self.assertEqual(len(var_zuul_items), 3)
+        self.assertIn("bundle_id", var_zuul_items[0])
+        bundle_id_0 = var_zuul_items[0]["bundle_id"]
+        self.assertIn("bundle_id", var_zuul_items[1])
+        bundle_id_1 = var_zuul_items[1]["bundle_id"]
+        self.assertEqual(bundle_id_0, bundle_id_1)
+        self.assertNotIn("bundle_id", var_zuul_items[2])
 
         self.executor_server.hold_jobs_in_build = False
         self.executor_server.release()
@@ -1177,7 +1345,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
 
         D.setMerged()
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         # Pretend D was merged so we can gate the cycle
         self.fake_gerrit.addEvent(A.addApproval("Approved", 1))
@@ -1185,7 +1353,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
 
         self.assertEqual(A.reported, 3)
         self.assertEqual(B.reported, 3)
-        self.assertEqual(C.reported, 3)
+        self.assertEqual(C.reported, 6)
         self.assertEqual(A.data["status"], "MERGED")
         self.assertEqual(B.data["status"], "MERGED")
         self.assertEqual(C.data["status"], "MERGED")
@@ -1211,7 +1379,7 @@ class TestGerritCircularDependencies(ZuulTestCase):
         self.assertEqual(A.patchsets[-1]["approvals"][0]["value"], "-1")
 
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         A.addApproval("Code-Review", 2)
         B.addApproval("Code-Review", 2)
@@ -1219,18 +1387,18 @@ class TestGerritCircularDependencies(ZuulTestCase):
         self.fake_gerrit.addEvent(A.addApproval("Approved", 1))
         self.waitUntilSettled()
 
-        self.assertEqual(A.reported, 1)
+        self.assertEqual(A.reported, 2)
         self.assertEqual(A.data["status"], "NEW")
         self.assertEqual(B.data["status"], "NEW")
 
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         B.setMerged()
         self.fake_gerrit.addEvent(A.addApproval("Approved", 1))
         self.waitUntilSettled()
 
-        self.assertEqual(A.reported, 3)
+        self.assertEqual(A.reported, 4)
         self.assertEqual(A.data["status"], "MERGED")
 
     def test_promote_cycle(self):
@@ -1255,14 +1423,9 @@ class TestGerritCircularDependencies(ZuulTestCase):
         self.fake_gerrit.addEvent(A.addApproval("Approved", 1))
         self.waitUntilSettled()
 
-        client = zuul.rpcclient.RPCClient("127.0.0.1",
-                                          self.gearman_server.port)
-        self.addCleanup(client.shutdown)
-        client.promote(
-            tenant="tenant-one",
-            pipeline="gate",
-            change_ids=["2,1"]
-        )
+        event = PromoteEvent('tenant-one', 'gate', ["2,1"])
+        self.scheds.first.sched.pipeline_management_events['tenant-one'][
+            'gate'].put(event)
         self.waitUntilSettled()
 
         self.assertEqual(len(self.builds), 4)
@@ -1289,10 +1452,236 @@ class TestGerritCircularDependencies(ZuulTestCase):
         self.assertEqual(B.data["status"], "MERGED")
         self.assertEqual(C.data["status"], "MERGED")
 
+    def test_shared_queue_removed(self):
+        self.executor_server.hold_jobs_in_build = True
+
+        A = self.fake_gerrit.addFakeChange('org/project', 'master', 'A')
+        B = self.fake_gerrit.addFakeChange('org/project1', 'master', 'B')
+        C = self.fake_gerrit.addFakeChange('org/project', 'master', 'C')
+
+        # A <-> B
+        A.data["commitMessage"] = "{}\n\nDepends-On: {}\n".format(
+            A.subject, B.data["url"]
+        )
+        B.data["commitMessage"] = "{}\n\nDepends-On: {}\n".format(
+            B.subject, A.data["url"]
+        )
+
+        A.addApproval('Code-Review', 2)
+        B.addApproval('Code-Review', 2)
+        C.addApproval('Code-Review', 2)
+
+        self.fake_gerrit.addEvent(A.addApproval('Approved', 1))
+        self.fake_gerrit.addEvent(B.addApproval('Approved', 1))
+        self.fake_gerrit.addEvent(C.addApproval('Approved', 1))
+
+        self.waitUntilSettled()
+        self.executor_server.release('.*-merge')
+        self.waitUntilSettled()
+        self.executor_server.release('.*-merge')
+        self.waitUntilSettled()
+
+        # Remove the shared queue.
+        self.commitConfigUpdate(
+            'common-config',
+            'layouts/circular-dependency-shared-queue-removed.yaml')
+
+        self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+        self.waitUntilSettled()
+
+        self.executor_server.hold_jobs_in_build = False
+        self.executor_server.release()
+        self.waitUntilSettled()
+
+        self.assertEqual(A.data['status'], 'NEW')
+        self.assertEqual(B.data['status'], 'NEW')
+        self.assertEqual(C.data['status'], 'MERGED')
+
+    def test_submitted_together(self):
+        self.fake_gerrit._fake_submit_whole_topic = True
+        A = self.fake_gerrit.addFakeChange('org/project1', "master", "A",
+                                           topic='test-topic')
+        B = self.fake_gerrit.addFakeChange('org/project2', "master", "B",
+                                           topic='test-topic')
+
+        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
+        self.fake_gerrit.addEvent(B.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+
+        self.assertEqual(len(A.patchsets[-1]["approvals"]), 1)
+        self.assertEqual(A.patchsets[-1]["approvals"][0]["type"], "Verified")
+        self.assertEqual(A.patchsets[-1]["approvals"][0]["value"], "1")
+
+        self.assertEqual(len(B.patchsets[-1]["approvals"]), 1)
+        self.assertEqual(B.patchsets[-1]["approvals"][0]["type"], "Verified")
+        self.assertEqual(B.patchsets[-1]["approvals"][0]["value"], "1")
+
+        # We're about to add approvals to changes without adding the
+        # triggering events to Zuul, so that we can be sure that it is
+        # enqueing the changes based on dependencies, not because of
+        # triggering events.  Since it will have the changes cached
+        # already (without approvals), we need to clear the cache
+        # first.
+        for connection in self.scheds.first.connections.connections.values():
+            connection.maintainCache([], max_age=0)
+
+        A.addApproval("Code-Review", 2)
+        B.addApproval("Code-Review", 2)
+        A.addApproval("Approved", 1)
+        self.fake_gerrit.addEvent(B.addApproval("Approved", 1))
+        self.waitUntilSettled()
+
+        self.assertEqual(A.reported, 3)
+        self.assertEqual(B.reported, 3)
+        self.assertEqual(A.data["status"], "MERGED")
+        self.assertEqual(B.data["status"], "MERGED")
+
+    def test_submitted_together_git(self):
+        self.fake_gerrit._fake_submit_whole_topic = True
+
+        A = self.fake_gerrit.addFakeChange('org/project1', "master", "A")
+        B = self.fake_gerrit.addFakeChange('org/project1', "master", "B")
+        C = self.fake_gerrit.addFakeChange('org/project1', "master", "C")
+        D = self.fake_gerrit.addFakeChange('org/project1', "master", "D")
+        E = self.fake_gerrit.addFakeChange('org/project1', "master", "E")
+        F = self.fake_gerrit.addFakeChange('org/project1', "master", "F")
+        G = self.fake_gerrit.addFakeChange('org/project1', "master", "G")
+        G.setDependsOn(F, 1)
+        F.setDependsOn(E, 1)
+        E.setDependsOn(D, 1)
+        D.setDependsOn(C, 1)
+        C.setDependsOn(B, 1)
+        B.setDependsOn(A, 1)
+
+        self.fake_gerrit.addEvent(C.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+
+        self.assertEqual(len(C.patchsets[-1]["approvals"]), 1)
+        self.assertEqual(C.patchsets[-1]["approvals"][0]["type"], "Verified")
+        self.assertEqual(C.patchsets[-1]["approvals"][0]["value"], "1")
+        self.assertEqual(A.queried, 1)
+        self.assertEqual(B.queried, 1)
+        self.assertEqual(C.queried, 1)
+        self.assertEqual(D.queried, 1)
+        self.assertEqual(E.queried, 1)
+        self.assertEqual(F.queried, 1)
+        self.assertEqual(G.queried, 1)
+        self.assertHistory([
+            dict(name="project1-job", result="SUCCESS",
+                 changes="1,1 2,1 3,1"),
+            dict(name="project-vars-job", result="SUCCESS",
+                 changes="1,1 2,1 3,1"),
+        ], ordered=False)
+
+    def test_submitted_together_git_topic(self):
+        self.fake_gerrit._fake_submit_whole_topic = True
+
+        A = self.fake_gerrit.addFakeChange('org/project1', "master", "A",
+                                           topic='test-topic')
+        B = self.fake_gerrit.addFakeChange('org/project1', "master", "B",
+                                           topic='test-topic')
+        C = self.fake_gerrit.addFakeChange('org/project1', "master", "C",
+                                           topic='test-topic')
+        D = self.fake_gerrit.addFakeChange('org/project1', "master", "D",
+                                           topic='test-topic')
+        E = self.fake_gerrit.addFakeChange('org/project1', "master", "E",
+                                           topic='test-topic')
+        F = self.fake_gerrit.addFakeChange('org/project1', "master", "F",
+                                           topic='test-topic')
+        G = self.fake_gerrit.addFakeChange('org/project1', "master", "G",
+                                           topic='test-topic')
+        G.setDependsOn(F, 1)
+        F.setDependsOn(E, 1)
+        E.setDependsOn(D, 1)
+        D.setDependsOn(C, 1)
+        C.setDependsOn(B, 1)
+        B.setDependsOn(A, 1)
+
+        self.fake_gerrit.addEvent(C.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+
+        self.assertEqual(len(C.patchsets[-1]["approvals"]), 1)
+        self.assertEqual(C.patchsets[-1]["approvals"][0]["type"], "Verified")
+        self.assertEqual(C.patchsets[-1]["approvals"][0]["value"], "1")
+        self.assertEqual(A.queried, 8)
+        self.assertEqual(B.queried, 8)
+        self.assertEqual(C.queried, 8)
+        self.assertEqual(D.queried, 8)
+        self.assertEqual(E.queried, 8)
+        self.assertEqual(F.queried, 8)
+        self.assertEqual(G.queried, 8)
+        self.assertHistory([
+            dict(name="project1-job", result="SUCCESS",
+                 changes="7,1 6,1 5,1 4,1 1,1 2,1 3,1"),
+            dict(name="project-vars-job", result="SUCCESS",
+                 changes="7,1 6,1 5,1 4,1 1,1 2,1 3,1"),
+        ], ordered=False)
+
+    @simple_layout('layouts/submitted-together-per-branch.yaml')
+    def test_submitted_together_per_branch(self):
+        self.fake_gerrit._fake_submit_whole_topic = True
+        self.create_branch('org/project2', 'stable/foo')
+        A = self.fake_gerrit.addFakeChange('org/project1', "master", "A",
+                                           topic='test-topic')
+        B = self.fake_gerrit.addFakeChange('org/project2', "stable/foo", "B",
+                                           topic='test-topic')
+
+        A.addApproval("Code-Review", 2)
+        B.addApproval("Code-Review", 2)
+        A.addApproval("Approved", 1)
+        self.fake_gerrit.addEvent(B.addApproval("Approved", 1))
+        self.waitUntilSettled()
+
+        self.assertEqual(A.reported, 0)
+        self.assertEqual(B.reported, 1)
+        self.assertEqual(A.data["status"], "NEW")
+        self.assertEqual(B.data["status"], "NEW")
+        self.assertIn("does not share a change queue", B.messages[-1])
+
+    @simple_layout('layouts/deps-by-topic.yaml')
+    def test_deps_by_topic(self):
+        A = self.fake_gerrit.addFakeChange('org/project1', "master", "A",
+                                           topic='test-topic')
+        B = self.fake_gerrit.addFakeChange('org/project2', "master", "B",
+                                           topic='test-topic')
+
+        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
+        self.fake_gerrit.addEvent(B.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+
+        self.assertEqual(len(A.patchsets[-1]["approvals"]), 1)
+        self.assertEqual(A.patchsets[-1]["approvals"][0]["type"], "Verified")
+        self.assertEqual(A.patchsets[-1]["approvals"][0]["value"], "1")
+
+        self.assertEqual(len(B.patchsets[-1]["approvals"]), 1)
+        self.assertEqual(B.patchsets[-1]["approvals"][0]["type"], "Verified")
+        self.assertEqual(B.patchsets[-1]["approvals"][0]["value"], "1")
+
+        # We're about to add approvals to changes without adding the
+        # triggering events to Zuul, so that we can be sure that it is
+        # enqueing the changes based on dependencies, not because of
+        # triggering events.  Since it will have the changes cached
+        # already (without approvals), we need to clear the cache
+        # first.
+        for connection in self.scheds.first.connections.connections.values():
+            connection.maintainCache([], max_age=0)
+
+        A.addApproval("Code-Review", 2)
+        B.addApproval("Code-Review", 2)
+        A.addApproval("Approved", 1)
+        self.fake_gerrit.addEvent(B.addApproval("Approved", 1))
+        self.waitUntilSettled()
+
+        self.assertEqual(A.reported, 3)
+        self.assertEqual(B.reported, 3)
+        self.assertEqual(A.data["status"], "MERGED")
+        self.assertEqual(B.data["status"], "MERGED")
+
 
 class TestGithubCircularDependencies(ZuulTestCase):
     config_file = "zuul-gerrit-github.conf"
     tenant_config_file = "config/circular-dependencies/main.yaml"
+    scheduler_count = 1
 
     def test_cycle_not_ready(self):
         A = self.fake_github.openFakePullRequest("gh/project", "master", "A")
