@@ -11,9 +11,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import configparser
 
 from zuul.lib.fingergw import FingerGateway
-from zuul.zk import ZooKeeperClient
 from zuul.zk.components import BaseComponent, ComponentRegistry
 
 from tests.base import iterate_timeout, ZuulTestCase, ZuulWebFixture
@@ -25,15 +25,6 @@ class TestComponentRegistry(ZuulTestCase):
     def setUp(self):
         super().setUp()
 
-        self.host = '::'
-        self.zk_client = ZooKeeperClient(
-            self.zk_chroot_fixture.zk_hosts,
-            tls_cert=self.zk_chroot_fixture.zookeeper_cert,
-            tls_key=self.zk_chroot_fixture.zookeeper_key,
-            tls_ca=self.zk_chroot_fixture.zookeeper_ca,
-        )
-        self.addCleanup(self.zk_client.disconnect)
-        self.zk_client.connect()
         self.component_registry = ComponentRegistry(self.zk_client)
 
     def assertComponentAttr(self, component_name, attr_name,
@@ -109,14 +100,17 @@ class TestComponentRegistry(ZuulTestCase):
             raise
 
     def test_fingergw_component(self):
-        self.config.read_dict({
+        config = configparser.ConfigParser()
+        config.read_dict(self.config)
+        config.read_dict({
             'fingergw': {
-                'listen_address': self.host,
+                'listen_address': '::',
                 'port': '0',
+                'hostname': 'janine',
             }
         })
         gateway = FingerGateway(
-            self.config,
+            config,
             command_socket=None,
             pid_file=None
         )
@@ -124,9 +118,9 @@ class TestComponentRegistry(ZuulTestCase):
 
         try:
             self.assertComponentState("fingergw", BaseComponent.RUNNING)
+            self.assertComponentAttr("fingergw", "hostname", "janine")
         finally:
             gateway.stop()
-            gateway.wait()
 
         self.assertComponentStopped("fingergw")
 
@@ -134,7 +128,7 @@ class TestComponentRegistry(ZuulTestCase):
         self.useFixture(
             ZuulWebFixture(
                 self.changes, self.config, self.additional_event_queues,
-                self.upstream_root, self.rpcclient, self.poller_events,
+                self.upstream_root, self.poller_events,
                 self.git_url_with_auth, self.addCleanup, self.test_root
             )
         )
