@@ -11,18 +11,53 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-from configparser import ConfigParser
-
 import fixtures
 import logging
 import textwrap
 import testtools
+import voluptuous as vs
+from collections import defaultdict
+from configparser import ConfigParser
 
 from zuul import model
-from zuul.configloader import AuthorizationRuleParser, safe_load_yaml
+from zuul.lib.ansible import AnsibleManager
+from zuul.configloader import (
+    AuthorizationRuleParser, ConfigLoader, safe_load_yaml
+)
+from zuul.model import Abide, MergeRequest, SourceContext
+from zuul.zk.locks import tenant_read_lock
 
-from tests.base import ZuulTestCase
-from zuul.model import SourceContext
+from tests.base import iterate_timeout, ZuulTestCase
+
+
+class TestConfigLoader(ZuulTestCase):
+    tenant_config_file = 'config/single-tenant/main.yaml'
+
+    def test_update_system_config(self):
+        """Test if the system config can be updated without a scheduler."""
+        sched = self.scheds.first.sched
+
+        # Get the current system config before instantiating a ConfigLoader.
+        unparsed_abide, zuul_globals = sched.system_config_cache.get()
+        ansible_manager = AnsibleManager(
+            default_version=zuul_globals.default_ansible_version)
+        loader = ConfigLoader(
+            sched.connections, self.zk_client, zuul_globals, sched.statsd,
+            keystorage=sched.keystore)
+        abide = Abide()
+        loader.loadTPCs(abide, unparsed_abide)
+        loader.loadAdminRules(abide, unparsed_abide)
+
+        for tenant_name in unparsed_abide.tenants:
+            tlock = tenant_read_lock(self.zk_client, tenant_name)
+            # Consider all caches valid (min. ltime -1)
+            min_ltimes = defaultdict(lambda: defaultdict(lambda: -1))
+            with tlock:
+                tenant = loader.loadTenant(
+                    abide, tenant_name, ansible_manager, unparsed_abide,
+                    min_ltimes=min_ltimes)
+
+                self.assertEqual(tenant.name, tenant_name)
 
 
 class TenantParserTestCase(ZuulTestCase):
@@ -72,6 +107,110 @@ class TestTenantSimple(TenantParserTestCase):
         self.assertTrue('project2-job' in
                         project2_config[1].pipelines['check'].job_list.jobs)
 
+    def test_cache(self):
+        # A full reconfiguration should issue cat jobs for all repos
+        with self.assertLogs('zuul.TenantParser', level='DEBUG') as full_logs:
+            self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+            self.waitUntilSettled()
+            self.log.debug("Full reconfigure logs:")
+            for x in full_logs.output:
+                self.log.debug(x)
+            self.assertRegexInList(
+                r'Submitting cat job (.*?) for gerrit common-config master',
+                full_logs.output)
+            self.assertRegexInList(
+                r'Submitting cat job (.*?) for gerrit org/project1 master',
+                full_logs.output)
+            self.assertRegexInList(
+                r'Submitting cat job (.*?) for gerrit org/project2 master',
+                full_logs.output)
+            self.assertRegexNotInList(
+                r'Using files from cache',
+                full_logs.output)
+
+        first = self.scheds.first
+        second = self.createScheduler()
+        second.start()
+        self.assertEqual(len(self.scheds), 2)
+        for _ in iterate_timeout(10, "until priming is complete"):
+            state_one = first.sched.local_layout_state.get("tenant-one")
+            if state_one:
+                break
+
+        for _ in iterate_timeout(
+                10, "all schedulers to have the same layout state"):
+            if (second.sched.local_layout_state.get(
+                    "tenant-one") == state_one):
+                break
+
+        self.log.debug("Freeze scheduler-1")
+        # Start the log context manager for the update test below now,
+        # so that it's already in place when we release the second
+        # scheduler lock.
+        with self.assertLogs('zuul.TenantParser', level='DEBUG'
+                             ) as update_logs:
+            lock1 = second.sched.layout_update_lock
+            lock2 = second.sched.run_handler_lock
+            with lock1, lock2:
+                # A tenant reconfiguration should use the cache except for the
+                # updated project.
+                file_dict = {'zuul.d/test.yaml': ''}
+
+                # Now start a second log context manager just for the
+                # tenant reconfig test
+                with self.assertLogs('zuul.TenantParser', level='DEBUG') \
+                     as tenant_logs:
+                    A = self.fake_gerrit.addFakeChange(
+                        'org/project1', 'master', 'A',
+                        files=file_dict)
+                    A.setMerged()
+                    self.fake_gerrit.addEvent(A.getChangeMergedEvent())
+                    self.waitUntilSettled(matcher=[first])
+                    self.log.debug("Tenant reconfigure logs:")
+                    for x in tenant_logs.output:
+                        self.log.debug(x)
+
+                    self.assertRegexNotInList(
+                        r'Submitting cat job (.*?) for '
+                        r'gerrit common-config master',
+                        tenant_logs.output)
+                    self.assertRegexInList(
+                        r'Submitting cat job (.*?) for '
+                        r'gerrit org/project1 master',
+                        tenant_logs.output)
+                    self.assertRegexNotInList(
+                        r'Submitting cat job (.*?) for '
+                        r'gerrit org/project2 master',
+                        tenant_logs.output)
+                    self.assertRegexNotInList(
+                        r'Using files from cache',
+                        tenant_logs.output)
+
+            # A layout update should use the unparsed config cache
+            # except for what needs to be refreshed from the files
+            # cache in ZK.
+            self.log.debug("Thaw scheduler-1")
+            self.waitUntilSettled()
+            self.log.debug("Layout update logs:")
+            for x in update_logs.output:
+                self.log.debug(x)
+
+            self.assertRegexNotInList(
+                r'Submitting cat job',
+                update_logs.output)
+            self.assertRegexNotInList(
+                r'Using files from cache for project '
+                r'review.example.com/common-config @master.*',
+                update_logs.output)
+            self.assertRegexInList(
+                r'Using files from cache for project '
+                r'review.example.com/org/project1 @master.*',
+                update_logs.output)
+            self.assertRegexNotInList(
+                r'Using files from cache for project '
+                r'review.example.com/org/project2 @master.*',
+                update_logs.output)
+
     def test_variant_description(self):
         tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
         job = tenant.layout.jobs.get("project2-job")
@@ -101,7 +240,9 @@ class TestTenantSimple(TenantParserTestCase):
             """)
         tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
         project = tenant.config_projects[0]
-        source_context = SourceContext(project, 'master', 'zuul.yaml', True)
+        source_context = SourceContext(
+            project.canonical_name, project.name, project.connection_name,
+            'master', 'zuul.yaml', True)
 
         data = safe_load_yaml(to_parse, source_context)
         self.assertEqual(len(data), 3)
@@ -326,9 +467,7 @@ class TestTenantGroups4(TenantParserTestCase):
                          tpc.load_classes)
         # Check that only one merger:cat job was requested
         # org/project1 and org/project2 have an empty load_classes
-        cat_jobs = [job for job in self.gearman_server.jobs_history
-                    if job.name == b'merger:cat']
-        self.assertEqual(1, len(cat_jobs))
+        self.assertEqual(1, len(self.merge_job_history.get(MergeRequest.CAT)))
         old_layout = tenant.layout
 
         # Check that creating a change in project1 doesn't cause a
@@ -361,9 +500,7 @@ class TestTenantGroups5(TenantParserTestCase):
                          tpc.load_classes)
         # Check that only one merger:cat job was requested
         # org/project1 and org/project2 have an empty load_classes
-        cat_jobs = [job for job in self.gearman_server.jobs_history
-                    if job.name == b'merger:cat']
-        self.assertEqual(1, len(cat_jobs))
+        self.assertEqual(1, len(self.merge_job_history.get(MergeRequest.CAT)))
 
 
 class TestTenantFromScript(TestTenantSimple):
@@ -479,12 +616,12 @@ class TestSplitConfig(ZuulTestCase):
         self.assertIn('project-test1', tenant.layout.jobs)
         self.assertIn('project-test2', tenant.layout.jobs)
         test1 = tenant.layout.getJob('project-test1')
-        self.assertEqual(test1.source_context.project.name, 'common-config')
+        self.assertEqual(test1.source_context.project_name, 'common-config')
         self.assertEqual(test1.source_context.branch, 'master')
         self.assertEqual(test1.source_context.path, 'zuul.d/jobs.yaml')
         self.assertEqual(test1.source_context.trusted, True)
         test2 = tenant.layout.getJob('project-test2')
-        self.assertEqual(test2.source_context.project.name, 'common-config')
+        self.assertEqual(test2.source_context.project_name, 'common-config')
         self.assertEqual(test2.source_context.branch, 'master')
         self.assertEqual(test2.source_context.path, 'zuul.d/more-jobs.yaml')
         self.assertEqual(test2.source_context.trusted, True)
@@ -628,7 +765,7 @@ class TestUnparsedConfigCache(ZuulTestCase):
         # Clear the unparsed branch cache so all projects (except for
         # org/project2) are retrieved from the cache in Zookeeper.
         sched.abide.unparsed_project_branch_cache.clear()
-        self.gearman_server.jobs_history.clear()
+        del self.merge_job_history
 
         # Create a tenant reconfiguration event with a known ltime that is
         # smaller than the ltime of the items in the cache.
@@ -640,16 +777,14 @@ class TestUnparsedConfigCache(ZuulTestCase):
 
         # As the cache should be valid (cache ltime of org/project2 newer than
         # event ltime) we don't expect any cat jobs.
-        cat_jobs = [job for job in self.gearman_server.jobs_history
-                    if job.name == b"merger:cat"]
-        self.assertEqual(len(cat_jobs), 0)
+        self.assertIsNone(self.merge_job_history.get(MergeRequest.CAT))
 
         # Set canary value so we can detect if the configloader used
         # the cache in Zookeeper (it shouldn't).
         common_cache = cache.getFilesCache("review.example.com/common-config",
                                            "master")
         common_cache.setValidFor({"CANARY"}, set(), common_cache.ltime)
-        self.gearman_server.jobs_history.clear()
+        del self.merge_job_history
 
         # Create a tenant reconfiguration event with a known ltime that is
         # smaller than the ltime of the items in the cache.
@@ -666,9 +801,7 @@ class TestUnparsedConfigCache(ZuulTestCase):
 
         # As the cache should be valid (cache ltime of org/project2 newer than
         # event ltime) we don't expect any cat jobs.
-        cat_jobs = [job for job in self.gearman_server.jobs_history
-                    if job.name == b"merger:cat"]
-        self.assertEqual(len(cat_jobs), 0)
+        self.assertIsNone(self.merge_job_history.get(MergeRequest.CAT))
         sched.apsched.start()
 
 
@@ -862,6 +995,7 @@ class TestTenantExtra(TenantParserTestCase):
         tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
         self.assertTrue('project2-extra-file' in tenant.layout.jobs)
         self.assertTrue('project2-extra-dir' in tenant.layout.jobs)
+        self.assertTrue('project6-extra-dir' in tenant.layout.jobs)
 
     def test_dynamic_extra(self):
         in_repo_conf = textwrap.dedent(
@@ -885,6 +1019,30 @@ class TestTenantExtra(TenantParserTestCase):
             dict(name='project2-extra-file2', result='SUCCESS', changes='1,1'),
         ], ordered=False)
 
+    def test_dynamic_extra_dir(self):
+        in_repo_conf = textwrap.dedent(
+            """
+            - job:
+                name: project6-extra-dir2
+                parent: common-config-job
+            - project:
+                check:
+                  jobs:
+                    - project6-extra-dir
+                    - project6-extra-dir2
+            """)
+        file_dict = {
+            'other/extra.d/new/extra.yaml': in_repo_conf,
+        }
+        A = self.fake_gerrit.addFakeChange('org/project6', 'master', 'A',
+                                           files=file_dict)
+        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+        self.assertHistory([
+            dict(name='project6-extra-dir', result='SUCCESS', changes='1,1'),
+            dict(name='project6-extra-dir2', result='SUCCESS', changes='1,1'),
+        ], ordered=False)
+
     def test_extra_reconfigure(self):
         in_repo_conf = textwrap.dedent(
             """
@@ -916,9 +1074,59 @@ class TestTenantExtra(TenantParserTestCase):
         ], ordered=False)
 
 
+class TestTenantExtraConfigsInvalidType(TenantParserTestCase):
+    tenant_config_file = 'config/tenant-parser/extra_invalid_type.yaml'
+
+    # This test raises a config error during the startup of the test
+    # case which makes the first scheduler fail during its startup.
+    # The second (or any additional) scheduler won't even run as the
+    # startup is serialized in tests/base.py.
+    # Thus it doesn't make sense to execute this test with multiple
+    # schedulers.
+    scheduler_count = 1
+
+    def setUp(self):
+        err = "Expected str or list of str for extra-config-paths.*"
+        with testtools.ExpectedException(vs.MultipleInvalid, err):
+            super().setUp()
+
+    def test_tenant_extra_configs_invalid_type(self):
+        # The magic is in setUp
+        pass
+
+
+class TestTenantExtraConfigsInvalidValue(TenantParserTestCase):
+    tenant_config_file = 'config/tenant-parser/extra_invalid_value.yaml'
+
+    # This test raises a config error during the startup of the test
+    # case which makes the first scheduler fail during its startup.
+    # The second (or any additional) scheduler won't even run as the
+    # startup is serialized in tests/base.py.
+    # Thus it doesn't make sense to execute this test with multiple
+    # schedulers.
+    scheduler_count = 1
+
+    def setUp(self):
+        err = "Default zuul configs are not allowed in extra-config-paths.*"
+        with testtools.ExpectedException(vs.MultipleInvalid, err):
+            super().setUp()
+
+    def test_tenant_extra_configs_invalid_value(self):
+        # The magic is in setUp
+        pass
+
+
 class TestTenantDuplicate(TenantParserTestCase):
     tenant_config_file = 'config/tenant-parser/duplicate.yaml'
 
+    # This test raises a config error during the startup of the test
+    # case which makes the first scheduler fail during its startup.
+    # The second (or any additional) scheduler won't even run as the
+    # startup is serialized in tests/base.py.
+    # Thus it doesn't make sense to execute this test with multiple
+    # schedulers.
+    scheduler_count = 1
+
     def setUp(self):
         with testtools.ExpectedException(Exception, 'Duplicate configuration'):
             super().setUp()
