@@ -27,15 +27,6 @@ from tests.base import ZuulTestCase, FIXTURE_DIR, \
     BaseTestCase, AnsibleZuulTestCase
 
 
-def _get_reporter_from_connection_name(reporters, connection_name):
-    # Reporters are placed into lists for each action they may exist in.
-    # Search through the given list for the correct reporter by its connection
-    # name
-    for r in reporters:
-        if r.connection.connection_name == connection_name:
-            return r
-
-
 class TestConnections(ZuulTestCase):
     config_file = 'zuul-connections-same-gerrit.conf'
     tenant_config_file = 'config/zuul-connections-same-gerrit/main.yaml'
@@ -82,7 +73,7 @@ class TestSQLConnectionMysql(ZuulTestCase):
         buildset_table = table_prefix + 'zuul_buildset'
         build_table = table_prefix + 'zuul_build'
 
-        self.assertEqual(16, len(insp.get_columns(buildset_table)))
+        self.assertEqual(19, len(insp.get_columns(buildset_table)))
         self.assertEqual(13, len(insp.get_columns(build_table)))
 
     def test_sql_tables_created(self):
@@ -132,73 +123,103 @@ class TestSQLConnectionMysql(ZuulTestCase):
             pipeline = tenant.layout.pipelines['check']
             reporter = self.scheds.first.connections.getSqlReporter(
                 pipeline)
-            conn = self.scheds.first.connections.getSqlConnection().\
-                engine.connect()
+            with self.scheds.first.connections.getSqlConnection().\
+                    engine.connect() as conn:
 
-            result = conn.execute(
-                sa.sql.select([reporter.connection.zuul_buildset_table]))
+                result = conn.execute(
+                    sa.sql.select([reporter.connection.zuul_buildset_table]))
 
-            buildsets = result.fetchall()
-            self.assertEqual(3, len(buildsets))
-            buildset0 = buildsets[0]
-            buildset1 = buildsets[1]
-            buildset2 = buildsets[2]
+                buildsets = result.fetchall()
+                self.assertEqual(4, len(buildsets))
+                buildset0 = buildsets[0]
+                buildset1 = buildsets[1]
+                buildset2 = buildsets[2]
+                buildset3 = buildsets[3]
 
-            self.assertEqual('check', buildset0['pipeline'])
-            self.assertEqual('org/project', buildset0['project'])
-            self.assertEqual(1, buildset0['change'])
-            self.assertEqual('1', buildset0['patchset'])
-            self.assertEqual('SUCCESS', buildset0['result'])
-            self.assertEqual('Build succeeded.', buildset0['message'])
-            self.assertEqual('tenant-one', buildset0['tenant'])
-            self.assertEqual(
-                'https://review.example.com/%d' % buildset0['change'],
-                buildset0['ref_url'])
-            self.assertNotEqual(None, buildset0['event_id'])
+                self.assertEqual('check', buildset0['pipeline'])
+                self.assertEqual('org/project', buildset0['project'])
+                self.assertEqual(1, buildset0['change'])
+                self.assertEqual('1', buildset0['patchset'])
+                self.assertEqual('SUCCESS', buildset0['result'])
+                self.assertEqual('Build succeeded.', buildset0['message'])
+                self.assertEqual('tenant-one', buildset0['tenant'])
+                self.assertEqual(
+                    'https://review.example.com/%d' % buildset0['change'],
+                    buildset0['ref_url'])
+                self.assertNotEqual(None, buildset0['event_id'])
+                self.assertNotEqual(None, buildset0['event_timestamp'])
 
-            buildset0_builds = conn.execute(
-                sa.sql.select([reporter.connection.zuul_build_table]).where(
-                    reporter.connection.zuul_build_table.c.buildset_id ==
-                    buildset0['id']
-                )
-            ).fetchall()
+                buildset0_builds = conn.execute(
+                    sa.sql.select([
+                        reporter.connection.zuul_build_table
+                    ]).where(
+                        reporter.connection.zuul_build_table.c.buildset_id ==
+                        buildset0['id']
+                    )
+                ).fetchall()
 
-            # Check the first result, which should be the project-merge job
-            self.assertEqual('project-merge', buildset0_builds[0]['job_name'])
-            self.assertEqual("SUCCESS", buildset0_builds[0]['result'])
-            self.assertEqual(None, buildset0_builds[0]['log_url'])
-            self.assertEqual('check', buildset1['pipeline'])
-            self.assertEqual('master', buildset1['branch'])
-            self.assertEqual('org/project', buildset1['project'])
-            self.assertEqual(2, buildset1['change'])
-            self.assertEqual('1', buildset1['patchset'])
-            self.assertEqual('FAILURE', buildset1['result'])
-            self.assertEqual('Build failed.', buildset1['message'])
+                # Check the first result, which should be the project-merge job
+                self.assertEqual(
+                    'project-merge', buildset0_builds[0]['job_name'])
+                self.assertEqual("SUCCESS", buildset0_builds[0]['result'])
+                self.assertEqual(None, buildset0_builds[0]['log_url'])
+                self.assertEqual('check', buildset1['pipeline'])
+                self.assertEqual('master', buildset1['branch'])
+                self.assertEqual('org/project', buildset1['project'])
+                self.assertEqual(2, buildset1['change'])
+                self.assertEqual('1', buildset1['patchset'])
+                self.assertEqual('FAILURE', buildset1['result'])
+                self.assertEqual('Build failed.', buildset1['message'])
 
-            buildset1_builds = conn.execute(
-                sa.sql.select([reporter.connection.zuul_build_table]).where(
-                    reporter.connection.zuul_build_table.c.buildset_id ==
-                    buildset1['id']
-                )
-            ).fetchall()
+                buildset1_builds = conn.execute(
+                    sa.sql.select([
+                        reporter.connection.zuul_build_table
+                    ]).where(
+                        reporter.connection.zuul_build_table.c.buildset_id ==
+                        buildset1['id']
+                    )
+                ).fetchall()
 
-            # Check the second result, which should be the project-test1 job
-            # which failed
-            self.assertEqual('project-test1', buildset1_builds[1]['job_name'])
-            self.assertEqual("FAILURE", buildset1_builds[1]['result'])
-            self.assertEqual(None, buildset1_builds[1]['log_url'])
+                # Check the second result, which should be the project-test1
+                # job which failed
+                self.assertEqual(
+                    'project-test1', buildset1_builds[1]['job_name'])
+                self.assertEqual("FAILURE", buildset1_builds[1]['result'])
+                self.assertEqual(None, buildset1_builds[1]['log_url'])
 
-            buildset2_builds = conn.execute(
-                sa.sql.select([reporter.connection.zuul_build_table]).where(
-                    reporter.connection.zuul_build_table.c.buildset_id ==
-                    buildset2['id']
-                )
-            ).fetchall()
+                buildset2_builds = conn.execute(
+                    sa.sql.select([
+                        reporter.connection.zuul_build_table
+                    ]).where(
+                        reporter.connection.zuul_build_table.c.buildset_id ==
+                        buildset2['id']
+                    )
+                ).fetchall()
 
-            # Check the first result, which should be the project-publish job
-            self.assertEqual('project-publish',
-                             buildset2_builds[0]['job_name'])
-            self.assertEqual("SUCCESS", buildset2_builds[0]['result'])
+                # Check the first result, which should be the project-publish
+                # job
+                self.assertEqual('project-publish',
+                                 buildset2_builds[0]['job_name'])
+                self.assertEqual("SUCCESS", buildset2_builds[0]['result'])
+
+                buildset3_builds = conn.execute(
+                    sa.sql.select([
+                        reporter.connection.zuul_build_table
+                    ]).where(
+                        reporter.connection.zuul_build_table.c.buildset_id ==
+                        buildset3['id']
+                    )
+                ).fetchall()
+
+                self.assertEqual(
+                    'project-test1', buildset3_builds[1]['job_name'])
+                self.assertEqual('NODE_FAILURE', buildset3_builds[1]['result'])
+                self.assertEqual(None, buildset3_builds[1]['log_url'])
+                self.assertIsNotNone(buildset3_builds[1]['start_time'])
+                self.assertIsNotNone(buildset3_builds[1]['end_time'])
+                self.assertGreaterEqual(
+                    buildset3_builds[1]['end_time'],
+                    buildset3_builds[1]['start_time'])
 
         self.executor_server.hold_jobs_in_build = True
 
@@ -228,6 +249,21 @@ class TestSQLConnectionMysql(ZuulTestCase):
         self.orderedRelease()
         self.waitUntilSettled()
 
+        # Add a node_failure result
+        self.fake_nodepool.pause()
+        C = self.fake_gerrit.addFakeChange('org/project', 'master', 'C')
+        C.addApproval('Code-Review', 2)
+        self.fake_gerrit.addEvent(C.addApproval('Approved', 1))
+        self.waitUntilSettled()
+        self.orderedRelease()
+        self.waitUntilSettled()
+        req = self.fake_nodepool.getNodeRequests()[0]
+        self.fake_nodepool.addFailRequest(req)
+        self.fake_nodepool.unpause()
+        self.waitUntilSettled()
+        self.orderedRelease()
+        self.waitUntilSettled()
+
         check_results()
 
     def test_sql_results_retry_builds(self):
@@ -309,64 +345,6 @@ class TestSQLConnectionMysql(ZuulTestCase):
 
         check_results()
 
-    def test_multiple_sql_connections(self):
-        "Test putting results in different databases"
-        # Add a successful result
-        A = self.fake_gerrit.addFakeChange('org/project', 'master', 'A')
-        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
-        self.waitUntilSettled()
-
-        # Add a failed result
-        B = self.fake_gerrit.addFakeChange('org/project', 'master', 'B')
-        self.executor_server.failJob('project-test1', B)
-        self.fake_gerrit.addEvent(B.getPatchsetCreatedEvent(1))
-        self.waitUntilSettled()
-
-        def check_results(connection_name_1, connection_name_2):
-            # Grab the sa tables for resultsdb
-            tenant = self.scheds.first.sched.abide.tenants.get("tenant-one")
-            pipeline = tenant.layout.pipelines['check']
-            reporter1 = self.scheds.first.connections.getSqlReporter(
-                pipeline)
-
-            conn = self.scheds.first.connections.getSqlConnection().\
-                engine.connect()
-            buildsets_resultsdb = conn.execute(sa.sql.select(
-                [reporter1.connection.zuul_buildset_table])).fetchall()
-            # Should have been 2 buildset reported to the resultsdb (both
-            # success and failure report)
-            self.assertEqual(2, len(buildsets_resultsdb))
-
-            # The first one should have passed
-            self.assertEqual('check', buildsets_resultsdb[0]['pipeline'])
-            self.assertEqual(
-                'org/project', buildsets_resultsdb[0]['project'])
-            self.assertEqual(1, buildsets_resultsdb[0]['change'])
-            self.assertEqual('1', buildsets_resultsdb[0]['patchset'])
-            self.assertEqual('SUCCESS', buildsets_resultsdb[0]['result'])
-            self.assertEqual(
-                'Build succeeded.', buildsets_resultsdb[0]['message'])
-
-            buildsets_resultsdb_failures = conn.execute(sa.sql.select(
-                [reporter1.connection.zuul_buildset_table])).fetchall()
-            # The failure db should only have 1 buildset failed
-            self.assertEqual(2, len(buildsets_resultsdb_failures))
-
-            self.assertEqual(
-                'check', buildsets_resultsdb_failures[1]['pipeline'])
-            self.assertEqual('org/project',
-                             buildsets_resultsdb_failures[1]['project'])
-            self.assertEqual(2,
-                             buildsets_resultsdb_failures[1]['change'])
-            self.assertEqual(
-                '1', buildsets_resultsdb_failures[1]['patchset'])
-            self.assertEqual(
-                'FAILURE', buildsets_resultsdb_failures[1]['result'])
-            self.assertEqual('Build failed.',
-                             buildsets_resultsdb_failures[1]['message'])
-
-        check_results('database', 'resultsdb_failures')
-
 
 class TestSQLConnectionPostgres(TestSQLConnectionMysql):
     config_file = 'zuul-sql-driver-postgres.conf'
@@ -675,6 +653,12 @@ class TestElasticsearchConnection(AnsibleZuulTestCase):
     config_file = 'zuul-elastic-driver.conf'
     tenant_config_file = 'config/elasticsearch-driver/main.yaml'
 
+    # These tests are storing the reported index on the fake
+    # elasticsearch backend which is a different instance for each
+    # scheduler. Thus, depending on which scheduler reports the
+    # item, the assertions in these test might pass or fail.
+    scheduler_count = 1
+
     def _getSecrets(self, job, pbtype):
         secrets = []
         build = self.getJobFromHistory(job)
