@@ -18,8 +18,12 @@ from unittest import mock
 
 import tests.base
 from tests.base import (
-    BaseTestCase, ZuulTestCase, AnsibleZuulTestCase,
-    simple_layout)
+    AnsibleZuulTestCase,
+    BaseTestCase,
+    simple_layout,
+    skipIfMultiScheduler,
+    ZuulTestCase,
+)
 from zuul.driver.gerrit import GerritDriver
 from zuul.driver.gerrit.gerritconnection import GerritConnection
 
@@ -123,7 +127,25 @@ class TestGerrit(BaseTestCase):
         project = gerrit.source.getProject('org/project')
         url = gerrit.source.getGitUrl(project)
         self.assertEqual(
-            'https://gerrit:1%2Fbadpassword@localhost/org/project',
+            'https://gerrit:1%2Fbadpassword@localhost/a/org/project',
+            url)
+
+    def test_git_over_ssh_getGitURL(self):
+        gerrit_config = {
+            'user': 'gerrit',
+            'server': 'localhost',
+            'password': '1/badpassword',
+            'git_over_ssh': 'true',
+        }
+        # The 1/ in the password ensures we test the url encoding
+        # path; this is the format of password we get from
+        # googlesource.com.
+        driver = GerritDriver()
+        gerrit = GerritConnection(driver, 'review_gerrit', gerrit_config)
+        project = gerrit.source.getProject('org/project')
+        url = gerrit.source.getGitUrl(project)
+        self.assertEqual(
+            'ssh://gerrit@localhost:29418/org/project',
             url)
 
 
@@ -184,6 +206,28 @@ class TestGerritWeb(ZuulTestCase):
                                        'username': 'jenkins'}}
         )
 
+    def test_message_too_long(self):
+        in_repo_conf = textwrap.dedent(
+            """
+            - job:
+                name: garbage-job
+                %s
+                garbage: True
+            """
+        ) % ('\n' * 16384)
+
+        file_dict = {'.zuul.yaml': in_repo_conf}
+        A = self.fake_gerrit.addFakeChange('org/project', 'master', 'A',
+                                           files=file_dict)
+        self.fake_gerrit.addEvent(A.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+
+        self.assertEqual(A.patchsets[0]['approvals'][0]['value'], "-1")
+        self.assertEqual(A.patchsets[0]['approvals'][0]['__tag'],
+                         "autogenerated:zuul:check")
+        self.assertIn('... (truncated)',
+                      A.messages[0])
+
     def test_dependent_dynamic_line_comment(self):
         in_repo_conf = textwrap.dedent(
             """
@@ -253,6 +297,48 @@ class TestGerritWeb(ZuulTestCase):
         self.waitUntilSettled()
         self.assertEqual(9, len(self.history))
 
+    def test_submitted_together_git(self):
+        # This tests that the circular dependency handling for submit
+        # whole topic doesn't activate for changes which are only in a
+        # git dependency.
+        A = self.fake_gerrit.addFakeChange('org/project1', "master", "A")
+        B = self.fake_gerrit.addFakeChange('org/project1', "master", "B")
+        C = self.fake_gerrit.addFakeChange('org/project1', "master", "C")
+        D = self.fake_gerrit.addFakeChange('org/project1', "master", "D")
+        E = self.fake_gerrit.addFakeChange('org/project1', "master", "E")
+        F = self.fake_gerrit.addFakeChange('org/project1', "master", "F")
+        G = self.fake_gerrit.addFakeChange('org/project1', "master", "G")
+        G.setDependsOn(F, 1)
+        F.setDependsOn(E, 1)
+        E.setDependsOn(D, 1)
+        D.setDependsOn(C, 1)
+        C.setDependsOn(B, 1)
+        B.setDependsOn(A, 1)
+
+        self.fake_gerrit.addEvent(C.getPatchsetCreatedEvent(1))
+        self.waitUntilSettled()
+
+        self.assertEqual(len(C.patchsets[-1]["approvals"]), 1)
+        self.assertEqual(C.patchsets[-1]["approvals"][0]["type"], "Verified")
+        self.assertEqual(C.patchsets[-1]["approvals"][0]["value"], "1")
+        self.assertEqual(A.queried, 1)
+        self.assertEqual(B.queried, 1)
+        self.assertEqual(C.queried, 1)
+        self.assertEqual(D.queried, 1)
+        self.assertEqual(E.queried, 1)
+        self.assertEqual(F.queried, 1)
+        self.assertEqual(G.queried, 1)
+        self.assertHistory([
+            dict(name="project-merge", result="SUCCESS",
+                 changes="1,1 2,1 3,1"),
+            dict(name="project-test1", result="SUCCESS",
+                 changes="1,1 2,1 3,1"),
+            dict(name="project-test2", result="SUCCESS",
+                 changes="1,1 2,1 3,1"),
+            dict(name="project1-project2-integration", result="SUCCESS",
+                 changes="1,1 2,1 3,1"),
+        ], ordered=False)
+
 
 class TestFileComments(AnsibleZuulTestCase):
     config_file = 'zuul-gerrit-web.conf'
@@ -440,6 +526,17 @@ class TestChecksApi(ZuulTestCase):
                          "start and success messages should be reported")
 
     @simple_layout('layouts/gerrit-checks-scheme.yaml')
+    @skipIfMultiScheduler()
+    # This is the only gerrit checks API test which is failing because
+    # it uses a check scheme rather than an UUID. The scheme must first
+    # be evaluated and mapped to an UUID.
+    # This shouldn't be a problem in production as the evaluation takes
+    # place on the gerrit webserver. However, in the tests we get a
+    # dedicated (fake) gerrit webserver for each fake gerrrit
+    # connection. Since each scheduler gets a new connection, only one
+    # of those webservers will be aware of the check. If any other
+    # webserver tries to evaluate the check it will fail with
+    # "Unable to find matching checker".
     def test_check_pipeline_scheme(self):
         self.fake_gerrit.addFakeChecker(uuid='zuul_check:abcd',
                                         repository='org/project',
