@@ -15,6 +15,7 @@
 from tests.base import (
     ZuulTestCase,
     simple_layout,
+    skipIfMultiScheduler,
 )
 
 
@@ -61,7 +62,7 @@ class TestGerritLegacyCRD(ZuulTestCase):
         self.assertEqual(B.data['status'], 'NEW')
 
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         self.executor_server.hold_jobs_in_build = True
         B.addApproval('Approved', 1)
@@ -193,7 +194,7 @@ class TestGerritLegacyCRD(ZuulTestCase):
 
         self.assertEqual(A.data['status'], 'NEW')
         self.assertEqual(B.data['status'], 'NEW')
-        self.assertEqual(A.reported, 0)
+        self.assertEqual(A.reported, 1)
         self.assertEqual(B.reported, 0)
         self.assertEqual(len(self.history), 0)
 
@@ -210,7 +211,7 @@ class TestGerritLegacyCRD(ZuulTestCase):
         self.waitUntilSettled()
 
         self.assertEqual(A.data['status'], 'MERGED')
-        self.assertEqual(A.reported, 2)
+        self.assertEqual(A.reported, 3)
 
     def test_crd_gate_reverse(self):
         "Test reverse cross-repo dependencies"
@@ -297,7 +298,7 @@ class TestGerritLegacyCRD(ZuulTestCase):
         # should not be processed in dependent pipeline
         self.assertEqual(A.data['status'], 'NEW')
         self.assertEqual(B.data['status'], 'NEW')
-        self.assertEqual(A.reported, 0)
+        self.assertEqual(A.reported, 1)
         self.assertEqual(B.reported, 0)
         self.assertEqual(len(self.history), 0)
 
@@ -316,7 +317,7 @@ class TestGerritLegacyCRD(ZuulTestCase):
         self.waitUntilSettled()
 
         self.assertEqual(A.data['status'], 'MERGED')
-        self.assertEqual(A.reported, 2)
+        self.assertEqual(A.reported, 3)
         self.assertEqual(B.data['status'], 'MERGED')
         self.assertEqual(B.reported, 0)
 
@@ -359,7 +360,6 @@ class TestGerritLegacyCRD(ZuulTestCase):
 
     def test_crd_check_git_depends(self):
         "Test single-repo dependencies in independent pipelines"
-        self.gearman_server.hold_jobs_in_build = True
         A = self.fake_gerrit.addFakeChange('org/project1', 'master', 'A')
         B = self.fake_gerrit.addFakeChange('org/project1', 'master', 'B')
 
@@ -371,10 +371,6 @@ class TestGerritLegacyCRD(ZuulTestCase):
         self.fake_gerrit.addEvent(B.getPatchsetCreatedEvent(1))
         self.waitUntilSettled()
 
-        self.orderedRelease()
-        self.gearman_server.hold_jobs_in_build = False
-        self.waitUntilSettled()
-
         self.assertEqual(A.data['status'], 'NEW')
         self.assertEqual(B.data['status'], 'NEW')
         self.assertEqual(A.reported, 1)
@@ -446,6 +442,7 @@ class TestGerritLegacyCRD(ZuulTestCase):
         self.waitUntilSettled()
 
         self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+        self.waitUntilSettled()
 
         # Make sure the items still share a change queue, and the
         # first one is not live.
@@ -470,9 +467,11 @@ class TestGerritLegacyCRD(ZuulTestCase):
         self.assertEqual(self.history[0].changes, '2,1 1,1')
         self.assertEqual(len(tenant.layout.pipelines['check'].queues), 0)
 
+    @skipIfMultiScheduler()
     def test_crd_check_reconfiguration(self):
         self._test_crd_check_reconfiguration('org/project1', 'org/project2')
 
+    @skipIfMultiScheduler()
     def test_crd_undefined_project(self):
         """Test that undefined projects in dependencies are handled for
         independent pipelines"""
