@@ -1,4 +1,5 @@
 # Copyright 2019 Red Hat
+# Copyright 2022 Acme Gating, LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,17 +18,17 @@ import os
 import git
 import yaml
 import socket
+import time
 
-import zuul.rpcclient
 from zuul.lib import strings
 from zuul.zk.layout import LayoutState
 
-from tests.base import random_sha1, simple_layout
+from tests.base import random_sha1, simple_layout, skipIfMultiScheduler
 from tests.base import ZuulTestCase, ZuulWebFixture
 
 from testtools.matchers import MatchesRegex
 
-EMPTY_LAYOUT_STATE = LayoutState("", "", 0)
+EMPTY_LAYOUT_STATE = LayoutState("", "", 0, None, {})
 
 
 class TestGitlabWebhook(ZuulTestCase):
@@ -40,7 +41,7 @@ class TestGitlabWebhook(ZuulTestCase):
         self.web = self.useFixture(
             ZuulWebFixture(self.changes, self.config,
                            self.additional_event_queues, self.upstream_root,
-                           self.rpcclient, self.poller_events,
+                           self.poller_events,
                            self.git_url_with_auth, self.addCleanup,
                            self.test_root))
 
@@ -108,7 +109,8 @@ class TestGitlabDriver(ZuulTestCase):
         self.assertEqual(str(A.number), zuulvars['change'])
         self.assertEqual(str(A.sha), zuulvars['patchset'])
         self.assertEqual('master', zuulvars['branch'])
-        self.assertEquals('https://gitlab/org/project/merge_requests/1',
+        self.assertEquals(f'{self.fake_gitlab._test_baseurl}/'
+                          'org/project/merge_requests/1',
                           zuulvars['items'][0]['change_url'])
         self.assertEqual(zuulvars["message"], strings.b64encode(description))
         self.assertEqual(2, len(self.history))
@@ -195,15 +197,21 @@ class TestGitlabDriver(ZuulTestCase):
         A = self.fake_gitlab.openFakeMergeRequest('org/project', 'master', 'A')
 
         self.fake_gitlab.emitEvent(A.getMergeRequestLabeledEvent(
-            labels=('label1', 'label2')))
+            add_labels=('label1', 'label2')))
         self.waitUntilSettled()
         self.assertEqual(0, len(self.history))
 
         self.fake_gitlab.emitEvent(A.getMergeRequestLabeledEvent(
-            labels=('gateit', )))
+            add_labels=('gateit', )))
         self.waitUntilSettled()
         self.assertEqual(1, len(self.history))
 
+        A.labels = ['verified']
+        self.fake_gitlab.emitEvent(A.getMergeRequestLabeledEvent(
+            remove_labels=('verified', )))
+        self.waitUntilSettled()
+        self.assertEqual(2, len(self.history))
+
     @simple_layout('layouts/basic-gitlab.yaml', driver='gitlab')
     def test_merge_request_merged(self):
 
@@ -214,6 +222,43 @@ class TestGitlabDriver(ZuulTestCase):
         self.assertEqual(1, len(self.history))
         self.assertHistory([{'name': 'project-promote'}])
 
+    @simple_layout('layouts/basic-gitlab.yaml', driver='gitlab')
+    def test_merge_push_does_not_reconfigure(self):
+        # Test that the push event that follows a merge doesn't
+        # needlessly trigger reconfiguration.
+
+        A = self.fake_gitlab.openFakeMergeRequest('org/project', 'master', 'A')
+
+        state1 = self.scheds.first.sched.local_layout_state.get("tenant-one")
+        self.fake_gitlab.emitEvent(A.getMergeRequestMergedEvent())
+        self.fake_gitlab.emitEvent(A.getMergeRequestMergedPushEvent())
+        self.waitUntilSettled()
+        self.assertEqual(2, len(self.history))
+        self.assertHistory([{'name': 'project-post-job'},
+                            {'name': 'project-promote'}
+                            ], ordered=False)
+        state2 = self.scheds.first.sched.local_layout_state.get("tenant-one")
+        self.assertEqual(state1, state2)
+
+    @simple_layout('layouts/basic-gitlab.yaml', driver='gitlab')
+    def test_merge_push_does_reconfigure(self):
+        # Test that the push event that follows a merge does
+        # trigger reconfiguration if .zuul.yaml is changed.
+
+        A = self.fake_gitlab.openFakeMergeRequest('org/project', 'master', 'A')
+
+        state1 = self.scheds.first.sched.local_layout_state.get("tenant-one")
+        self.fake_gitlab.emitEvent(A.getMergeRequestMergedEvent())
+        self.fake_gitlab.emitEvent(A.getMergeRequestMergedPushEvent(
+            modified_files=['.zuul.yaml']))
+        self.waitUntilSettled()
+        self.assertEqual(2, len(self.history))
+        self.assertHistory([{'name': 'project-post-job'},
+                            {'name': 'project-promote'}
+                            ], ordered=False)
+        state2 = self.scheds.first.sched.local_layout_state.get("tenant-one")
+        self.assertNotEqual(state1, state2)
+
     @simple_layout('layouts/basic-gitlab.yaml', driver='gitlab')
     def test_merge_request_updated_builds_aborted(self):
 
@@ -263,6 +308,17 @@ class TestGitlabDriver(ZuulTestCase):
         self.assertEqual(4, len(self.history))
 
     @simple_layout('layouts/basic-gitlab.yaml', driver='gitlab')
+    @skipIfMultiScheduler()
+    # This test fails reproducibly with multiple schedulers because
+    # the fake_gitlab._test_baseurl used in the assertion for the
+    # change_url doesn't match.
+    # An explanation for this would be that each scheduler (and the
+    # test case itself) use different (fake) Gitlab connections.
+    # However, the interesting part is that only this test fails
+    # although there are other gitlab tests with a similar assertion.
+    # Apart from that I'm wondering why this test first fails with
+    # multiple schedulers as each scheduler should have a different
+    # gitlab connection than the test case itself.
     def test_ref_updated(self):
 
         event = self.fake_gitlab.getPushEvent('org/project')
@@ -282,7 +338,8 @@ class TestGitlabDriver(ZuulTestCase):
         self.assertEqual('project-post-job', zuulvars['job'])
         self.assertEqual('master', zuulvars['branch'])
         self.assertEqual(
-            'https://gitlab/org/project/tree/%s' % zuulvars['newrev'],
+            f'{self.fake_gitlab._test_baseurl}/org/project/tree/'
+            f'{zuulvars["newrev"]}',
             zuulvars['change_url'])
         self.assertEqual(expected_newrev, zuulvars['newrev'])
         self.assertEqual(expected_oldrev, zuulvars['oldrev'])
@@ -409,7 +466,8 @@ class TestGitlabDriver(ZuulTestCase):
              'job.yaml': playbook},
             message='Add InRepo configuration'
         )
-        event = self.fake_gitlab.getPushEvent('org/project')
+        event = self.fake_gitlab.getPushEvent('org/project',
+                                              modified_files=['.zuul.yaml'])
         self.fake_gitlab.emitEvent(event)
         self.waitUntilSettled()
 
@@ -424,80 +482,6 @@ class TestGitlabDriver(ZuulTestCase):
             ], ordered=False
         )
 
-    @simple_layout('layouts/basic-gitlab.yaml', driver='gitlab')
-    def test_client_dequeue_change(self):
-
-        client = zuul.rpcclient.RPCClient('127.0.0.1',
-                                          self.gearman_server.port)
-        self.addCleanup(client.shutdown)
-
-        self.executor_server.hold_jobs_in_build = True
-        A = self.fake_gitlab.openFakeMergeRequest('org/project', 'master', 'A')
-
-        self.fake_gitlab.emitEvent(A.getMergeRequestOpenedEvent())
-        self.waitUntilSettled()
-
-        client.dequeue(
-            tenant='tenant-one',
-            pipeline='check',
-            project='org/project',
-            change='%s,%s' % (A.number, A.sha),
-            ref=None)
-
-        self.waitUntilSettled()
-
-        tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
-        check_pipeline = tenant.layout.pipelines['check']
-        self.assertEqual(check_pipeline.getAllItems(), [])
-        self.assertEqual(self.countJobResults(self.history, 'ABORTED'), 2)
-
-        self.executor_server.hold_jobs_in_build = False
-        self.executor_server.release()
-        self.waitUntilSettled()
-
-    @simple_layout('layouts/basic-gitlab.yaml', driver='gitlab')
-    def test_client_enqueue_change(self):
-
-        A = self.fake_gitlab.openFakeMergeRequest('org/project', 'master', 'A')
-
-        client = zuul.rpcclient.RPCClient('127.0.0.1',
-                                          self.gearman_server.port)
-        self.addCleanup(client.shutdown)
-        r = client.enqueue(tenant='tenant-one',
-                           pipeline='check',
-                           project='org/project',
-                           trigger='gitlab',
-                           change='%s,%s' % (A.number, A.sha))
-        self.waitUntilSettled()
-
-        self.assertEqual(self.getJobFromHistory('project-test1').result,
-                         'SUCCESS')
-        self.assertEqual(self.getJobFromHistory('project-test2').result,
-                         'SUCCESS')
-        self.assertEqual(r, True)
-
-    @simple_layout('layouts/basic-gitlab.yaml', driver='gitlab')
-    def test_client_enqueue_ref(self):
-        repo_path = os.path.join(self.upstream_root, 'org/project')
-        repo = git.Repo(repo_path)
-        headsha = repo.head.commit.hexsha
-
-        client = zuul.rpcclient.RPCClient('127.0.0.1',
-                                          self.gearman_server.port)
-        self.addCleanup(client.shutdown)
-        r = client.enqueue_ref(
-            tenant='tenant-one',
-            pipeline='post',
-            project='org/project',
-            trigger='gitlab',
-            ref='master',
-            oldrev='1' * 40,
-            newrev=headsha)
-        self.waitUntilSettled()
-        self.assertEqual(self.getJobFromHistory('project-post-job').result,
-                         'SUCCESS')
-        self.assertEqual(r, True)
-
     @simple_layout('layouts/crd-gitlab.yaml', driver='gitlab')
     def test_crd_independent(self):
 
@@ -622,10 +606,28 @@ class TestGitlabDriver(ZuulTestCase):
 
         A.labels = ['gateit', 'prio:low']
 
+        self.fake_gitlab.emitEvent(A.getMergeRequestUpdatedEvent())
+        self.waitUntilSettled()
+        self.assertEqual(0, len(self.history))
+
+        A.labels = ['gateit', 'prio:low', 'another_label']
+
         self.fake_gitlab.emitEvent(A.getMergeRequestUpdatedEvent())
         self.waitUntilSettled()
         self.assertEqual(1, len(self.history))
 
+    @simple_layout('layouts/gitlab-label-add-remove.yaml', driver='gitlab')
+    def test_label_add_remove(self):
+
+        A = self.fake_gitlab.openFakeMergeRequest(
+            'org/project1', 'master', 'A')
+        A.labels = ['removeme1', 'removeme2']
+
+        self.fake_gitlab.emitEvent(A.getMergeRequestOpenedEvent())
+        self.waitUntilSettled()
+        self.assertEqual(1, len(self.history))
+        self.assertEqual(set(A.labels), {'addme1', 'addme2'})
+
     @simple_layout('layouts/merging-gitlab.yaml', driver='gitlab')
     def test_merge_action_in_independent(self):
 
@@ -703,6 +705,34 @@ class TestGitlabDriver(ZuulTestCase):
         self.assertTrue(A.is_merged)
         self.assertTrue(B.is_merged)
 
+    @simple_layout('layouts/basic-gitlab.yaml', driver='gitlab')
+    def test_timer_event(self):
+        self.executor_server.hold_jobs_in_build = True
+        self.commitConfigUpdate('org/common-config',
+                                'layouts/timer-gitlab.yaml')
+        self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+        time.sleep(2)
+        self.waitUntilSettled()
+        self.assertEqual(len(self.builds), 1)
+        self.executor_server.hold_jobs_in_build = False
+        # Stop queuing timer triggered jobs so that the assertions
+        # below don't race against more jobs being queued.
+        self.commitConfigUpdate('org/common-config',
+                                'layouts/no-timer-gitlab.yaml')
+        self.scheds.execute(lambda app: app.sched.reconfigure(app.config))
+        self.waitUntilSettled()
+        # If APScheduler is in mid-event when we remove the job, we
+        # can end up with one more event firing, so give it an extra
+        # second to settle.
+        time.sleep(1)
+        self.waitUntilSettled()
+        self.executor_server.release()
+        self.waitUntilSettled()
+        self.assertHistory([
+            dict(name='project-bitrot', result='SUCCESS',
+                 ref='refs/heads/master'),
+        ], ordered=False)
+
     @simple_layout('layouts/crd-gitlab.yaml', driver='gitlab')
     def test_api_token(self):
         tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
@@ -711,7 +741,8 @@ class TestGitlabDriver(ZuulTestCase):
         project_git_url = self.fake_gitlab.real_getGitUrl(project)
         # cloneurl created from config 'server' should be used
         # without credentials
-        self.assertEqual("https://gitlab/org/project1.git", project_git_url)
+        self.assertEqual(f"{self.fake_gitlab._test_baseurl}/org/project1.git",
+                         project_git_url)
 
     @simple_layout('layouts/crd-gitlab.yaml', driver='gitlab2')
     def test_api_token_cloneurl(self):
@@ -743,7 +774,9 @@ class TestGitlabDriver(ZuulTestCase):
         project_git_url = self.fake_gitlab4.real_getGitUrl(project)
         # cloneurl is not set, generate one from token name, token secret and
         # server
-        self.assertEqual("https://tokenname4:444@gitlabfour/org/project1.git",
+        self.assertEqual("http://tokenname4:444@localhost:"
+                         f"{self.fake_gitlab4._test_web_server.port}"
+                         "/org/project1.git",
                          project_git_url)
 
     @simple_layout('layouts/crd-gitlab.yaml', driver='gitlab5')
@@ -763,6 +796,13 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
     config_file = 'zuul-gitlab-driver.conf'
     tenant_config_file = 'config/unprotected-branches-gitlab/main.yaml'
 
+    @skipIfMultiScheduler()
+    # This test is failing with multiple schedulers depending on which
+    # scheduler did the tenant reconfiguration first. As the
+    # assertions are all done on the objects from scheduler-0 they
+    # will fail if scheduler-1 did the reconfig first.
+    # To make this work with multiple schedulers, we might want to wait
+    # until all schedulers completed their tenant reconfiguration.
     def test_unprotected_branches(self):
         tenant = self.scheds.first.sched.abide.tenants\
             .get('tenant-one')
@@ -1009,8 +1049,7 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
                                      expect_reconfigure=False,
                                      old_sha=None, new_sha=None,
                                      modified_files=None,
-                                     removed_files=None,
-                                     expected_cat_jobs=None):
+                                     removed_files=None):
         pevent = self.fake_gitlab.getPushEvent(
             project=project,
             branch='refs/heads/%s' % branch,
@@ -1022,11 +1061,6 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
             'tenant-one', EMPTY_LAYOUT_STATE)
         self.waitUntilSettled()
 
-        if expected_cat_jobs is not None:
-            # clear the gearman jobs history so we can count the cat jobs
-            # issued during reconfiguration
-            self.gearman_server.jobs_history.clear()
-
         self.fake_gitlab.emitEvent(pevent)
         self.waitUntilSettled()
         new = self.scheds.first.sched.tenant_layout_state.get(
@@ -1039,13 +1073,6 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
             # Timestamps should be equal as no reconfiguration shall happen
             self.assertEqual(old, new)
 
-        if expected_cat_jobs is not None:
-            # Check the expected number of cat jobs here as the (empty) config
-            # of org/project should be cached.
-            cat_jobs = set([job for job in self.gearman_server.jobs_history
-                           if job.name == b'merger:cat'])
-            self.assertEqual(expected_cat_jobs, len(cat_jobs), cat_jobs)
-
     def test_push_event_reconfigure_complex_branch(self):
 
         branch = 'feature/somefeature'
@@ -1073,8 +1100,7 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
                                           expect_reconfigure=False,
                                           old_sha=old_sha,
                                           new_sha=new_sha,
-                                          modified_files=['zuul.yaml'],
-                                          expected_cat_jobs=0)
+                                          modified_files=['zuul.yaml'])
 
         # branch is not protected: no reconfiguration
         self.fake_gitlab.deleteBranch(*project.split('/'), branch)
@@ -1084,3 +1110,44 @@ class TestGitlabUnprotectedBranches(ZuulTestCase):
                                           old_sha=new_sha,
                                           new_sha='0' * 40,
                                           removed_files=['zuul.yaml'])
+
+
+class TestGitlabDriverNoPool(ZuulTestCase):
+    config_file = 'zuul-gitlab-driver-no-pool.conf'
+
+    @simple_layout('layouts/basic-gitlab.yaml', driver='gitlab')
+    def test_merge_request_opened(self):
+
+        description = "This is the\nMR description."
+        A = self.fake_gitlab.openFakeMergeRequest(
+            'org/project', 'master', 'A', description=description)
+        self.fake_gitlab.emitEvent(
+            A.getMergeRequestOpenedEvent(), project='org/project')
+        self.waitUntilSettled()
+
+        self.assertEqual('SUCCESS',
+                         self.getJobFromHistory('project-test1').result)
+
+        self.assertEqual('SUCCESS',
+                         self.getJobFromHistory('project-test2').result)
+
+        job = self.getJobFromHistory('project-test2')
+        zuulvars = job.parameters['zuul']
+        self.assertEqual(str(A.number), zuulvars['change'])
+        self.assertEqual(str(A.sha), zuulvars['patchset'])
+        self.assertEqual('master', zuulvars['branch'])
+        self.assertEquals(f'{self.fake_gitlab._test_baseurl}/'
+                          'org/project/merge_requests/1',
+                          zuulvars['items'][0]['change_url'])
+        self.assertEqual(zuulvars["message"], strings.b64encode(description))
+        self.assertEqual(2, len(self.history))
+        self.assertEqual(2, len(A.notes))
+        self.assertEqual(
+            A.notes[0]['body'], "Starting check jobs.")
+        self.assertThat(
+            A.notes[1]['body'],
+            MatchesRegex(r'.*project-test1.*SUCCESS.*', re.DOTALL))
+        self.assertThat(
+            A.notes[1]['body'],
+            MatchesRegex(r'.*project-test2.*SUCCESS.*', re.DOTALL))
+        self.assertTrue(A.approved)
