@@ -22,7 +22,10 @@ import testtools
 
 from zuul.merger.merger import MergerTree, Repo
 import zuul.model
-from tests.base import BaseTestCase, ZuulTestCase, FIXTURE_DIR, simple_layout
+from zuul.model import MergeRequest
+from tests.base import (
+    BaseTestCase, ZuulTestCase, FIXTURE_DIR, simple_layout, iterate_timeout
+)
 
 
 class TestMergerRepo(ZuulTestCase):
@@ -192,9 +195,10 @@ class TestMergerRepo(ZuulTestCase):
         parent_path = os.path.join(self.upstream_root, 'org/project1')
         self.patch(git.Git, 'GIT_PYTHON_GIT_EXECUTABLE',
                    os.path.join(FIXTURE_DIR, 'fake_git.sh'))
+        self.patch(Repo, 'retry_attempts', 1)
         work_repo = Repo(parent_path, self.workspace_root,
                          'none@example.org', 'User Name', '0', '0',
-                         git_timeout=0.001, retry_attempts=1)
+                         git_timeout=0.001)
         # TODO: have the merger and repo classes catch fewer
         # exceptions, including this one on initialization.  For the
         # test, we try cloning again.
@@ -204,9 +208,9 @@ class TestMergerRepo(ZuulTestCase):
 
     def test_fetch_timeout(self):
         parent_path = os.path.join(self.upstream_root, 'org/project1')
+        self.patch(Repo, 'retry_attempts', 1)
         work_repo = Repo(parent_path, self.workspace_root,
-                         'none@example.org', 'User Name', '0', '0',
-                         retry_attempts=1)
+                         'none@example.org', 'User Name', '0', '0')
         work_repo.git_timeout = 0.001
         self.patch(git.Git, 'GIT_PYTHON_GIT_EXECUTABLE',
                    os.path.join(FIXTURE_DIR, 'fake_git.sh'))
@@ -216,9 +220,9 @@ class TestMergerRepo(ZuulTestCase):
 
     def test_fetch_retry(self):
         parent_path = os.path.join(self.upstream_root, 'org/project1')
+        self.patch(Repo, 'retry_interval', 1)
         work_repo = Repo(parent_path, self.workspace_root,
-                         'none@example.org', 'User Name', '0', '0',
-                         retry_interval=1)
+                         'none@example.org', 'User Name', '0', '0')
         self.patch(git.Git, 'GIT_PYTHON_GIT_EXECUTABLE',
                    os.path.join(FIXTURE_DIR, 'git_fetch_error.sh'))
         work_repo.update()
@@ -342,12 +346,22 @@ class TestMergerRepo(ZuulTestCase):
     def test_files_changes(self):
         parent_path = os.path.join(self.upstream_root, 'org/project1')
         self.create_branch('org/project1', 'feature')
+        files = {'feature.txt': 'feature'}
+        self.create_commit('org/project1', files=files, head='feature',
+                           message='Add feature file')
+
+        # Let the master diverge from the feature branch. This new file should
+        # NOT be included in the changed files list.
+        files = {'must-not-be-in-changelist.txt': 'FAIL'}
+        self.create_commit('org/project1', files=files, head='master',
+                           message='Add master file')
 
         work_repo = Repo(parent_path, self.workspace_root,
                          'none@example.org', 'User Name', '0', '0')
         changed_files = work_repo.getFilesChanges('feature', 'master')
 
-        self.assertEqual(['README'], changed_files)
+        self.assertEqual(sorted(['README', 'feature.txt']),
+                         sorted(changed_files))
 
     def test_files_changes_master_fork_merges(self):
         """Regression test for getFilesChanges()
@@ -896,9 +910,10 @@ class TestMerger(ZuulTestCase):
     def test_stale_index_lock_cleanup(self):
         # Stop the running executor's merger. We needed it running to merge
         # things during test boostrapping but now it is just in the way.
-        self.executor_server.merger_gearworker.stop()
-        self.executor_server.merger_gearworker.join()
-        # Start the merger and do a merge to populate the repo on disk
+        self.executor_server._merger_running = False
+        self.executor_server.merger_loop_wake_event.set()
+        self.executor_server.merger_thread.join()
+        # Start a dedicated merger and do a merge to populate the repo on disk
         self._startMerger()
 
         A = self.fake_gerrit.addFakeChange('org/project1', 'master', 'A')
@@ -1004,6 +1019,65 @@ class TestMerger(ZuulTestCase):
         self.assertEqual(upstream_repo.commit(change_ref).hexsha,
                          zuul_repo.commit('HEAD').hexsha)
 
+    def test_lost_merge_requests(self):
+        # Test the cleanupLostMergeRequests method of the merger
+        # client.  This is normally called from apsched from the
+        # scheduler.  To exercise it, we need to produce a fake lost
+        # merge request and then invoke it ourselves.
+
+        # Stop the actual merger which will see this as garbage:
+        self.executor_server._merger_running = False
+        self.executor_server.merger_loop_wake_event.set()
+        self.executor_server.merger_thread.join()
+
+        merger_client = self.scheds.first.sched.merger
+        merger_api = merger_client.merger_api
+
+        # Create a fake lost merge request.  This is based on
+        # test_lost_merge_requests in test_zk.
+
+        payload = {'merge': 'test'}
+        merger_api.submit(MergeRequest(
+            uuid='B',
+            job_type=MergeRequest.MERGE,
+            build_set_uuid='BB',
+            tenant_name='tenant',
+            pipeline_name='check',
+            event_id='1',
+        ), payload)
+
+        b = merger_api.get(f"{merger_api.REQUEST_ROOT}/B")
+
+        b.state = MergeRequest.RUNNING
+        merger_api.update(b)
+
+        # Wait until the latest state transition is reflected in the Merger
+        # APIs cache. Using a DataWatch for this purpose could lead to race
+        # conditions depending on which DataWatch is executed first. The
+        # DataWatch might be triggered for the correct event, but the cache
+        # might still be outdated as the DataWatch that updates the cache
+        # itself wasn't triggered yet.
+        cache = merger_api._cached_requests
+        for _ in iterate_timeout(30, "cache to be up-to-date"):
+            if (cache and cache[b.path].state == MergeRequest.RUNNING):
+                break
+
+        # The lost_merges method should only return merges which are running
+        # but not locked by any merger, in this case merge b
+        lost_merge_requests = list(merger_api.lostRequests())
+
+        self.assertEqual(1, len(lost_merge_requests))
+        self.assertEqual(b.path, lost_merge_requests[0].path)
+
+        # Exercise the cleanup code
+        self.log.debug("Removing lost merge requests")
+        merger_client.cleanupLostMergeRequests()
+
+        cache = merger_api._cached_requests
+        for _ in iterate_timeout(30, "cache to be empty"):
+            if not cache:
+                break
+
 
 class TestMergerTree(BaseTestCase):
 
