@@ -17,6 +17,7 @@ import collections
 import os
 import random
 import types
+from unittest import mock
 
 import fixtures
 import testtools
@@ -29,6 +30,7 @@ import zuul.lib.connections
 
 from tests.base import BaseTestCase, FIXTURE_DIR
 from zuul.lib.ansible import AnsibleManager
+from zuul.zk.zkobject import LocalZKContext
 
 
 class Dummy(object):
@@ -53,16 +55,25 @@ class TestJob(BaseTestCase):
         self.layout = model.Layout(self.tenant)
         self.tenant.layout = self.layout
         self.project = model.Project('project', self.source)
-        self.context = model.SourceContext(self.project, 'master',
-                                           'test', True)
-        self.untrusted_context = model.SourceContext(self.project, 'master',
-                                                     'test', False)
+        self.context = model.SourceContext(
+            self.project.canonical_name, self.project.name,
+            self.project.connection_name, 'master', 'test', True)
+        self.untrusted_context = model.SourceContext(
+            self.project.canonical_name, self.project.name,
+            self.project.connection_name, 'master', 'test', False)
         self.tpc = model.TenantProjectConfig(self.project)
         self.tenant.addUntrustedProject(self.tpc)
         self.pipeline = model.Pipeline('gate', self.tenant)
         self.pipeline.source_context = self.context
+        self.pipeline.manager = mock.Mock()
+        self.pipeline.tenant = self.tenant
+        self.zk_context = LocalZKContext(self.log)
+        self.pipeline.manager.current_context = self.zk_context
+        self.pipeline.state = model.PipelineState()
+        self.pipeline.state._set(pipeline=self.pipeline)
         self.layout.addPipeline(self.pipeline)
-        self.queue = model.ChangeQueue(self.pipeline)
+        self.queue = model.ChangeQueue.new(
+            self.zk_context, pipeline=self.pipeline)
         self.pcontext = configloader.ParseContext(
             self.connections, None, self.tenant, AnsibleManager())
 
@@ -72,7 +83,7 @@ class TestJob(BaseTestCase):
             self.project.private_secrets_key = priv
             self.project.public_secrets_key = pub
         m = yaml.Mark('name', 0, 0, 0, '', 0)
-        self.start_mark = configloader.ZuulMark(m, m, '')
+        self.start_mark = model.ZuulMark(m, m, '')
 
     @property
     def job(self):
@@ -159,9 +170,8 @@ class TestJob(BaseTestCase):
                 "Unable to modify final job"):
             job.applyVariant(bad_final, self.layout)
 
-    def test_job_inheritance_job_tree(self):
-        queue = model.ChangeQueue(self.pipeline)
-
+    @mock.patch("zuul.model.zkobject.ZKObject._save")
+    def test_job_inheritance_job_tree(self, save_mock):
         base = self.pcontext.job_parser.fromYaml({
             '_source_context': self.context,
             '_start_mark': self.start_mark,
@@ -204,34 +214,37 @@ class TestJob(BaseTestCase):
 
         change = model.Change(self.project)
         change.branch = 'master'
-        item = queue.enqueueChange(change, None)
+        item = self.queue.enqueueChange(change, None)
 
         self.assertTrue(base.changeMatchesBranch(change))
         self.assertTrue(python27.changeMatchesBranch(change))
         self.assertFalse(python27diablo.changeMatchesBranch(change))
 
-        item.freezeJobGraph(self.layout)
+        item.freezeJobGraph(self.layout, self.zk_context,
+                            skip_file_matcher=False,
+                            redact_secrets_and_keys=False)
         self.assertEqual(len(item.getJobs()), 1)
         job = item.getJobs()[0]
         self.assertEqual(job.name, 'python27')
         self.assertEqual(job.timeout, 70)
 
         change.branch = 'stable/diablo'
-        item = queue.enqueueChange(change, None)
+        item = self.queue.enqueueChange(change, None)
 
         self.assertTrue(base.changeMatchesBranch(change))
         self.assertTrue(python27.changeMatchesBranch(change))
         self.assertTrue(python27diablo.changeMatchesBranch(change))
 
-        item.freezeJobGraph(self.layout)
+        item.freezeJobGraph(self.layout, self.zk_context,
+                            skip_file_matcher=False,
+                            redact_secrets_and_keys=False)
         self.assertEqual(len(item.getJobs()), 1)
         job = item.getJobs()[0]
         self.assertEqual(job.name, 'python27')
         self.assertEqual(job.timeout, 70)
 
-    def test_inheritance_keeps_matchers(self):
-        queue = model.ChangeQueue(self.pipeline)
-
+    @mock.patch("zuul.model.zkobject.ZKObject._save")
+    def test_inheritance_keeps_matchers(self, save_mock):
         base = self.pcontext.job_parser.fromYaml({
             '_source_context': self.context,
             '_start_mark': self.start_mark,
@@ -265,18 +278,22 @@ class TestJob(BaseTestCase):
         change = model.Change(self.project)
         change.branch = 'master'
         change.files = ['/COMMIT_MSG', 'ignored-file']
-        item = queue.enqueueChange(change, None)
+        item = self.queue.enqueueChange(change, None)
 
         self.assertTrue(base.changeMatchesFiles(change))
         self.assertFalse(python27.changeMatchesFiles(change))
 
-        item.freezeJobGraph(self.layout)
+        self.pipeline.manager.getFallbackLayout = mock.Mock(return_value=None)
+        item.freezeJobGraph(self.layout, self.zk_context,
+                            skip_file_matcher=False,
+                            redact_secrets_and_keys=False)
         self.assertEqual([], item.getJobs())
 
     def test_job_source_project(self):
         base_project = model.Project('base_project', self.source)
-        base_context = model.SourceContext(base_project, 'master',
-                                           'test', True)
+        base_context = model.SourceContext(
+            base_project.canonical_name, base_project.name,
+            base_project.connection_name, 'master', 'test', True)
         tpc = model.TenantProjectConfig(base_project)
         self.tenant.addUntrustedProject(tpc)
 
@@ -289,8 +306,9 @@ class TestJob(BaseTestCase):
         self.layout.addJob(base)
 
         other_project = model.Project('other_project', self.source)
-        other_context = model.SourceContext(other_project, 'master',
-                                            'test', True)
+        other_context = model.SourceContext(
+            other_project.canonical_name, other_project.name,
+            other_project.connection_name, 'master', 'test', True)
         tpc = model.TenantProjectConfig(other_project)
         self.tenant.addUntrustedProject(tpc)
         base2 = self.pcontext.job_parser.fromYaml({
@@ -304,7 +322,8 @@ class TestJob(BaseTestCase):
                 "to shadow job base in base_project"):
             self.layout.addJob(base2)
 
-    def test_job_pipeline_allow_untrusted_secrets(self):
+    @mock.patch("zuul.model.zkobject.ZKObject._save")
+    def test_job_pipeline_allow_untrusted_secrets(self, save_mock):
         self.pipeline.post_review = False
         job = self.pcontext.job_parser.fromYaml({
             '_source_context': self.context,
@@ -334,97 +353,17 @@ class TestJob(BaseTestCase):
         # Test master
         change.branch = 'master'
         item = self.queue.enqueueChange(change, None)
-        item.layout = self.layout
         with testtools.ExpectedException(
                 Exception,
                 "Pre-review pipeline gate does not allow post-review job"):
-            item.freezeJobGraph(self.layout)
-
-
-class TestJobTimeData(BaseTestCase):
-    def setUp(self):
-        super(TestJobTimeData, self).setUp()
-        self.tmp_root = self.useFixture(fixtures.TempDir(
-            rootdir=os.environ.get("ZUUL_TEST_ROOT"))
-        ).path
-
-    def test_empty_timedata(self):
-        path = os.path.join(self.tmp_root, 'job-name')
-        self.assertFalse(os.path.exists(path))
-        self.assertFalse(os.path.exists(path + '.tmp'))
-        td = model.JobTimeData(path)
-        self.assertEqual(td.success_times, [0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
-        self.assertEqual(td.failure_times, [0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
-        self.assertEqual(td.results, [0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
-
-    def test_save_reload(self):
-        path = os.path.join(self.tmp_root, 'job-name')
-        self.assertFalse(os.path.exists(path))
-        self.assertFalse(os.path.exists(path + '.tmp'))
-        td = model.JobTimeData(path)
-        self.assertEqual(td.success_times, [0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
-        self.assertEqual(td.failure_times, [0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
-        self.assertEqual(td.results, [0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
-        success_times = []
-        failure_times = []
-        results = []
-        for x in range(10):
-            success_times.append(int(random.random() * 1000))
-            failure_times.append(int(random.random() * 1000))
-            results.append(0)
-            results.append(1)
-        random.shuffle(results)
-        s = f = 0
-        for result in results:
-            if result:
-                td.add(failure_times[f], 'FAILURE')
-                f += 1
-            else:
-                td.add(success_times[s], 'SUCCESS')
-                s += 1
-        self.assertEqual(td.success_times, success_times)
-        self.assertEqual(td.failure_times, failure_times)
-        self.assertEqual(td.results, results[10:])
-        td.save()
-        self.assertTrue(os.path.exists(path))
-        self.assertFalse(os.path.exists(path + '.tmp'))
-        td = model.JobTimeData(path)
-        td.load()
-        self.assertEqual(td.success_times, success_times)
-        self.assertEqual(td.failure_times, failure_times)
-        self.assertEqual(td.results, results[10:])
-
-
-class TestTimeDataBase(BaseTestCase):
-    def setUp(self):
-        super(TestTimeDataBase, self).setUp()
-        self.tmp_root = self.useFixture(fixtures.TempDir(
-            rootdir=os.environ.get("ZUUL_TEST_ROOT"))
-        ).path
-        self.db = model.TimeDataBase(self.tmp_root)
-
-    def test_timedatabase(self):
-        pipeline = Dummy(tenant=Dummy(name='test-tenant'))
-        change = Dummy(project=Dummy(canonical_name='git.example.com/foo/bar'))
-        job = Dummy(name='job-name')
-        item = Dummy(pipeline=pipeline,
-                     change=change)
-        build = Dummy(build_set=Dummy(item=item),
-                      job=job)
-
-        self.assertEqual(self.db.getEstimatedTime(build), 0)
-        self.db.update(build, 50, 'SUCCESS')
-        self.assertEqual(self.db.getEstimatedTime(build), 50)
-        self.db.update(build, 100, 'SUCCESS')
-        self.assertEqual(self.db.getEstimatedTime(build), 75)
-        for x in range(10):
-            self.db.update(build, 100, 'SUCCESS')
-        self.assertEqual(self.db.getEstimatedTime(build), 100)
+            item.freezeJobGraph(self.layout, self.zk_context,
+                                skip_file_matcher=False,
+                                redact_secrets_and_keys=False)
 
 
 class TestGraph(BaseTestCase):
     def test_job_graph_disallows_multiple_jobs_with_same_name(self):
-        graph = model.JobGraph()
+        graph = model.JobGraph({})
         job1 = model.Job('job')
         job2 = model.Job('job')
         graph.addJob(job1)
@@ -433,7 +372,7 @@ class TestGraph(BaseTestCase):
             graph.addJob(job2)
 
     def test_job_graph_disallows_circular_dependencies(self):
-        graph = model.JobGraph()
+        graph = model.JobGraph({})
         jobs = [model.Job('job%d' % i) for i in range(0, 10)]
         prevjob = None
         for j in jobs[:3]:
@@ -487,14 +426,14 @@ class TestGraph(BaseTestCase):
             model.JobDependency(parent.name, True)])
 
         # With the parent
-        graph = model.JobGraph()
+        graph = model.JobGraph({})
         graph.addJob(parent)
         graph.addJob(child)
         self.assertEqual(graph.getParentJobsRecursively(child.name),
                          [parent])
 
         # Skip the parent
-        graph = model.JobGraph()
+        graph = model.JobGraph({})
         graph.addJob(child)
         self.assertEqual(graph.getParentJobsRecursively(child.name), [])
 
@@ -512,7 +451,7 @@ class TestGraph(BaseTestCase):
             model.JobDependency(parents[4].name),
             model.JobDependency(parents[5].name)])
         # Run them all
-        graph = model.JobGraph()
+        graph = model.JobGraph({})
         for j in parents:
             graph.addJob(j)
         graph.addJob(child)
@@ -520,7 +459,7 @@ class TestGraph(BaseTestCase):
                          set(parents))
 
         # Skip first parent, therefore its recursive dependencies don't appear
-        graph = model.JobGraph()
+        graph = model.JobGraph({})
         for j in parents:
             if j is not parents[0]:
                 graph.addJob(j)
@@ -530,7 +469,7 @@ class TestGraph(BaseTestCase):
                          set([parents[0], parents[2], parents[3]]))
 
         # Skip a leaf node
-        graph = model.JobGraph()
+        graph = model.JobGraph({})
         for j in parents:
             if j is not parents[3]:
                 graph.addJob(j)
