@@ -20,14 +20,13 @@ import socket
 
 from testtools.matchers import MatchesRegex
 
-import zuul.rpcclient
 from zuul.lib import strings
 from zuul.zk.layout import LayoutState
 
 from tests.base import ZuulTestCase, simple_layout
 from tests.base import ZuulWebFixture
 
-EMPTY_LAYOUT_STATE = LayoutState("", "", 0)
+EMPTY_LAYOUT_STATE = LayoutState("", "", 0, None, {})
 
 
 class TestPagureDriver(ZuulTestCase):
@@ -333,80 +332,6 @@ class TestPagureDriver(ZuulTestCase):
         self.assertEqual('project-tag-job', zuulvars['job'])
         self.assertEqual(tagsha, zuulvars['newrev'])
 
-    @simple_layout('layouts/basic-pagure.yaml', driver='pagure')
-    def test_client_dequeue_change_pagure(self):
-
-        client = zuul.rpcclient.RPCClient('127.0.0.1',
-                                          self.gearman_server.port)
-        self.addCleanup(client.shutdown)
-
-        self.executor_server.hold_jobs_in_build = True
-        A = self.fake_pagure.openFakePullRequest('org/project', 'master', 'A')
-
-        self.fake_pagure.emitEvent(A.getPullRequestOpenedEvent())
-        self.waitUntilSettled()
-
-        client.dequeue(
-            tenant='tenant-one',
-            pipeline='check',
-            project='org/project',
-            change='%s,%s' % (A.number, A.commit_stop),
-            ref=None)
-
-        self.waitUntilSettled()
-
-        tenant = self.scheds.first.sched.abide.tenants.get('tenant-one')
-        check_pipeline = tenant.layout.pipelines['check']
-        self.assertEqual(check_pipeline.getAllItems(), [])
-        self.assertEqual(self.countJobResults(self.history, 'ABORTED'), 2)
-
-        self.executor_server.hold_jobs_in_build = False
-        self.executor_server.release()
-        self.waitUntilSettled()
-
-    @simple_layout('layouts/basic-pagure.yaml', driver='pagure')
-    def test_client_enqueue_change_pagure(self):
-
-        A = self.fake_pagure.openFakePullRequest('org/project', 'master', 'A')
-
-        client = zuul.rpcclient.RPCClient('127.0.0.1',
-                                          self.gearman_server.port)
-        self.addCleanup(client.shutdown)
-        r = client.enqueue(tenant='tenant-one',
-                           pipeline='check',
-                           project='org/project',
-                           trigger='pagure',
-                           change='%s,%s' % (A.number, A.commit_stop))
-        self.waitUntilSettled()
-
-        self.assertEqual(self.getJobFromHistory('project-test1').result,
-                         'SUCCESS')
-        self.assertEqual(self.getJobFromHistory('project-test2').result,
-                         'SUCCESS')
-        self.assertEqual(r, True)
-
-    @simple_layout('layouts/basic-pagure.yaml', driver='pagure')
-    def test_client_enqueue_ref_pagure(self):
-        repo_path = os.path.join(self.upstream_root, 'org/project')
-        repo = git.Repo(repo_path)
-        headsha = repo.head.commit.hexsha
-
-        client = zuul.rpcclient.RPCClient('127.0.0.1',
-                                          self.gearman_server.port)
-        self.addCleanup(client.shutdown)
-        r = client.enqueue_ref(
-            tenant='tenant-one',
-            pipeline='post',
-            project='org/project',
-            trigger='pagure',
-            ref='master',
-            oldrev='90f173846e3af9154517b88543ffbd1691f31366',
-            newrev=headsha)
-        self.waitUntilSettled()
-        self.assertEqual(self.getJobFromHistory('project-post-job').result,
-                         'SUCCESS')
-        self.assertEqual(r, True)
-
     @simple_layout('layouts/requirements-pagure.yaml', driver='pagure')
     def test_pr_score_require_1_vote(self):
 
@@ -760,7 +685,7 @@ class TestPagureToGerritCRD(ZuulTestCase):
         self.assertEqual(B.data['status'], 'NEW')
 
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         B.addApproval('Approved', 1)
         self.fake_pagure.emitEvent(
@@ -845,7 +770,7 @@ class TestGerritToPagureCRD(ZuulTestCase):
         self.assertFalse(B.is_merged)
 
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         B.addFlag('success', 'https://url', 'Build passed')
         B.threshold_reached = True
@@ -922,7 +847,7 @@ class TestPagureToGithubCRD(ZuulTestCase):
         self.assertFalse(B.is_merged)
 
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         B.addLabel('approved')
         self.fake_pagure.emitEvent(
@@ -973,6 +898,12 @@ class TestGithubToPagureCRD(ZuulTestCase):
     config_file = 'zuul-crd-pagure.conf'
     tenant_config_file = 'config/cross-source-pagure/github.yaml'
 
+    # Those tests are also using the fake github implementation which
+    # means that every scheduler gets a different fake github instance.
+    # Thus, assertions might fail depending on which scheduler did the
+    # interaction with Github.
+    scheduler_count = 1
+
     def test_crd_gate(self):
         "Test cross-repo dependencies"
         A = self.fake_github.openFakePullRequest('github/project1', 'master',
@@ -991,7 +922,7 @@ class TestGithubToPagureCRD(ZuulTestCase):
         self.assertFalse(B.is_merged)
 
         for connection in self.scheds.first.connections.connections.values():
-            connection.maintainCache([])
+            connection.maintainCache([], max_age=0)
 
         B.addFlag('success', 'https://url', 'Build passed')
         B.threshold_reached = True
@@ -1019,11 +950,8 @@ class TestGithubToPagureCRD(ZuulTestCase):
             'pagure/project2', 'master', 'B')
 
         # A Depends-On: B
-        A.editBody('Depends-On: %s\n' % B.url)
-
         self.executor_server.hold_jobs_in_build = True
-
-        self.fake_github.emitEvent(A.getPullRequestEditedEvent())
+        self.fake_github.emitEvent(A.editBody('Depends-On: %s\n' % B.url))
         self.waitUntilSettled()
 
         self.assertTrue(self.builds[0].hasChanges(A, B))
@@ -1051,7 +979,7 @@ class TestPagureWebhook(ZuulTestCase):
         self.web = self.useFixture(
             ZuulWebFixture(self.changes, self.config,
                            self.additional_event_queues, self.upstream_root,
-                           self.rpcclient, self.poller_events,
+                           self.poller_events,
                            self.git_url_with_auth, self.addCleanup,
                            self.test_root))
 
@@ -1099,7 +1027,7 @@ class TestPagureWebhookWhitelist(ZuulTestCase):
         self.web = self.useFixture(
             ZuulWebFixture(self.changes, self.config,
                            self.additional_event_queues, self.upstream_root,
-                           self.rpcclient, self.poller_events,
+                           self.poller_events,
                            self.git_url_with_auth, self.addCleanup,
                            self.test_root))
 
