@@ -25,7 +25,7 @@ if type apt-get; then
     $SUDO apt-get install -y apt-transport-https curl gnupg2
     # Install recent NodeJS repo
     curl -sS https://deb.nodesource.com/gpgkey/nodesource.gpg.key | $SUDO apt-key add -
-    echo "deb https://deb.nodesource.com/node_10.x bionic main" | $SUDO tee /etc/apt/sources.list.d/nodesource.list
+    echo "deb https://deb.nodesource.com/node_16.x focal main" | $SUDO tee /etc/apt/sources.list.d/nodesource.list
     # Install yarn repo
     curl -sS https://dl.yarnpkg.com/debian/pubkey.gpg | $SUDO apt-key add -
     echo "deb https://dl.yarnpkg.com/debian/ stable main" | $SUDO tee /etc/apt/sources.list.d/yarn.list
