@@ -3,6 +3,7 @@
 set -eu
 
 cd $(dirname $0)
+SCRIPT_DIR="$(pwd)"
 
 # Select docker or podman
 if command -v docker > /dev/null; then
@@ -25,7 +26,7 @@ else
 fi
 
 
-MYSQL="${DOCKER} exec zuul-test-mysql mysql  -u root -pinsecure_slave"
+MYSQL="${DOCKER} exec zuul-test-mysql mysql  -u root -pinsecure_worker"
 
 if [ "${COMPOSE}" == "docker-compose" ]; then
   docker-compose rm -sf
@@ -33,6 +34,11 @@ else
   podman-compose down
 fi
 
+CA_DIR=$SCRIPT_DIR/ca
+
+mkdir -p $CA_DIR
+$SCRIPT_DIR/zk-ca.sh $CA_DIR zuul-test-zookeeper
+
 ${COMPOSE} up -d
 
 echo "Waiting for mysql"
