@@ -16,6 +16,8 @@ import Axios from 'axios'
 
 import * as API from '../api'
 
+import { fetchLogfile } from './logfile'
+
 export const BUILD_FETCH_REQUEST = 'BUILD_FETCH_REQUEST'
 export const BUILD_FETCH_SUCCESS = 'BUILD_FETCH_SUCCESS'
 export const BUILD_FETCH_FAIL = 'BUILD_FETCH_FAIL'
@@ -27,10 +29,12 @@ export const BUILDSET_FETCH_FAIL =    'BUILDSET_FETCH_FAIL'
 export const BUILD_OUTPUT_REQUEST = 'BUILD_OUTPUT_FETCH_REQUEST'
 export const BUILD_OUTPUT_SUCCESS = 'BUILD_OUTPUT_FETCH_SUCCESS'
 export const BUILD_OUTPUT_FAIL = 'BUILD_OUTPUT_FETCH_FAIL'
+export const BUILD_OUTPUT_NOT_AVAILABLE = 'BUILD_OUTPUT_NOT_AVAILABLE'
 
 export const BUILD_MANIFEST_REQUEST = 'BUILD_MANIFEST_FETCH_REQUEST'
 export const BUILD_MANIFEST_SUCCESS = 'BUILD_MANIFEST_FETCH_SUCCESS'
 export const BUILD_MANIFEST_FAIL = 'BUILD_MANIFEST_FETCH_FAIL'
+export const BUILD_MANIFEST_NOT_AVAILABLE = 'BUILD_MANIFEST_NOT_AVAILABLE'
 
 export const requestBuild = () => ({
   type: BUILD_FETCH_REQUEST
@@ -43,10 +47,11 @@ export const receiveBuild = (buildId, build) => ({
   receivedAt: Date.now()
 })
 
-const failedBuild = (error, url) => {
+const failedBuild = (buildId, error, url) => {
   error.url = url
   return {
     type: BUILD_FETCH_FAIL,
+    buildId,
     error
   }
 }
@@ -63,7 +68,7 @@ export function renderTree(tenant, build, path, obj, textRenderer, defaultRender
   if ('children' in obj && obj.children) {
     node.nodes = obj.children.map(
       n => renderTree(tenant, build, path+obj.name+'/', n,
-                     textRenderer, defaultRenderer))
+        textRenderer, defaultRenderer))
   }
   if (obj.mimetype === 'application/directory') {
     name = obj.name + '/'
@@ -138,8 +143,18 @@ export function taskPathMatches (ref, test) {
 }
 
 
-const receiveBuildOutput = (buildId, output) => {
+export const receiveBuildOutput = (buildId, output) => {
   const hosts = {}
+
+  const taskFailed = (taskResult) => {
+    if (taskResult.rc && taskResult.failed_when_result !== false)
+      return true
+    else if (taskResult.failed)
+      return true
+    else
+      return false
+  }
+
   // Compute stats
   output.forEach(phase => {
     Object.entries(phase.stats).forEach(([host, stats]) => {
@@ -159,12 +174,12 @@ const receiveBuildOutput = (buildId, output) => {
               if (task.hosts[host].results &&
                   task.hosts[host].results.length > 0) {
                 task.hosts[host].results.forEach(result => {
-                  if (result.failed) {
+                  if (taskFailed(result)) {
                     result.name = task.task.name
                     hosts[host].failed.push(result)
                   }
                 })
-              } else if (task.hosts[host].rc || task.hosts[host].failed) {
+              } else if (taskFailed(task.hosts[host])) {
                 let result = task.hosts[host]
                 result.name = task.task.name
                 hosts[host].failed.push(result)
@@ -206,10 +221,11 @@ const receiveBuildOutput = (buildId, output) => {
   }
 }
 
-const failedBuildOutput = (error, url) => {
+const failedBuildOutput = (buildId, error, url) => {
   error.url = url
   return {
     type: BUILD_OUTPUT_FAIL,
+    buildId,
     error
   }
 }
@@ -218,7 +234,7 @@ export const requestBuildManifest = () => ({
   type: BUILD_MANIFEST_REQUEST
 })
 
-const receiveBuildManifest = (buildId, manifest) => {
+export const receiveBuildManifest = (buildId, manifest) => {
   const index = {}
 
   const renderNode = (root, object) => {
@@ -236,80 +252,155 @@ const receiveBuildManifest = (buildId, manifest) => {
     type: BUILD_MANIFEST_SUCCESS,
     buildId: buildId,
     manifest: {tree: manifest.tree, index: index,
-               index_links: manifest.index_links},
+      index_links: manifest.index_links},
     receivedAt: Date.now()
   }
 }
 
-const failedBuildManifest = (error, url) => {
+const failedBuildManifest = (buildId, error, url) => {
   error.url = url
   return {
     type: BUILD_MANIFEST_FAIL,
+    buildId,
     error
   }
 }
 
-export const fetchBuild = (tenant, buildId, state, force) => dispatch => {
-  const build = state.build.builds[buildId]
-  if (!force && build) {
-    return Promise.resolve()
+function buildOutputNotAvailable(buildId) {
+  return {
+    type: BUILD_OUTPUT_NOT_AVAILABLE,
+    buildId: buildId,
   }
-  dispatch(requestBuild())
-  return API.fetchBuild(tenant.apiPrefix, buildId)
-    .then(response => {
+}
+
+function buildManifestNotAvailable(buildId) {
+  return {
+    type: BUILD_MANIFEST_NOT_AVAILABLE,
+    buildId: buildId,
+  }
+}
+
+export function fetchBuild(tenant, buildId, state) {
+  return async function (dispatch) {
+    // Although it feels a little weird to not do anything in an action creator
+    // based on the redux state, we do this in here because the function is
+    // called from multiple places and it's easier to check for the build in
+    // here than in all the other places before calling this function.
+    if (state.build.builds[buildId]) {
+      return Promise.resolve()
+    }
+
+    dispatch(requestBuild())
+    try {
+      const response = await API.fetchBuild(tenant.apiPrefix, buildId)
       dispatch(receiveBuild(buildId, response.data))
-    })
-    .catch(error => dispatch(failedBuild(error, tenant.apiPrefix)))
-}
-
-const fetchBuildOutput = (buildId, state, force) => dispatch => {
-  const build = state.build.builds[buildId]
-  const url = build.log_url.substr(0, build.log_url.lastIndexOf('/') + 1)
-  if (!force && build.output) {
-    return Promise.resolve()
-  }
-  dispatch(requestBuildOutput())
-  return Axios.get(url + 'job-output.json.gz')
-    .then(response => dispatch(receiveBuildOutput(buildId, response.data)))
-    .catch(error => {
-      if (!error.request) {
-        throw error
-      }
-      // Try without compression
-      Axios.get(url + 'job-output.json')
-        .then(response => dispatch(receiveBuildOutput(
-          buildId, response.data)))
-    })
-    .catch(error => dispatch(failedBuildOutput(error, url)))
-}
-
-export const fetchBuildManifest = (buildId, state, force) => dispatch => {
-  const build = state.build.builds[buildId]
-  if (!force && build.manifest) {
-    return Promise.resolve()
-  }
-
-  dispatch(requestBuildManifest())
-  for (let artifact of build.artifacts) {
-    if ('metadata' in artifact &&
-        'type' in artifact.metadata &&
-        artifact.metadata.type === 'zuul_manifest') {
-      return Axios.get(artifact.url)
-        .then(manifest => {
-          dispatch(receiveBuildManifest(buildId, manifest.data))
-        })
-        .catch(error => dispatch(failedBuildManifest(error, artifact.url)))
+    } catch (error) {
+      dispatch(failedBuild(buildId, error, tenant.apiPrefix))
+      // Raise the error again, so fetchBuildAllInfo() doesn't call the
+      // remaining fetch methods.
+      throw error
     }
   }
-  dispatch(failedBuildManifest('no manifest found'))
 }
 
-export const fetchBuildIfNeeded = (tenant, buildId, force) => (dispatch, getState) => {
-  dispatch(fetchBuild(tenant, buildId, getState(), force))
-    .then(() => {
-      dispatch(fetchBuildOutput(buildId, getState(), force))
-      dispatch(fetchBuildManifest(buildId, getState(), force))
-    })
+function fetchBuildOutput(buildId, state) {
+  return async function (dispatch) {
+    // In case the value is already set in our local state, directly resolve the
+    // promise. A null value means that the output could not be found for this
+    // build id.
+    if (state.build.outputs[buildId] !== undefined) {
+      return Promise.resolve()
+    }
+    // As this function is only called after fetchBuild() we can assume that
+    // the build is in the state. Otherwise an error would have been thrown and
+    // this function wouldn't be called.
+    const build = state.build.builds[buildId]
+    if (!build.log_url) {
+      // Don't treat a missing log URL as failure as we don't want to show a
+      // toast for that. The UI already informs about the missing log URL in
+      // multiple places.
+      return dispatch(buildOutputNotAvailable(buildId))
+    }
+    const url = build.log_url.substr(0, build.log_url.lastIndexOf('/') + 1)
+    dispatch(requestBuildOutput())
+    try {
+      const response = await Axios.get(url + 'job-output.json.gz')
+      dispatch(receiveBuildOutput(buildId, response.data))
+    } catch (error) {
+      if (!error.request) {
+        dispatch(failedBuildOutput(buildId, error, url))
+        // Raise the error again, so fetchBuildAllInfo() doesn't call the
+        // remaining fetch methods.
+        throw error
+      }
+      try {
+        // Try without compression
+        const response = await Axios.get(url + 'job-output.json')
+        dispatch(receiveBuildOutput(buildId, response.data))
+      } catch (error) {
+        dispatch(failedBuildOutput(buildId, error, url))
+        // Raise the error again, so fetchBuildAllInfo() doesn't call the
+        // remaining fetch methods.
+        throw error
+      }
+    }
+  }
+}
+
+export function fetchBuildManifest(buildId, state) {
+  return async function(dispatch) {
+    // In case the value is already set in our local state, directly resolve the
+    // promise. A null value means that the manifest could not be found for this
+    // build id.
+    if (state.build.manifests[buildId] !== undefined) {
+      return Promise.resolve()
+    }
+    // As this function is only called after fetchBuild() we can assume that
+    // the build is in the state. Otherwise an error would have been thrown and
+    // this function wouldn't be called.
+    const build = state.build.builds[buildId]
+    dispatch(requestBuildManifest())
+    for (let artifact of build.artifacts) {
+      if (
+        'metadata' in artifact &&
+        'type' in artifact.metadata &&
+        artifact.metadata.type === 'zuul_manifest'
+      ) {
+        try {
+          const response = await Axios.get(artifact.url)
+          return dispatch(receiveBuildManifest(buildId, response.data))
+        } catch(error) {
+          // Show the error since we expected a manifest but did not
+          // receive it.
+          dispatch(failedBuildManifest(buildId, error, artifact.url))
+        }
+      }
+    }
+    // Don't treat a missing manifest file as failure as we don't want to show a
+    // toast for that.
+    dispatch(buildManifestNotAvailable(buildId))
+  }
+}
+
+export function fetchBuildAllInfo(tenant, buildId, logfileName) {
+  // This wraps the calls to fetch the build, output and manifest together as
+  // this is the common use case we have when loading the build info.
+  return async function (dispatch, getState) {
+    try {
+      // Wait for the build to be available as fetchBuildOutput and
+      // fetchBuildManifest require information from the build object.
+      await dispatch(fetchBuild(tenant, buildId, getState()))
+      dispatch(fetchBuildOutput(buildId, getState()))
+      // Wait for the manifest info to be available as this is needed in case
+      // we also download a logfile.
+      await dispatch(fetchBuildManifest(buildId, getState()))
+      if (logfileName) {
+        dispatch(fetchLogfile(buildId, logfileName, getState()))
+      }
+    } catch (error) {
+      dispatch(failedBuild(buildId, error, tenant.apiPrefix))
+    }
+  }
 }
 
 export const requestBuildset = () => ({
@@ -323,23 +414,22 @@ export const receiveBuildset = (buildsetId, buildset) => ({
   receivedAt: Date.now()
 })
 
-const failedBuildset = error => ({
+const failedBuildset = (buildsetId, error) => ({
   type: BUILDSET_FETCH_FAIL,
+  buildsetId,
   error
 })
 
-const fetchBuildset = (tenant, buildset) => dispatch => {
-  dispatch(requestBuildset())
-  return API.fetchBuildset(tenant.apiPrefix, buildset)
-    .then(response => {
-      if (response.data.builds) {
-        response.data.builds.forEach(build => {
-          dispatch(receiveBuild(build.uuid, build))
-        })
-      }
-      dispatch(receiveBuildset(buildset, response.data))
-    })
-    .catch(error => dispatch(failedBuildset(error)))
+export function fetchBuildset(tenant, buildsetId) {
+  return async function(dispatch) {
+    dispatch(requestBuildset())
+    try {
+      const response = await API.fetchBuildset(tenant.apiPrefix, buildsetId)
+      dispatch(receiveBuildset(buildsetId, response.data))
+    } catch (error) {
+      dispatch(failedBuildset(buildsetId, error))
+    }
+  }
 }
 
 const shouldFetchBuildset = (buildsetId, state) => {
@@ -355,7 +445,7 @@ const shouldFetchBuildset = (buildsetId, state) => {
 
 export const fetchBuildsetIfNeeded = (tenant, buildsetId, force) => (
   dispatch, getState) => {
-    if (force || shouldFetchBuildset(buildsetId, getState())) {
-      return dispatch(fetchBuildset(tenant, buildsetId))
-    }
+  if (force || shouldFetchBuildset(buildsetId, getState())) {
+    return dispatch(fetchBuildset(tenant, buildsetId))
+  }
 }
