@@ -20,14 +20,18 @@ import Change from './Change'
 
 class ChangeQueue extends React.Component {
   static propTypes = {
-    pipeline: PropTypes.string.isRequired,
+    pipeline: PropTypes.object.isRequired,
     queue: PropTypes.object.isRequired,
     expanded: PropTypes.bool.isRequired
   }
 
-  render () {
+  render() {
     const { queue, pipeline, expanded } = this.props
-    let shortName = queue.name
+    let fullName = queue.name
+    if (queue.branch) {
+      fullName = `${fullName} (${queue.branch})`
+    }
+    let shortName = fullName
     if (shortName.length > 32) {
       shortName = shortName.substr(0, 32) + '...'
     }
@@ -39,13 +43,14 @@ class ChangeQueue extends React.Component {
             change={change}
             queue={queue}
             expanded={expanded}
+            pipeline={pipeline}
             key={changeIdx.toString() + idx}
           />)
       })
     })
     return (
-      <div className="change-queue" data-zuul-pipeline={pipeline}>
-        <p>Queue: <abbr title={queue.name}>{shortName}</abbr></p>
+      <div className="change-queue" data-zuul-pipeline={pipeline.name}>
+        <p>Queue: <abbr title={fullName}>{shortName}</abbr></p>
         {changesList}
       </div>)
   }
