@@ -15,13 +15,14 @@
 import * as React from 'react'
 import PropTypes from 'prop-types'
 import { connect } from 'react-redux'
+import { PageSection, PageSectionVariants } from '@patternfly/react-core'
 
 import { fetchChangeIfNeeded } from '../actions/change'
 import ChangePanel from '../containers/status/ChangePanel'
-import Refreshable from '../containers/Refreshable'
+import { Fetchable } from '../containers/Fetching'
 
 
-class ChangeStatusPage extends Refreshable {
+class ChangeStatusPage extends React.Component {
   static propTypes = {
     match: PropTypes.object.isRequired,
     tenant: PropTypes.object,
@@ -42,7 +43,15 @@ class ChangeStatusPage extends Refreshable {
 
   componentDidMount () {
     document.title = this.props.match.params.changeId + ' | Zuul Status'
-    super.componentDidMount()
+    if (this.props.tenant.name) {
+      this.updateData()
+    }
+  }
+
+  componentDidUpdate (prevProps) {
+    if (this.props.tenant.name !== prevProps.tenant.name) {
+      this.updateData()
+    }
   }
 
   componentWillUnmount () {
@@ -56,19 +65,22 @@ class ChangeStatusPage extends Refreshable {
     const { remoteData } = this.props
     const change = remoteData.change
     return (
-      <React.Fragment>
-        <div style={{float: 'right'}}>
-          {this.renderSpinner()}
-        </div><br />
+      <PageSection variant={PageSectionVariants.light}>
+        <PageSection style={{paddingRight: '5px'}}>
+          <Fetchable
+            isFetching={remoteData.isFetching}
+            fetchCallback={this.updateData}
+          />
+        </PageSection>
         {change && change.map((item, idx) => (
-          <div className='row' key={idx}>
+          <div className='row zuul-change-content' key={idx}>
             <ChangePanel
               globalExpanded={true}
               change={item}
-              />
+            />
           </div>
         ))}
-      </React.Fragment>)
+      </PageSection>)
   }
 }
 
