@@ -14,9 +14,12 @@
 
 import { combineReducers } from 'redux'
 
+import auth from './auth'
+import autoholds from './autoholds'
 import configErrors from './configErrors'
 import change from './change'
-import errors from './errors'
+import component from './component'
+import notifications from './notifications'
 import build from './build'
 import info from './info'
 import job from './job'
@@ -32,12 +35,16 @@ import status from './status'
 import tenant from './tenant'
 import tenants from './tenants'
 import timezone from './timezone'
+import user from './user'
 
 const reducers = {
+  auth,
+  autoholds,
   build,
   change,
+  component,
   configErrors,
-  errors,
+  notifications,
   info,
   job,
   jobs,
@@ -52,6 +59,7 @@ const reducers = {
   tenants,
   timezone,
   preferences,
+  user,
 }
 
 export default combineReducers(reducers)
