@@ -23,22 +23,45 @@ from zuul.model import Change, TriggerEvent
 from zuul.driver.util import time_to_seconds
 from zuul import exceptions
 
-
 EMPTY_GIT_REF = '0' * 40  # git sha of all zeros, used during creates/deletes
 
 
 class GerritChange(Change):
     def __init__(self, project):
         super(GerritChange, self).__init__(project)
+        self.id = None
         self.status = None
         self.wip = None
         self.approvals = []
+        self.missing_labels = set()
+        self.commit = None
 
     def update(self, data, connection):
         if data.format == data.SSH:
             self.updateFromSSH(data.data, connection)
         else:
-            self.updateFromHTTP(data.data, connection)
+            self.updateFromHTTP(data.data, data.files, connection)
+
+    def serialize(self):
+        d = super().serialize()
+        d.update({
+            "id": self.id,
+            "status": self.status,
+            "wip": self.wip,
+            "approvals": self.approvals,
+            "missing_labels": list(self.missing_labels),
+            "commit": self.commit,
+        })
+        return d
+
+    def deserialize(self, data):
+        super().deserialize(data)
+        self.id = data.get("id")
+        self.status = data["status"]
+        self.wip = data["wip"]
+        self.approvals = data["approvals"]
+        self.missing_labels = set(data["missing_labels"])
+        self.commit = data.get("commit")
 
     def updateFromSSH(self, data, connection):
         if self.patchset is None:
@@ -65,6 +88,8 @@ class GerritChange(Change):
             if str(ps['number']) == self.patchset:
                 self.ref = ps['ref']
                 self.commit = ps['revision']
+                # SSH queries gives us a list of dicts for the files. We
+                # convert that to a list of filenames.
                 for f in ps.get('files', []):
                     files.append(f['file'])
             if int(ps['number']) > int(max_ps):
@@ -73,6 +98,13 @@ class GerritChange(Change):
             self.is_current_patchset = True
         else:
             self.is_current_patchset = False
+        if len(data.get('parents', [])) > 1:
+            # This is a merge commit, and the SSH query only reports
+            # files in this commit's content (not files changed by the
+            # underlying merged changes).  Set files to None to
+            # instruct Zuul to ask the mergers to get the full file
+            # list.
+            self.files = None
         self.files = files
         self.id = data['id']
         self.is_merged = data.get('status', '') == 'MERGED'
@@ -82,6 +114,7 @@ class GerritChange(Change):
         self.wip = data.get('wip', False)
         self.owner = data['owner'].get('username')
         self.message = data['commitMessage']
+        self.topic = data.get('topic')
 
         self.missing_labels = set()
         for sr in data.get('submitRecords', []):
@@ -92,7 +125,7 @@ class GerritChange(Change):
                     elif label['status'] in ['NEED', 'REJECT']:
                         self.missing_labels.add(label['label'])
 
-    def updateFromHTTP(self, data, connection):
+    def updateFromHTTP(self, data, files, connection):
         urlparse = urllib.parse.urlparse(connection.baseurl)
         baseurl = "%s://%s%s" % (urlparse.scheme, urlparse.netloc,
                                  urlparse.path)
@@ -111,15 +144,19 @@ class GerritChange(Change):
             '%s/c/%s/+/%s' % (baseurl, self.project.name, self.number),
         ]
 
-        files = []
         if str(current_revision['_number']) == self.patchset:
             self.ref = current_revision['ref']
             self.commit = data['current_revision']
-            files = list(current_revision.get('files', []).keys())
             self.is_current_patchset = True
         else:
             self.is_current_patchset = False
-        self.files = files
+
+        # HTTP queries give us a dict of files in the form of
+        # {filename: { attrs }}. We only want a list of filenames here.
+        if files:
+            self.files = list(files.keys())
+        else:
+            self.files = []
 
         self.is_merged = data['status'] == 'MERGED'
         self.approvals = []
@@ -153,6 +190,7 @@ class GerritChange(Change):
         self.wip = data.get('work_in_progress', False)
         self.owner = data['owner'].get('username')
         self.message = current_revision['commit']['message']
+        self.topic = data.get('topic')
 
 
 class GerritTriggerEvent(TriggerEvent):
