@@ -60,9 +60,6 @@ class GitWatcher(threading.Thread):
             connection.sched.zk_client,
             connection.connection_name,
             election_name)
-        self._connection_lost_event = threading.Event()
-        connection.sched.zk_client.on_connection_lost_listeners.append(
-            self._connection_lost_event.set)
         # This is used by the test framework
         self._event_count = 0
         self._pause = False
@@ -124,7 +121,7 @@ class GitWatcher(threading.Thread):
     def _poll(self):
         self.log.debug("Walk through projects refs for connection: %s" %
                        self.connection.connection_name)
-        for project in self.connection.projects:
+        for project in list(self.connection.projects)[:]:
             refs = self.lsRemote(project)
             self.log.debug("Read %s refs for project %s",
                            len(refs), project)
@@ -142,7 +139,7 @@ class GitWatcher(threading.Thread):
                 self._event_count += 1
 
     def _run(self):
-        while not (self._stopped or self._connection_lost_event.is_set()):
+        while not self._stopped and self.watcher_election.is_still_valid():
             if not self._pause:
                 # during tests, a sub-class _poll method is used to send
                 # notifications
@@ -158,7 +155,6 @@ class GitWatcher(threading.Thread):
                 self.watcher_election.run(self._run)
             except Exception:
                 self.log.exception("Unexpected issue in _run loop:")
-            self._connection_lost_event.clear()
 
     def stop(self):
         self.log.debug("Stopping watcher")
