@@ -30,7 +30,6 @@ EMPTY_GIT_REF = '0' * 40  # git sha of all zeros, used during creates/deletes
 class PullRequest(Change):
     def __init__(self, project):
         super(PullRequest, self).__init__(project)
-        self.project = None
         self.pr = None
         self.updated_at = None
         self.title = None
@@ -65,6 +64,37 @@ class PullRequest(Change):
             return True
         return False
 
+    def serialize(self):
+        d = super().serialize()
+        d.update({
+            "pr": self.pr,
+            "updated_at": self.updated_at,
+            "title": self.title,
+            "body_text": self.body_text,
+            "reviews": list(self.reviews),
+            "labels": self.labels,
+            "draft": self.draft,
+            "review_decision": self.review_decision,
+            "required_contexts": list(self.required_contexts),
+            "contexts": list(self.contexts),
+            "branch_protected": self.branch_protected,
+        })
+        return d
+
+    def deserialize(self, data):
+        super().deserialize(data)
+        self.pr = data.get("pr")
+        self.updated_at = data.get("updated_at")
+        self.title = data.get("title")
+        self.body_text = data.get("body_text")
+        self.reviews = data.get("reviews", [])
+        self.labels = data.get("labels", [])
+        self.draft = data.get("draft")
+        self.review_decision = data.get("review_decision")
+        self.required_contexts = set(data.get("required_contexts", []))
+        self.contexts = set(tuple(c) for c in data.get("contexts", []))
+        self.branch_protected = data.get("branch_protected", False)
+
 
 class GithubTriggerEvent(TriggerEvent):
     def __init__(self):
@@ -77,6 +107,8 @@ class GithubTriggerEvent(TriggerEvent):
         self.check_run = None
         self.status = None
         self.commits = []
+        self.body_edited = None
+        self.branch_protection_changed = None
 
     def toDict(self):
         d = super().toDict()
@@ -88,6 +120,8 @@ class GithubTriggerEvent(TriggerEvent):
         d["check_run"] = self.check_run
         d["status"] = self.status
         d["commits"] = self.commits
+        d["body_edited"] = self.body_edited
+        d["branch_protection_changed"] = self.branch_protection_changed
         return d
 
     def updateFromDict(self, d):
@@ -100,12 +134,20 @@ class GithubTriggerEvent(TriggerEvent):
         self.check_run = d["check_run"]
         self.status = d["status"]
         self.commits = d["commits"]
+        self.body_edited = d["body_edited"]
+        self.branch_protection_changed = d.get("branch_protection_changed")
+
+    def isBranchProtectionChanged(self):
+        return bool(self.branch_protection_changed)
 
     def isPatchsetCreated(self):
         if self.type == 'pull_request':
             return self.action in ['opened', 'changed']
         return False
 
+    def isMessageChanged(self):
+        return bool(self.body_edited)
+
     def isChangeAbandoned(self):
         if self.type == 'pull_request':
             return 'closed' == self.action
