@@ -1,4 +1,5 @@
 # Copyright 2019 Red Hat, Inc.
+# Copyright 2022 Acme Gating, LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -21,20 +22,44 @@ import time
 import uuid
 import re
 import requests
+import urllib3
 
 import dateutil.parser
 
 from urllib.parse import quote_plus
 from typing import List, Optional
 
-from zuul.connection import CachedBranchConnection
+from zuul.connection import (
+    BaseConnection, ZKChangeCacheMixin, ZKBranchCacheMixin
+)
 from zuul.web.handler import BaseWebController
+from zuul.lib.http import ZuulHTTPAdapter
 from zuul.lib.logutil import get_annotated_logger
+from zuul.lib.config import any_to_bool
 from zuul.exceptions import MergeFailure
 from zuul.model import Branch, Project, Ref, Tag
 from zuul.driver.gitlab.gitlabmodel import GitlabTriggerEvent, MergeRequest
+from zuul.zk.branch_cache import BranchCache
+from zuul.zk.change_cache import (
+    AbstractChangeCache,
+    ConcurrentUpdateError,
+)
 from zuul.zk.event_queues import ConnectionEventQueue
 
+# HTTP timeout in seconds
+TIMEOUT = 30
+
+
+class GitlabChangeCache(AbstractChangeCache):
+    log = logging.getLogger("zuul.driver.GitlabChangeCache")
+
+    CHANGE_TYPE_MAP = {
+        "Ref": Ref,
+        "Tag": Tag,
+        "Branch": Branch,
+        "MergeRequest": MergeRequest,
+    }
+
 
 class GitlabEventConnector(threading.Thread):
     """Move events from Gitlab into the scheduler"""
@@ -66,6 +91,11 @@ class GitlabEventConnector(threading.Thread):
         return not self._stopped
 
     def run(self):
+        # Wait for the scheduler to prime its config so that we have
+        # the full tenant list before we start moving events.
+        self.connection.sched.primed_event.wait()
+        if self._stopped:
+            return
         self.event_queue.registerEventWatch(self._onNewEvent)
         while not self._stopped:
             try:
@@ -108,6 +138,8 @@ class GitlabEventConnector(threading.Thread):
         event.patch_number = attrs['last_commit']['id']
         event.change_url = self.connection.getMRUrl(event.project_name,
                                                     event.change_number)
+        if attrs['action'] == 'update' and "description" in body["changes"]:
+            event.merge_request_description_changed = True
         if attrs['action'] == 'open':
             event.action = 'opened'
         elif attrs['action'] == 'merge':
@@ -124,6 +156,8 @@ class GitlabEventConnector(threading.Thread):
                 label in body["changes"]["labels"]["current"]]
             new_labels = set(current_labels) - set(previous_labels)
             event.labels = list(new_labels)
+            removed_labels = set(previous_labels) - set(current_labels)
+            event.unlabels = list(removed_labels)
         elif attrs['action'] in ('approved', 'unapproved'):
             event.action = attrs['action']
         else:
@@ -157,6 +191,7 @@ class GitlabEventConnector(threading.Thread):
         event.newrev = body['after']
         event.oldrev = body['before']
         event.type = 'gl_push'
+        event.commits = body.get('commits')
 
         self.connection.clearConnectionCacheOnBranchEvent(event)
 
@@ -176,39 +211,37 @@ class GitlabEventConnector(threading.Thread):
         if self._stopped:
             return
 
+        zuul_event_id = str(uuid.uuid4())
+        log = get_annotated_logger(self.log, zuul_event_id)
         timestamp = time.time()
         json_body = connection_event["payload"]
-        self.log.info(json_body)
+        log.debug("Received payload: %s", json_body)
 
         event_type = json_body['object_kind']
-        self.log.info("Received event: %s", event_type)
+        log.debug("Received event: %s", event_type)
 
         if event_type not in self.event_handler_mapping:
             message = "Unhandled Gitlab event: %s" % event_type
-            self.log.info(message)
+            log.info(message)
             return
 
         if event_type in self.event_handler_mapping:
-            self.log.debug("Handling event: %s" % event_type)
+            log.info("Handling event: %s" % event_type)
 
         try:
             event = self.event_handler_mapping[event_type](json_body)
         except Exception:
-            self.log.exception(
+            log.exception(
                 'Exception when handling event: %s' % event_type)
             event = None
 
         if event:
-            event.zuul_event_id = str(uuid.uuid4())
+            event.zuul_event_id = zuul_event_id
             event.timestamp = timestamp
             event.project_hostname = self.connection.canonical_hostname
             if event.change_number:
-                project = self.connection.source.getProject(event.project_name)
-                self.connection._getChange(project,
-                                           event.change_number,
-                                           event.patch_number,
-                                           refresh=True,
-                                           url=event.change_url,
+                change_key = self.connection.source.getChangeKey(event)
+                self.connection._getChange(change_key, refresh=True,
                                            event=event)
 
             # If this event references a branch and we're excluding
@@ -230,13 +263,34 @@ class GitlabAPIClientException(Exception):
 class GitlabAPIClient():
     log = logging.getLogger("zuul.GitlabAPIClient")
 
-    def __init__(self, baseurl, api_token):
-        self.session = requests.Session()
+    def __init__(self, baseurl, api_token, keepalive, disable_pool):
+        self._session = None
+        self._orig_baseurl = baseurl
         self.baseurl = '%s/api/v4' % baseurl
         self.api_token = api_token
+        self.keepalive = keepalive
+        self.disable_pool = disable_pool
         self.headers = {'Authorization': 'Bearer %s' % (
             self.api_token)}
 
+        if not self.disable_pool:
+            self._session = self._makeSession()
+
+    def _makeSession(self):
+        session = requests.Session()
+        retry = urllib3.util.Retry(total=8,
+                                   backoff_factor=0.1)
+        adapter = ZuulHTTPAdapter(keepalive=self.keepalive,
+                                  max_retries=retry)
+        session.mount(self._orig_baseurl, adapter)
+        return session
+
+    @property
+    def session(self):
+        if self.disable_pool:
+            return self._makeSession()
+        return self._session
+
     def _manage_error(self, data, code, url, verb, zuul_event_id=None):
         if code < 400:
             return
@@ -249,7 +303,8 @@ class GitlabAPIClient():
     def get(self, url, zuul_event_id=None):
         log = get_annotated_logger(self.log, zuul_event_id)
         log.debug("Getting resource %s ..." % url)
-        ret = self.session.get(url, headers=self.headers)
+        ret = self.session.get(url, headers=self.headers,
+                               timeout=TIMEOUT)
         log.debug("GET returned (code: %s): %s" % (
             ret.status_code, ret.text))
         return ret.json(), ret.status_code, ret.url, 'GET'
@@ -258,7 +313,8 @@ class GitlabAPIClient():
         log = get_annotated_logger(self.log, zuul_event_id)
         log.info(
             "Posting on resource %s, params (%s) ..." % (url, params))
-        ret = self.session.post(url, data=params, headers=self.headers)
+        ret = self.session.post(url, data=params, headers=self.headers,
+                                timeout=TIMEOUT)
         log.debug("POST returned (code: %s): %s" % (
             ret.status_code, ret.text))
         return ret.json(), ret.status_code, ret.url, 'POST'
@@ -267,7 +323,8 @@ class GitlabAPIClient():
         log = get_annotated_logger(self.log, zuul_event_id)
         log.info(
             "Put on resource %s, params (%s) ..." % (url, params))
-        ret = self.session.put(url, data=params, headers=self.headers)
+        ret = self.session.put(url, data=params, headers=self.headers,
+                               timeout=TIMEOUT)
         log.debug("PUT returned (code: %s): %s" % (
             ret.status_code, ret.text))
         return ret.json(), ret.status_code, ret.url, 'PUT'
@@ -367,18 +424,37 @@ class GitlabAPIClient():
 
     # https://docs.gitlab.com/ee/api/merge_requests.html#accept-mr
     def merge_mr(self, project_name, number,
+                 method,
                  zuul_event_id=None):
         path = "/projects/%s/merge_requests/%s/merge" % (
             quote_plus(project_name), number)
+        params = {}
+        if method == "squash":
+            params['squash'] = True
         resp = self.put(self.baseurl + path, zuul_event_id=zuul_event_id)
         try:
             self._manage_error(*resp, zuul_event_id=zuul_event_id)
+            if resp[0]['state'] != 'merged':
+                raise MergeFailure(
+                    "Merge request merge failed: %s" % resp.get('merge_error')
+                )
         except GitlabAPIClientException as e:
             raise MergeFailure('Merge request merge failed: %s' % e)
         return resp[0]
 
+    # https://docs.gitlab.com/ee/api/merge_requests.html#update-mr
+    def update_mr(self, project_name, number,
+                  zuul_event_id=None,
+                  **params):
+        path = "/projects/%s/merge_requests/%s" % (
+            quote_plus(project_name), number)
+        resp = self.put(self.baseurl + path, params=params,
+                        zuul_event_id=zuul_event_id)
+        self._manage_error(*resp, zuul_event_id=zuul_event_id)
+        return resp[0]
 
-class GitlabConnection(CachedBranchConnection):
+
+class GitlabConnection(ZKChangeCacheMixin, ZKBranchCacheMixin, BaseConnection):
     driver_name = 'gitlab'
     log = logging.getLogger("zuul.GitlabConnection")
     payload_path = 'payload'
@@ -387,7 +463,6 @@ class GitlabConnection(CachedBranchConnection):
         super(GitlabConnection, self).__init__(
             driver, connection_name, connection_config)
         self.projects = {}
-        self._change_cache = {}
         self.server = self.connection_config.get('server', 'gitlab.com')
         self.baseurl = self.connection_config.get(
             'baseurl', 'https://%s' % self.server).rstrip('/')
@@ -401,7 +476,12 @@ class GitlabConnection(CachedBranchConnection):
             'api_token_name', '')
         self.api_token = self.connection_config.get(
             'api_token', '')
-        self.gl_client = GitlabAPIClient(self.baseurl, self.api_token)
+        self.keepalive = self.connection_config.get('keepalive', 60)
+        self.disable_pool = any_to_bool(self.connection_config.get(
+            'disable_connection_pool', False))
+
+        self.gl_client = GitlabAPIClient(self.baseurl, self.api_token,
+                                         self.keepalive, self.disable_pool)
         self.sched = None
         self.source = driver.getSource(self)
 
@@ -414,12 +494,29 @@ class GitlabConnection(CachedBranchConnection):
             self.gitlab_event_connector.stop()
             self.gitlab_event_connector.join()
 
-    def onLoad(self):
+    def onLoad(self, zk_client, component_registry):
         self.log.info('Starting Gitlab connection: %s', self.connection_name)
+
+        # Set the project branch cache to read only if no scheduler is
+        # provided to prevent fetching the branches from the connection.
+        self.read_only = not self.sched
+
+        self.log.debug('Creating Zookeeper branch cache')
+        self._branch_cache = BranchCache(zk_client, self, component_registry)
+
         self.log.info('Creating Zookeeper event queue')
         self.event_queue = ConnectionEventQueue(
-            self.sched.zk_client, self.connection_name
+            zk_client, self.connection_name
         )
+
+        # If the connection was not loaded by a scheduler, but by e.g.
+        # zuul-web, we want to stop here.
+        if not self.sched:
+            return
+
+        self.log.debug('Creating Zookeeper change cache')
+        self._change_cache = GitlabChangeCache(zk_client, self)
+
         self.log.info('Starting event connector')
         self._start_event_connector()
 
@@ -475,72 +572,94 @@ class GitlabConnection(CachedBranchConnection):
                 project.name)
         return cloneurl
 
-    def getChange(self, event, refresh=False):
-        project = self.source.getProject(event.project_name)
-        if event.change_number:
+    def getChange(self, change_key, refresh=False, event=None):
+        if change_key.connection_name != self.connection_name:
+            return None
+        if change_key.change_type == 'MergeRequest':
             self.log.info("Getting change for %s#%s" % (
-                project, event.change_number))
-            change = self._getChange(
-                project, event.change_number, event.patch_number,
-                refresh=refresh, event=event)
-            change.is_current_patchset = (change.patchset ==
-                                          event.patch_number)
+                change_key.project_name, change_key.stable_id))
+            change = self._getChange(change_key,
+                                     refresh=refresh, event=event)
         else:
             self.log.info("Getting change for %s ref:%s" % (
-                project, event.ref))
-            if event.ref and event.ref.startswith('refs/tags/'):
-                change = Tag(project)
-                change.tag = event.tag
-                change.branch = None
-            elif event.ref and event.ref.startswith('refs/heads/'):
-                change = Branch(project)
-                change.branch = event.branch
-            else:
-                change = Ref(project)
-                change.branch = None
-            change.ref = event.ref
-            change.oldrev = event.oldrev
-            change.newrev = event.newrev
-            change.url = self.getGitwebUrl(project, sha=event.newrev)
-
-            change.files = None
+                change_key.project_name, change_key.stable_id))
+            change = self._getNonMRRef(change_key, event=event)
         return change
 
-    def _getChange(self, project, number, patch_number=None,
-                   refresh=False, url=None, event=None):
+    def _getChange(self, change_key, refresh=False, event=None):
         log = get_annotated_logger(self.log, event)
-        key = (project.name, str(number), str(patch_number))
-        change = self._change_cache.get(key)
+        number = int(change_key.stable_id)
+        change = self._change_cache.get(change_key)
         if change and not refresh:
-            log.debug("Getting change from cache %s" % str(key))
+            log.debug("Getting change from cache %s" % str(change_key))
             return change
+        project = self.source.getProject(change_key.project_name)
         if not change:
+            if not event:
+                self.log.error("Change %s not found in cache and no event",
+                               change_key)
+            if event:
+                url = event.change_url
             change = MergeRequest(project.name)
             change.project = project
             change.number = number
             # patch_number is the tips commit SHA of the MR
-            change.patchset = patch_number
+            change.patchset = change_key.revision
             change.url = url or self.getMRUrl(project.name, number)
             change.uris = [change.url.split('://', 1)[-1]]  # remove scheme
-        self._change_cache[key] = change
-        try:
-            log.debug("Getting change mr#%s from project %s" % (
-                number, project.name))
-            self._updateChange(change, event)
-        except Exception:
-            if key in self._change_cache:
-                del self._change_cache[key]
-            raise
+
+        log.debug("Getting change mr#%s from project %s" % (
+            number, project.name))
+        log.info("Updating change from Gitlab %s" % change)
+        mr = self.getMR(change.project.name, change.number, event=event)
+
+        def _update_change(c):
+            self._updateChange(c, event, mr)
+
+        change = self._change_cache.updateChangeWithRetry(change_key, change,
+                                                          _update_change)
         return change
 
-    def _updateChange(self, change, event):
+    def _getNonMRRef(self, change_key, refresh=False, event=None):
+        change = self._change_cache.get(change_key)
+        if change:
+            if refresh:
+                self._change_cache.updateChangeWithRetry(
+                    change_key, change, lambda c: None)
+            return change
+        if not event:
+            self.log.error("Change %s not found in cache and no event",
+                           change_key)
+        project = self.source.getProject(change_key.project_name)
+        if change_key.change_type == 'Tag':
+            change = Tag(project)
+            tag = change_key.stable_id
+            change.tag = tag
+            change.ref = f'refs/tags/{tag}'
+        elif change_key.change_type == 'Branch':
+            branch = change_key.stable_id
+            change = Branch(project)
+            change.branch = branch
+            change.ref = f'refs/heads/{branch}'
+        else:
+            change = Ref(project)
+            change.ref = change_key.stable_id
+        change.oldrev = change_key.oldrev
+        change.newrev = change_key.newrev
+        change.url = self.getGitwebUrl(project, sha=change.newrev)
+        change.files = self.getPushedFileNames(event)
+        try:
+            self._change_cache.set(change_key, change)
+        except ConcurrentUpdateError:
+            change = self._change_cache.get(change_key)
+        return change
+
+    def _updateChange(self, change, event, mr):
         log = get_annotated_logger(self.log, event)
-        log.info("Updating change from Gitlab %s" % change)
-        change.mr = self.getMR(
-            change.project.name, change.number, event=event)
+        change.mr = mr
         change.ref = "refs/merge-requests/%s/head" % change.number
         change.branch = change.mr['target_branch']
-        change.patchset = change.mr['sha']
+        change.is_current_patchset = (change.mr['sha'] == change.patchset)
         change.commit_id = change.mr['diff_refs'].get('head_sha')
         change.owner = change.mr['author'].get('username')
         # Files changes are not part of the Merge Request data
@@ -553,17 +672,30 @@ class GitlabConnection(CachedBranchConnection):
         # Can be "can_be_merged"
         change.merge_status = change.mr['merge_status']
         change.approved = change.mr['approved']
-        change.message = change.mr['description']
+        change.message = change.mr.get('description', "")
         change.labels = change.mr['labels']
         change.updated_at = int(dateutil.parser.parse(
             change.mr['updated_at']).timestamp())
         log.info("Updated change from Gitlab %s" % change)
-
-        if self.sched:
-            self.sched.onChangeUpdated(change, event)
-
         return change
 
+    def getPushedFileNames(self, event):
+        if event is None:
+            return None
+        if not hasattr(event, 'total_commits_count'):
+            return None
+        if event.total_commits_count > 20:
+            # Gitlab only includes files for the most recent 20
+            # commits.  If we have more than that, set files to None
+            # so that mergers will perform lookups if necessary, and
+            # the scheduler will assume a reconfiguration is required.
+            return None
+        files = set()
+        for c in event.commits:
+            for f in c.get('added') + c.get('modified') + c.get('removed'):
+                files.add(f)
+        return list(files)
+
     def canMerge(self, change, allow_needs, event=None):
         log = get_annotated_logger(self.log, event)
         can_merge = True if change.merge_status == "can_be_merged" else False
@@ -609,11 +741,22 @@ class GitlabConnection(CachedBranchConnection):
         # Will be done in a folloup commit
         return []
 
-    def mergeMR(self, project_name, number, event=None):
+    def mergeMR(self, project_name, number, method='merge', event=None):
         log = get_annotated_logger(self.log, event)
-        self.gl_client.merge_mr(project_name, number, zuul_event_id=event)
+        self.gl_client.merge_mr(
+            project_name, number, method, zuul_event_id=event)
         log.info("Merged MR %s#%s", project_name, number)
 
+    def updateMRLabels(self, project_name, mr_number, labels, unlabels,
+                       zuul_event_id=None):
+        log = get_annotated_logger(self.log, zuul_event_id)
+        self.gl_client.update_mr(
+            project_name, mr_number, zuul_event_id=zuul_event_id,
+            add_labels=','.join(labels),
+            remove_labels=','.join(unlabels))
+        log.debug("Added labels %s to, and removed labels %s from %s#%s",
+                  labels, unlabels, project_name, mr_number)
+
 
 class GitlabWebController(BaseWebController):
 
