@@ -1,4 +1,5 @@
 # Copyright 2019 Red Hat, Inc.
+# Copyright 2022 Acme Gating, LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -25,6 +26,9 @@ class MergeRequest(Change):
         self.updated_at = None
         self.approved = None
         self.labels = None
+        self.merge_status = None
+        self.mr = None
+        self.title = None
 
     def __repr__(self):
         r = ['<Change 0x%x' % id(self)]
@@ -46,6 +50,27 @@ class MergeRequest(Change):
             r.append('labels: %s' % ', '.join(self.labels))
         return ' '.join(r) + '>'
 
+    def serialize(self):
+        d = super().serialize()
+        d.update({
+            "updated_at": self.updated_at,
+            "approved": self.approved,
+            "labels": self.labels,
+            "merge_status": self.merge_status,
+            "mr": self.mr,
+            "title": self.title,
+        })
+        return d
+
+    def deserialize(self, data):
+        super().deserialize(data)
+        self.updated_at = data.get("updated_at")
+        self.approved = data.get("approved")
+        self.labels = data.get("labels")
+        self.merge_status = data.get("merge_status")
+        self.mr = data.get("mr")
+        self.title = data.get("title")
+
     def isUpdateOf(self, other):
         if (self.project == other.project and
             hasattr(other, 'number') and self.number == other.number and
@@ -62,8 +87,12 @@ class GitlabTriggerEvent(TriggerEvent):
         self.title = None
         self.action = None
         self.labels = []
+        self.unlabels = []
         self.change_number = None
+        self.merge_request_description_changed = None
         self.tag = None
+        self.commits = []
+        self.total_commits_count = 0
 
     def toDict(self):
         d = super().toDict()
@@ -71,8 +100,13 @@ class GitlabTriggerEvent(TriggerEvent):
         d["title"] = self.title
         d["action"] = self.action
         d["labels"] = self.labels
+        d["unlabels"] = self.unlabels
         d["change_number"] = self.change_number
+        d["merge_request_description_changed"] = \
+            self.merge_request_description_changed
         d["tag"] = self.tag
+        d["commits"] = self.commits
+        d["total_commits_count"] = self.total_commits_count
         return d
 
     def updateFromDict(self, d):
@@ -81,8 +115,14 @@ class GitlabTriggerEvent(TriggerEvent):
         self.title = d["title"]
         self.action = d["action"]
         self.labels = d["labels"]
+        self.unlabels = d.get("unlabels", [])
         self.change_number = d["change_number"]
+        self.merge_request_description_changed = \
+            d["merge_request_description_changed"]
         self.tag = d["tag"]
+        self.commits = d.get("commits", [])
+        self.total_commits_count = d.get("total_commits_count",
+                                         len(self.commits))
 
     def _repr(self):
         r = [super(GitlabTriggerEvent, self)._repr()]
@@ -93,6 +133,8 @@ class GitlabTriggerEvent(TriggerEvent):
             r.append("mr:%s" % self.change_number)
         if self.labels:
             r.append("labels:%s" % ', '.join(self.labels))
+        if self.unlabels:
+            r.append("unlabels:%s" % ', '.join(self.unlabels))
         return ' '.join(r)
 
     def isPatchsetCreated(self):
@@ -100,11 +142,15 @@ class GitlabTriggerEvent(TriggerEvent):
             return self.action in ['opened', 'changed']
         return False
 
+    def isMessageChanged(self):
+        return bool(self.merge_request_description_changed)
+
 
 class GitlabEventFilter(EventFilter):
     def __init__(
             self, connection_name, trigger, types=None, actions=None,
-            comments=None, refs=None, labels=None, ignore_deletes=True):
+            comments=None, refs=None, labels=None, unlabels=None,
+            ignore_deletes=True):
         super().__init__(connection_name, trigger)
         self._types = types or []
         self.types = [re.compile(x) for x in self._types]
@@ -114,6 +160,7 @@ class GitlabEventFilter(EventFilter):
         self._refs = refs or []
         self.refs = [re.compile(x) for x in self._refs]
         self.labels = labels or []
+        self.unlabels = unlabels or []
         self.ignore_deletes = ignore_deletes
 
     def __repr__(self):
@@ -132,6 +179,8 @@ class GitlabEventFilter(EventFilter):
             ret += ' ignore_deletes: %s' % self.ignore_deletes
         if self.labels:
             ret += ' labels: %s' % ', '.join(self.labels)
+        if self.unlabels:
+            ret += ' unlabels: %s' % ', '.join(self.unlabels)
         ret += '>'
 
         return ret
@@ -178,6 +227,10 @@ class GitlabEventFilter(EventFilter):
             if not set(event.labels).intersection(set(self.labels)):
                 return False
 
+        if self.unlabels:
+            if not set(event.unlabels).intersection(set(self.unlabels)):
+                return False
+
         return True
 
 
@@ -219,7 +272,7 @@ class GitlabRefFilter(RefFilter):
                 return False
 
         if self.labels:
-            if not set(change.labels).intersection(set(self.labels)):
+            if not set(self.labels).issubset(set(change.labels)):
                 return False
 
         return True
