@@ -22,12 +22,10 @@ EMPTY_GIT_REF = '0' * 40  # git sha of all zeros, used during creates/deletes
 class PullRequest(Change):
     def __init__(self, project):
         super(PullRequest, self).__init__(project)
-        self.project = None
         self.pr = None
         self.updated_at = None
         self.title = None
         self.score = 0
-        self.files = []
         self.tags = []
         self.status = None
 
@@ -53,6 +51,27 @@ class PullRequest(Change):
             r.append('state: open')
         return ' '.join(r) + '>'
 
+    def serialize(self):
+        d = super().serialize()
+        d.update({
+            "pr": self.pr,
+            "updated_at": self.updated_at,
+            "title": self.title,
+            "score": self.score,
+            "tags": self.tags,
+            "status": self.status,
+        })
+        return d
+
+    def deserialize(self, data):
+        super().deserialize(data)
+        self.pr = data.get("pr")
+        self.updated_at = data.get("updated_at")
+        self.title = data.get("title")
+        self.score = data.get("score", 0)
+        self.tags = data.get("tags", [])
+        self.status = data.get("status")
+
     def isUpdateOf(self, other):
         if (self.project == other.project and
             hasattr(other, 'number') and self.number == other.number and
@@ -68,6 +87,7 @@ class PagureTriggerEvent(TriggerEvent):
         self.trigger_name = 'pagure'
         self.title = None
         self.action = None
+        self.initial_comment_changed = None
         self.status = None
         self.tags = []
         self.tag = None
@@ -77,6 +97,7 @@ class PagureTriggerEvent(TriggerEvent):
         d["trigger_name"] = self.trigger_name
         d["title"] = self.title
         d["action"] = self.action
+        d["initial_comment_changed"] = self.initial_comment_changed
         d["status"] = self.status
         d["tags"] = list(self.tags)
         d["tag"] = self.tag
@@ -87,6 +108,7 @@ class PagureTriggerEvent(TriggerEvent):
         self.trigger_name = d.get("trigger_name", "pagure")
         self.title = d.get("title")
         self.action = d.get("action")
+        self.initial_comment_changed = d.get("initial_comment_changed")
         self.status = d.get("status")
         self.tags = d.get("tags", [])
         self.tag = d.get("tag")
@@ -109,6 +131,9 @@ class PagureTriggerEvent(TriggerEvent):
             return self.action in ['opened', 'changed']
         return False
 
+    def isMessageChanged(self):
+        return bool(self.initial_comment_changed)
+
 
 class PagureEventFilter(EventFilter):
     def __init__(self, connection_name, trigger, types=[], refs=[],
