@@ -14,16 +14,18 @@
 
 import logging
 import time
+from urllib.parse import quote_plus
 
 import alembic
 import alembic.command
 import alembic.config
+import alembic.migration
 import sqlalchemy as sa
-from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import orm
 import sqlalchemy.pool
 
 from zuul.connection import BaseConnection
+from zuul.zk.locks import CONNECTION_LOCK_ROOT, locked, SessionAwareLock
 
 BUILDSET_TABLE = 'zuul_buildset'
 BUILD_TABLE = 'zuul_build'
@@ -59,9 +61,12 @@ class DatabaseSession(object):
 
     def getBuilds(self, tenant=None, project=None, pipeline=None,
                   change=None, branch=None, patchset=None, ref=None,
-                  newrev=None, event_id=None, uuid=None, job_name=None,
-                  voting=None, nodeset=None, result=None, provides=None,
-                  final=None, held=None, complete=None, limit=50, offset=0):
+                  newrev=None, event_id=None, event_timestamp=None,
+                  first_build_start_time=None, last_build_end_time=None,
+                  uuid=None, job_name=None, voting=None, nodeset=None,
+                  result=None, provides=None, final=None, held=None,
+                  complete=None, sort_by_buildset=False, limit=50,
+                  offset=0, idx_min=None, idx_max=None):
 
         build_table = self.connection.zuul_build_table
         buildset_table = self.connection.zuul_buildset_table
@@ -98,6 +103,12 @@ class DatabaseSession(object):
         q = self.listFilter(q, buildset_table.c.ref, ref)
         q = self.listFilter(q, buildset_table.c.newrev, newrev)
         q = self.listFilter(q, buildset_table.c.event_id, event_id)
+        q = self.listFilter(
+            q, buildset_table.c.event_timestamp, event_timestamp)
+        q = self.listFilter(
+            q, buildset_table.c.first_build_start_time, first_build_start_time)
+        q = self.listFilter(
+            q, buildset_table.c.last_build_end_time, last_build_end_time)
         q = self.listFilter(q, build_table.c.uuid, uuid)
         q = self.listFilter(q, build_table.c.job_name, job_name)
         q = self.listFilter(q, build_table.c.voting, voting)
@@ -110,10 +121,19 @@ class DatabaseSession(object):
             q = q.filter(build_table.c.result == None)  # noqa
         q = self.listFilter(q, provides_table.c.name, provides)
         q = self.listFilter(q, build_table.c.held, held)
+        if idx_min:
+            q = q.filter(build_table.c.id >= idx_min)
+        if idx_max:
+            q = q.filter(build_table.c.id <= idx_max)
 
-        q = q.order_by(build_table.c.id.desc()).\
-            limit(limit).\
-            offset(offset)
+        if sort_by_buildset:
+            # If we don't need the builds to be strictly ordered, this
+            # query can be much faster as it may avoid the use of a
+            # temporary table.
+            q = q.order_by(buildset_table.c.id.desc())
+        else:
+            q = q.order_by(build_table.c.id.desc())
+        q = q.limit(limit).offset(offset)
 
         try:
             return q.all()
@@ -153,7 +173,7 @@ class DatabaseSession(object):
     def getBuildsets(self, tenant=None, project=None, pipeline=None,
                      change=None, branch=None, patchset=None, ref=None,
                      newrev=None, uuid=None, result=None, complete=None,
-                     limit=50, offset=0):
+                     limit=50, offset=0, idx_min=None, idx_max=None):
 
         buildset_table = self.connection.zuul_buildset_table
 
@@ -172,6 +192,11 @@ class DatabaseSession(object):
         q = self.listFilter(q, buildset_table.c.newrev, newrev)
         q = self.listFilter(q, buildset_table.c.uuid, uuid)
         q = self.listFilter(q, buildset_table.c.result, result)
+        if idx_min:
+            q = q.filter(buildset_table.c.id >= idx_min)
+        if idx_max:
+            q = q.filter(buildset_table.c.id <= idx_max)
+
         if complete is True:
             q = q.filter(buildset_table.c.result != None)  # noqa
         elif complete is False:
@@ -237,7 +262,8 @@ class SQLConnection(BaseConnection):
             self.engine = sa.create_engine(
                 self.dburi,
                 poolclass=sqlalchemy.pool.QueuePool,
-                pool_recycle=self.connection_config.get('pool_recycle', 1))
+                pool_recycle=self.connection_config.get('pool_recycle', 1),
+                future=True)
 
             # If we want the objects returned from query() to be
             # usable outside of the session, we need to expunge them
@@ -247,7 +273,8 @@ class SQLConnection(BaseConnection):
             # objects when it does so.
             self.session_factory = orm.sessionmaker(bind=self.engine,
                                                     expire_on_commit=False,
-                                                    autoflush=False)
+                                                    autoflush=False,
+                                                    future=True)
             self.session = orm.scoped_session(self.session_factory)
         except sa.exc.NoSuchModuleError:
             self.log.error(
@@ -257,7 +284,7 @@ class SQLConnection(BaseConnection):
     def getSession(self):
         return DatabaseSession(self)
 
-    def _migrate(self):
+    def _migrate(self, revision='head'):
         """Perform the alembic migrations for this connection"""
         with self.engine.begin() as conn:
             context = alembic.migration.MigrationContext.configure(conn)
@@ -268,7 +295,8 @@ class SQLConnection(BaseConnection):
             config.set_main_option("script_location",
                                    "zuul:driver/sql/alembic")
             config.set_main_option("sqlalchemy.url",
-                                   self.connection_config.get('dburi'))
+                                   self.connection_config.get('dburi').
+                                   replace('%', '%%'))
 
             # Alembic lets us add arbitrary data in the tag argument. We can
             # leverage that to tell the upgrade scripts about the table prefix.
@@ -276,14 +304,20 @@ class SQLConnection(BaseConnection):
 
             if current_rev is None and not self.force_migrations:
                 self.metadata.create_all(self.engine)
-                alembic.command.stamp(config, "head", tag=tag)
+                alembic.command.stamp(config, revision, tag=tag)
             else:
-                alembic.command.upgrade(config, 'head', tag=tag)
+                alembic.command.upgrade(config, revision, tag=tag)
 
-    def onLoad(self):
+    def onLoad(self, zk_client, component_registry=None):
+        safe_connection = quote_plus(self.connection_name)
         while True:
             try:
-                self._migrate()
+                with locked(
+                    SessionAwareLock(
+                        zk_client.client,
+                        f"{CONNECTION_LOCK_ROOT}/{safe_connection}/migration")
+                ):
+                    self._migrate()
                 break
             except sa.exc.OperationalError:
                 self.log.error(
@@ -292,7 +326,7 @@ class SQLConnection(BaseConnection):
             time.sleep(10)
 
     def _setup_models(self):
-        Base = declarative_base(metadata=self.metadata)
+        Base = orm.declarative_base(metadata=self.metadata)
 
         class BuildSetModel(Base):
             __tablename__ = self.table_prefix + BUILDSET_TABLE
@@ -312,6 +346,9 @@ class SQLConnection(BaseConnection):
             branch = sa.Column(sa.String(255))
             uuid = sa.Column(sa.String(36))
             event_id = sa.Column(sa.String(255), nullable=True)
+            event_timestamp = sa.Column(sa.DateTime, nullable=True)
+            first_build_start_time = sa.Column(sa.DateTime, nullable=True)
+            last_build_end_time = sa.Column(sa.DateTime, nullable=True)
 
             sa.Index(self.table_prefix + 'project_pipeline_idx',
                      project, pipeline)
@@ -352,6 +389,15 @@ class SQLConnection(BaseConnection):
             sa.Index(self.table_prefix + 'uuid_buildset_id_idx',
                      uuid, buildset_id)
 
+            @property
+            def duration(self):
+                if self.start_time and self.end_time:
+                    return max(0.0,
+                               (self.end_time -
+                                self.start_time).total_seconds())
+                else:
+                    return None
+
             def createArtifact(self, *args, **kw):
                 session = orm.session.Session.object_session(self)
                 # SQLAlchemy reserves the 'metadata' attribute on
