@@ -17,7 +17,7 @@ import os
 from zuul.lib import strings
 
 
-def construct_build_params(uuid, sched, nodeset, job, item, pipeline,
+def construct_build_params(uuid, connections, job, item, pipeline,
                            dependent_changes=[], merger_items=[],
                            redact_secrets_and_keys=True):
     """Returns a list of all the parameters needed to build a job.
@@ -48,17 +48,11 @@ def construct_build_params(uuid, sched, nodeset, job, item, pipeline,
         pipeline=pipeline.name,
         post_review=pipeline.post_review,
         job=job.name,
-        voting=job.voting,
         project=project,
         tenant=tenant.name,
-        timeout=job.timeout,
         event_id=item.event.zuul_event_id if item.event else None,
         jobtags=sorted(job.tags),
-        _inheritance_path=list(job.inheritance_path))
-    if job.artifact_data:
-        zuul_params['artifacts'] = job.artifact_data
-    if job.override_checkout:
-        zuul_params['override_checkout'] = job.override_checkout
+    )
     if hasattr(item.change, 'branch'):
         zuul_params['branch'] = item.change.branch
     if hasattr(item.change, 'tag'):
@@ -79,61 +73,22 @@ def construct_build_params(uuid, sched, nodeset, job, item, pipeline,
         zuul_params['newrev'] = item.change.newrev
     zuul_params['projects'] = {}  # Set below
     zuul_params['items'] = dependent_changes
-    zuul_params['child_jobs'] = list(item.job_graph.getDirectDependentJobs(
-        job.name))
+    zuul_params['child_jobs'] = list(item.current_build_set.job_graph.
+                                     getDirectDependentJobs(job.name))
 
     params = dict()
-    params['job'] = job.name
-    params['timeout'] = job.timeout
-    params['post_timeout'] = job.post_timeout
+    params['job_ref'] = job.getPath()
     params['items'] = merger_items
     params['projects'] = []
     if hasattr(item.change, 'branch'):
         params['branch'] = item.change.branch
     else:
         params['branch'] = None
-    params['override_branch'] = job.override_branch
-    params['override_checkout'] = job.override_checkout
-    params['repo_state'] = item.current_build_set.repo_state
-    params['ansible_version'] = job.ansible_version
-    params['workspace_scheme'] = job.workspace_scheme
+    merge_rs = item.current_build_set.merge_repo_state
+    params['merge_repo_state_ref'] = merge_rs and merge_rs.getPath()
+    extra_rs = item.current_build_set.extra_repo_state
+    params['extra_repo_state_ref'] = extra_rs and extra_rs.getPath()
 
-    def make_playbook(playbook):
-        d = playbook.toDict(redact_secrets=redact_secrets_and_keys)
-        for role in d['roles']:
-            if role['type'] != 'zuul':
-                continue
-            project_metadata = item.job_graph.getProjectMetadata(
-                role['project_canonical_name'])
-            if project_metadata:
-                role['project_default_branch'] = \
-                    project_metadata.default_branch
-            else:
-                role['project_default_branch'] = 'master'
-            role_trusted, role_project = item.pipeline.tenant.getProject(
-                role['project_canonical_name'])
-            role_connection = role_project.source.connection
-            role['connection'] = role_connection.connection_name
-            role['project'] = role_project.name
-        return d
-
-    if job.name != 'noop':
-        params['playbooks'] = [make_playbook(x) for x in job.run]
-        params['pre_playbooks'] = [make_playbook(x) for x in job.pre_run]
-        params['post_playbooks'] = [make_playbook(x) for x in job.post_run]
-        params['cleanup_playbooks'] = [make_playbook(x)
-                                       for x in job.cleanup_run]
-
-    # TODO(corvus): Remove nodes and groups since they're included in
-    # nodeset
-    nodes = []
-    for node in nodeset.getNodes():
-        n = node.toDict()
-        n.update(dict(name=node.name, label=node.label))
-        nodes.append(n)
-    params['nodes'] = nodes
-    params['groups'] = [group.toDict() for group in nodeset.getGroups()]
-    params["nodeset"] = nodeset.toDict()
     params['ssh_keys'] = []
     if pipeline.post_review:
         if redact_secrets_and_keys:
@@ -142,19 +97,14 @@ def construct_build_params(uuid, sched, nodeset, job, item, pipeline,
             params['ssh_keys'].append(dict(
                 connection_name=item.change.project.connection_name,
                 project_name=item.change.project.name))
-    params['vars'] = job.combined_variables
-    params['extra_vars'] = job.extra_variables
-    params['host_vars'] = job.host_variables
-    params['group_vars'] = job.group_variables
-    params['secret_vars'] = job.secret_parent_data
     params['zuul'] = zuul_params
     projects = set()
     required_projects = set()
 
     def make_project_dict(project, override_branch=None,
                           override_checkout=None):
-        project_metadata = item.job_graph.getProjectMetadata(
-            project.canonical_name)
+        project_metadata = item.current_build_set.job_graph.\
+            getProjectMetadata(project.canonical_name)
         if project_metadata:
             project_default_branch = project_metadata.default_branch
         else:
@@ -181,11 +131,18 @@ def construct_build_params(uuid, sched, nodeset, job, item, pipeline,
             projects.add(project)
             required_projects.add(project)
     for change in dependent_changes:
-        # We have to find the project this way because it may not
-        # be registered in the tenant (ie, a foreign project).
-        source = sched.connections.getSourceByCanonicalHostname(
-            change['project']['canonical_hostname'])
-        project = source.getProject(change['project']['name'])
+        try:
+            (_, project) = item.pipeline.tenant.getProject(
+                change['project']['canonical_name'])
+            if not project:
+                raise KeyError()
+        except Exception:
+            # We have to find the project this way because it may not
+            # be registered in the tenant (ie, a foreign project).
+            source = connections.getSourceByCanonicalHostname(
+                change['project']['canonical_hostname'])
+            project = source.getProject(change['project']['name'])
+
         if project not in projects:
             params['projects'].append(make_project_dict(project))
             projects.add(project)
@@ -208,3 +165,18 @@ def construct_build_params(uuid, sched, nodeset, job, item, pipeline,
     if item.event:
         params['zuul_event_id'] = item.event.zuul_event_id
     return params
+
+
+def zuul_params_from_job(job):
+    zuul_params = {
+        "job": job.name,
+        "voting": job.voting,
+        "timeout": job.timeout,
+        "jobtags": sorted(job.tags),
+        "_inheritance_path": list(job.inheritance_path),
+    }
+    if job.artifact_data:
+        zuul_params['artifacts'] = job.artifact_data
+    if job.override_checkout:
+        zuul_params['override_checkout'] = job.override_checkout
+    return zuul_params
