@@ -49,8 +49,6 @@ class ManagedAnsible:
         self.install_root = self._ansible_roots[-1]
 
     def ensure_ansible(self, upgrade=False):
-        self._ensure_venv()
-
         self.log.info('Installing ansible %s, requirements: %s, '
                       'extra packages: %s',
                       self.version, self._requirements, self.extra_packages)
@@ -76,7 +74,7 @@ class ManagedAnsible:
                                              p.stderr.decode()))
         self.log.debug('Successfully installed packages %s', requirements)
 
-    def _ensure_venv(self):
+    def ensure_venv(self):
         if self.python_path:
             self.log.debug(
                 'Virtual environment %s already existing', self.venv_path)
@@ -192,6 +190,13 @@ class AnsibleManager:
         self.default_version = default_version
 
     def install(self, upgrade=False):
+        # virtualenv sets up a shared directory of pip seed packages per
+        # python version. If we run virtualenv in parallel we can have one
+        # create the dirs but not be finished filling them with content, a
+        # second notice the dir is there so it just goes on with what it's
+        # done, and thus races leaving us with virtualenvs minus pip.
+        for a in self._supported_versions.values():
+            a.ensure_venv()
         # Note: With higher number of threads pip seems to have some race
         # leading to occasional failures during setup of all ansible
         # environments. Thus we limit the number of workers to reduce the risk
@@ -226,7 +231,7 @@ class AnsibleManager:
         return result
 
     def _validate_packages(self, version):
-        result = True
+        result = False
         try:
             extra_packages = self._getAnsible(version).extra_packages
             python_package_check = \
@@ -235,11 +240,22 @@ class AnsibleManager:
 
             command = [self.getAnsibleCommand(version, 'python'),
                        '-c', python_package_check]
-            subprocess.run(command, check=True)
+            ret = subprocess.run(command,
+                                 stdout=subprocess.PIPE,
+                                 stderr=subprocess.STDOUT)
+            # We check manually so that we can log the stdout and stderr
+            # properly which aren't going to be available if we have
+            # subprocess.run() check and raise.
+            if ret.returncode != 0:
+                self.log.error(
+                    'Ansible version %s installation is missing packages' %
+                    version)
+                self.log.debug("Ansible package check output: %s", ret.stdout)
+            else:
+                result = True
         except Exception:
-            result = False
             self.log.exception(
-                'Ansible version %s installation is missing packages' %
+                'Exception checking Ansible version %s packages' %
                 version)
         return result
 
@@ -316,6 +332,11 @@ class AnsibleManager:
 
     def copyAnsibleFiles(self):
         if os.path.exists(self.zuul_ansible_dir):
+            # Ensure we can delete the files by setting writtable mode
+            for dirpath, dirnames, filenames in os.walk(self.zuul_ansible_dir):
+                os.chmod(dirpath, 0o755)
+                for filename in filenames:
+                    os.chmod(os.path.join(dirpath, filename), 0o600)
             shutil.rmtree(self.zuul_ansible_dir)
 
         library_path = os.path.dirname(os.path.abspath(zuul.ansible.__file__))
