@@ -1,4 +1,5 @@
 # Copyright 2017 Red Hat, Inc.
+# Copyright 2021-2022 Acme Gating, LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -15,19 +16,24 @@
 import functools
 import logging
 import socket
+import ssl
 import threading
 from configparser import ConfigParser
 from typing import Optional
 
-import zuul.rpcclient
+from zuul.exceptions import StreamingError
 from zuul.lib import streamer_utils
-from zuul.lib.commandsocket import CommandSocket
+from zuul.lib import commandsocket
 from zuul.lib.config import get_default
+from zuul.lib.monitoring import MonitoringServer
+from zuul.version import get_version_string
 from zuul.zk import ZooKeeperClient
-from zuul.zk.components import FingerGatewayComponent
+from zuul.zk.components import COMPONENT_REGISTRY, FingerGatewayComponent
+from zuul.zk.executor import ExecutorApi
 
-
-COMMANDS = ['stop']
+COMMANDS = [
+    commandsocket.StopCommand,
+]
 
 
 class RequestHandler(streamer_utils.BaseFingerRequestHandler):
@@ -38,10 +44,10 @@ class RequestHandler(streamer_utils.BaseFingerRequestHandler):
     log = logging.getLogger("zuul.fingergw")
 
     def __init__(self, *args, **kwargs):
-        self.rpc = kwargs.pop('rpc')
+        self.fingergw = kwargs.pop('fingergw')
         super(RequestHandler, self).__init__(*args, **kwargs)
 
-    def _fingerClient(self, server, port, build_uuid):
+    def _fingerClient(self, server, port, build_uuid, use_ssl):
         '''
         Open a finger connection and return all streaming results.
 
@@ -52,6 +58,15 @@ class RequestHandler(streamer_utils.BaseFingerRequestHandler):
         Both IPv4 and IPv6 are supported.
         '''
         with socket.create_connection((server, port), timeout=10) as s:
+            if use_ssl:
+                context = ssl.SSLContext(ssl.PROTOCOL_TLS)
+                context.verify_mode = ssl.CERT_REQUIRED
+                context.check_hostname = self.fingergw.tls_verify_hostnames
+                context.load_cert_chain(self.fingergw.tls_cert,
+                                        self.fingergw.tls_key)
+                context.load_verify_locations(self.fingergw.tls_ca)
+                s = context.wrap_socket(s, server_hostname=server)
+
             # timeout only on the connection, let recv() wait forever
             s.settimeout(None)
             msg = "%s\n" % build_uuid    # Must have a trailing newline!
@@ -72,7 +87,9 @@ class RequestHandler(streamer_utils.BaseFingerRequestHandler):
         port = None
         try:
             build_uuid = self.getCommand()
-            port_location = self.rpc.get_job_log_stream_address(build_uuid)
+            port_location = streamer_utils.getJobLogStreamAddress(
+                self.fingergw.executor_api,
+                build_uuid, source_zone=self.fingergw.zone)
 
             if not port_location:
                 msg = 'Invalid build UUID %s' % build_uuid
@@ -81,7 +98,10 @@ class RequestHandler(streamer_utils.BaseFingerRequestHandler):
 
             server = port_location['server']
             port = port_location['port']
-            self._fingerClient(server, port, build_uuid)
+            use_ssl = port_location.get('use_ssl', False)
+            self._fingerClient(server, port, build_uuid, use_ssl)
+        except StreamingError as e:
+            self.request.sendall(str(e).encode("utf-8"))
         except BrokenPipeError:   # Client disconnect
             return
         except Exception:
@@ -99,11 +119,12 @@ class FingerGateway(object):
 
     For each incoming finger request, a new thread is started that will
     be responsible for finding which Zuul executor is executing the
-    requested build (by asking Gearman), forwarding the request to that
+    requested build (by asking ZooKeeper), forwarding the request to that
     executor, and streaming the results back to our client.
     '''
 
     log = logging.getLogger("zuul.fingergw")
+    handler_class = RequestHandler
 
     def __init__(
         self,
@@ -115,36 +136,20 @@ class FingerGateway(object):
         Initialize the finger gateway.
 
         :param config: The parsed Zuul configuration.
-        :param tuple gearman: Gearman connection information. This should
-            include the server, port, SSL key, SSL cert, and SSL CA.
-        :param tuple address: The address and port to bind to for our gateway.
-        :param str user: The user to which we should drop privileges after
-            binding to our address.
         :param str command_socket: Path to the daemon command socket.
         :param str pid_file: Path to the daemon PID file.
         '''
 
-        gear_server = get_default(config, 'gearman', 'server')
-        gear_port = get_default(config, 'gearman', 'port', 4730)
-        gear_ssl_key = get_default(config, 'gearman', 'ssl_key')
-        gear_ssl_cert = get_default(config, 'gearman', 'ssl_cert')
-        gear_ssl_ca = get_default(config, 'gearman', 'ssl_ca')
-
-        self.gear_server = gear_server
-        self.gear_port = gear_port
-        self.gear_ssl_key = gear_ssl_key
-        self.gear_ssl_cert = gear_ssl_cert
-        self.gear_ssl_ca = gear_ssl_ca
-
         host = get_default(config, 'fingergw', 'listen_address', '::')
-        port = int(get_default(config, 'fingergw', 'port', 79))
+        self.port = int(get_default(config, 'fingergw', 'port', 79))
+        self.public_port = int(get_default(
+            config, 'fingergw', 'public_port', self.port))
         user = get_default(config, 'fingergw', 'user', None)
 
-        self.address = (host, port)
+        self.address = (host, self.port)
         self.user = user
         self.pid_file = pid_file
 
-        self.rpc = None
         self.server = None
         self.server_thread = None
 
@@ -153,24 +158,52 @@ class FingerGateway(object):
         self.command_socket_path = command_socket
         self.command_socket = None
 
-        self.command_map = dict(
-            stop=self.stop,
-        )
+        self.tls_key = get_default(config, 'fingergw', 'tls_key')
+        self.tls_cert = get_default(config, 'fingergw', 'tls_cert')
+        self.tls_ca = get_default(config, 'fingergw', 'tls_ca')
+        self.tls_verify_hostnames = get_default(
+            config, 'fingergw', 'tls_verify_hostnames', default=True)
+        client_only = get_default(config, 'fingergw', 'tls_client_only',
+                                  default=False)
+        if (all([self.tls_key, self.tls_cert, self.tls_ca])
+            and not client_only):
+            self.tls_listen = True
+        else:
+            self.tls_listen = False
+
+        self.command_map = {
+            commandsocket.StopCommand.name: self.stop,
+        }
+
+        self.hostname = get_default(config, 'fingergw', 'hostname',
+                                    socket.getfqdn())
+        self.zone = get_default(config, 'fingergw', 'zone')
 
         self.zk_client = ZooKeeperClient.fromConfig(config)
         self.zk_client.connect()
-        self.hostname = socket.getfqdn()
         self.component_info = FingerGatewayComponent(
-            self.zk_client, self.hostname
+            self.zk_client, self.hostname, version=get_version_string()
         )
+        if self.zone is not None:
+            self.component_info.zone = self.zone
+            self.component_info.public_port = self.public_port
+        if self.tls_listen:
+            self.component_info.use_ssl = True
         self.component_info.register()
+        COMPONENT_REGISTRY.create(self.zk_client)
+
+        self.monitoring_server = MonitoringServer(config, 'fingergw',
+                                                  self.component_info)
+        self.monitoring_server.start()
+
+        self.executor_api = ExecutorApi(self.zk_client, use_cache=False)
 
     def _runCommand(self):
         while self.command_running:
             try:
-                command = self.command_socket.get().decode('utf8')
+                command, args = self.command_socket.get()
                 if command != '_stop':
-                    self.command_map[command]()
+                    self.command_map[command](*args)
                 else:
                     return
             except Exception:
@@ -184,24 +217,32 @@ class FingerGateway(object):
             raise
 
     def start(self):
-        self.rpc = zuul.rpcclient.RPCClient(
-            self.gear_server,
-            self.gear_port,
-            self.gear_ssl_key,
-            self.gear_ssl_cert,
-            self.gear_ssl_ca,
-            client_id='Zuul Finger Gateway')
+        kwargs = dict(
+            user=self.user,
+            pid_file=self.pid_file,
+        )
+        if self.tls_listen:
+            kwargs.update(dict(
+                server_ssl_ca=self.tls_ca,
+                server_ssl_cert=self.tls_cert,
+                server_ssl_key=self.tls_key,
+            ))
 
         self.server = streamer_utils.CustomThreadingTCPServer(
             self.address,
-            functools.partial(RequestHandler, rpc=self.rpc),
-            user=self.user,
-            pid_file=self.pid_file)
+            functools.partial(self.handler_class, fingergw=self),
+            **kwargs)
+
+        # Update port that we really use if we configured a port of 0
+        if self.public_port == 0:
+            self.public_port = self.server.socket.getsockname()[1]
+            self.component_info.public_port = self.public_port
 
         # Start the command processor after the server and privilege drop
         if self.command_socket_path:
             self.log.debug("Starting command processor")
-            self.command_socket = CommandSocket(self.command_socket_path)
+            self.command_socket = commandsocket.CommandSocket(
+                self.command_socket_path)
             self.command_socket.start()
             self.command_running = True
             self.command_thread = threading.Thread(
@@ -215,10 +256,12 @@ class FingerGateway(object):
         self.server_thread.daemon = True
         self.server_thread.start()
         self.component_info.state = self.component_info.RUNNING
+
         self.log.info("Finger gateway is started")
 
     def stop(self):
         self.component_info.state = self.component_info.STOPPED
+
         if self.server:
             try:
                 self.server.shutdown()
@@ -227,13 +270,6 @@ class FingerGateway(object):
             except Exception:
                 self.log.exception("Error stopping TCP server:")
 
-        if self.rpc:
-            try:
-                self.rpc.shutdown()
-                self.rpc = None
-            except Exception:
-                self.log.exception("Error stopping RCP client:")
-
         if self.command_socket:
             self.command_running = False
 
@@ -243,6 +279,7 @@ class FingerGateway(object):
                 self.log.exception("Error stopping command socket:")
 
         self.zk_client.disconnect()
+        self.monitoring_server.stop()
 
         self.log.info("Finger gateway is stopped")
 
@@ -254,3 +291,4 @@ class FingerGateway(object):
 
         if self.command_thread:
             self.command_thread.join()
+        self.monitoring_server.join()
