@@ -21,14 +21,14 @@ from yaml import YAMLError  # noqa: F401
 try:
     # Explicit type ignore to deal with provisional import failure
     # Details at https://github.com/python/mypy/issues/1153
-    from yaml import cyaml  # type: ignore
+    from yaml import cyaml
     import _yaml
     SafeLoader = cyaml.CSafeLoader
     SafeDumper = cyaml.CSafeDumper
     Mark = _yaml.Mark
 except ImportError:
-    SafeLoader = yaml.SafeLoader  # type: ignore
-    SafeDumper = yaml.SafeDumper  # type: ignore
+    SafeLoader = yaml.SafeLoader
+    SafeDumper = yaml.SafeDumper
     Mark = yaml.Mark
 
 
@@ -142,6 +142,11 @@ class AnsibleUnsafeDumper(yaml.SafeDumper):
     pass
 
 
+class AnsibleUnsafeDumperWithoutAliases(yaml.SafeDumper):
+    def ignore_aliases(self, data):
+        return True
+
+
 class AnsibleUnsafeLoader(yaml.SafeLoader):
     pass
 
@@ -153,7 +158,12 @@ AnsibleUnsafeLoader.add_constructor(AnsibleUnsafeStr.yaml_tag,
 
 
 def ansible_unsafe_dump(data, *args, **kwargs):
-    return yaml.dump(data, *args, Dumper=AnsibleUnsafeDumper, **kwargs)
+    ignore_aliases = kwargs.pop('ignore_aliases', False)
+    if ignore_aliases:
+        return yaml.dump(data, *args, Dumper=AnsibleUnsafeDumperWithoutAliases,
+                         **kwargs)
+    else:
+        return yaml.dump(data, *args, Dumper=AnsibleUnsafeDumper, **kwargs)
 
 
 def ansible_unsafe_load(stream, *args, **kwargs):
