@@ -18,10 +18,12 @@ class SerialPipelineManager(SharedQueuePipelineManager):
     """PipelineManager with shared queues and a window of 1"""
 
     changes_merge = False
+    type = 'serial'
 
     def constructChangeQueue(self, queue_name):
-        return model.ChangeQueue(
-            self.pipeline,
+        return model.ChangeQueue.new(
+            self.pipeline.manager.current_context,
+            pipeline=self.pipeline,
             window=1,
             window_floor=1,
             window_increase_type='none',
