@@ -23,6 +23,7 @@ import os
 import re
 import shutil
 import time
+from concurrent.futures.process import BrokenProcessPool
 
 import git
 import gitdb
@@ -68,10 +69,12 @@ def nullcontext():
 class Repo(object):
     commit_re = re.compile(r'^commit ([0-9a-f]{40})$')
     diff_re = re.compile(r'^@@ -\d+,\d \+(\d+),\d @@$')
+    retry_attempts = 3
+    retry_interval = 30
 
     def __init__(self, remote, local, email, username, speed_limit, speed_time,
                  sshkey=None, cache_path=None, logger=None, git_timeout=300,
-                 retry_attempts=3, retry_interval=30, zuul_event_id=None):
+                 zuul_event_id=None):
         if logger is None:
             self.log = logging.getLogger("zuul.Repo")
         else:
@@ -92,8 +95,6 @@ class Repo(object):
         self.username = username
         self.cache_path = cache_path
         self._initialized = False
-        self.retry_attempts = retry_attempts
-        self.retry_interval = retry_interval
         try:
             self._setup_known_hosts()
         except Exception:
@@ -440,6 +441,17 @@ class Repo(object):
         repo = self.createRepoObject(zuul_event_id)
         return repo.refs
 
+    def getPackedRefs(self, zuul_event_id=None):
+        repo = self.createRepoObject(zuul_event_id)
+        refs = repo.git.for_each_ref(
+            '--format=%(objectname) %(refname)'
+        )
+        for ref in refs.splitlines():
+            parts = ref.split(" ")
+            if len(parts) == 2:
+                commit, ref = parts
+                yield commit, ref
+
     def setRef(self, path, hexsha, repo=None, zuul_event_id=None):
         ref_log = get_annotated_logger(
             logging.getLogger("zuul.Repo.Ref"), zuul_event_id)
@@ -641,7 +653,12 @@ class Repo(object):
                 ret[fn] = None
         if dirs:
             for dn in dirs:
-                if dn not in tree:
+                try:
+                    sub_tree = tree[dn]
+                except KeyError:
+                    continue
+
+                if sub_tree.type != "tree":
                     continue
 
                 # Some people like to keep playbooks, etc. grouped
@@ -649,7 +666,7 @@ class Repo(object):
                 # directories of any .zuul.ignore files and prune them
                 # from the config read.
                 to_ignore = []
-                for blob in tree[dn].traverse():
+                for blob in sub_tree.traverse():
                     if blob.path.endswith(".zuul.ignore"):
                         to_ignore.append(os.path.split(blob.path)[0])
 
@@ -659,7 +676,7 @@ class Repo(object):
                             return True
                     return False
 
-                for blob in tree[dn].traverse():
+                for blob in sub_tree.traverse():
                     if not _ignored(blob) and blob.path.endswith(".yaml"):
                         ret[blob.path] = blob.data_stream.read().decode(
                             'utf-8')
@@ -778,7 +795,7 @@ class Merger(object):
             self.log = logging.getLogger("zuul.Merger")
         else:
             self.log = logger
-        self.repos = {}  # type: ignore
+        self.repos = {}
         self.working_root = working_root
         os.makedirs(working_root, exist_ok=True)
         self.connections = connections
@@ -875,8 +892,7 @@ class Merger(object):
                                 process_worker=process_worker)
 
     def updateRepo(self, connection_name, project_name, repo_state=None,
-                   zuul_event_id=None,
-                   build=None, process_worker=None):
+                   zuul_event_id=None, build=None, process_worker=None):
         """Fetch from origin if needed
 
         If repo_state is None, then this will always git fetch.
@@ -930,25 +946,27 @@ class Merger(object):
             self._restoreRepoState(connection_name, project_name, repo,
                                    repo_state, zuul_event_id,
                                    process_worker=process_worker)
-        repo.checkout(branch, zuul_event_id=zuul_event_id)
+        return repo.checkout(branch, zuul_event_id=zuul_event_id)
 
     def _saveRepoState(self, connection_name, project_name, repo,
                        repo_state, recent, branches):
         projects = repo_state.setdefault(connection_name, {})
         project = projects.setdefault(project_name, {})
-        for ref in repo.getRefs():
-            if ref.path.startswith('refs/zuul/'):
+
+        for commit, ref in repo.getPackedRefs():
+            if ref.startswith('refs/zuul/'):
                 continue
-            if ref.path.startswith('refs/remotes/'):
+            if ref.startswith('refs/remotes/'):
                 continue
-            if ref.path.startswith('refs/heads/'):
-                branch = ref.path[len('refs/heads/'):]
+            if ref.startswith('refs/heads/'):
+                branch = ref[len('refs/heads/'):]
                 if branches is not None and branch not in branches:
                     continue
                 key = (connection_name, project_name, branch)
                 if key not in recent:
-                    recent[key] = ref.object
-            project[ref.path] = ref.object.hexsha
+                    recent[key] = commit
+
+            project[ref] = commit
 
     def _alterRepoState(self, connection_name, project_name,
                         repo_state, path, hexsha):
@@ -1041,6 +1059,8 @@ class Merger(object):
             try:
                 repo.reset(zuul_event_id=zuul_event_id,
                            process_worker=process_worker)
+            except BrokenProcessPool:
+                raise
             except Exception:
                 log.exception("Unable to reset repo %s" % repo)
                 return None, None
@@ -1068,7 +1088,7 @@ class Merger(object):
         if not commit:
             return None, None
         # Store this commit as the most recent for this project-branch
-        recent[key] = commit
+        recent[key] = commit.hexsha
 
         # Make sure to have a local ref that points to the  most recent
         # (intermediate) speculative state of a branch, so commits are not
@@ -1084,7 +1104,7 @@ class Merger(object):
 
     def mergeChanges(self, items, files=None, dirs=None, repo_state=None,
                      repo_locks=None, branches=None, zuul_event_id=None,
-                     process_worker=None):
+                     process_worker=None, errors=None):
         """Merge changes
 
         Call Merger.updateRepo() first.
@@ -1106,13 +1126,27 @@ class Merger(object):
                     item['connection'], item['project'])
             else:
                 lock = nullcontext()
+            err_msg = (
+                f"Error merging {item['connection']}/{item['project']} "
+                f"for {item['number']},{item['patchset']}"
+            )
             with lock:
                 log.debug("Merging for change %s,%s" %
                           (item["number"], item["patchset"]))
-                orig_commit, commit = self._mergeItem(
-                    item, recent, repo_state, zuul_event_id, branches=branches,
-                    process_worker=process_worker)
+                try:
+                    orig_commit, commit = self._mergeItem(
+                        item, recent, repo_state, zuul_event_id,
+                        branches=branches,
+                        process_worker=process_worker)
+                except BrokenProcessPool:
+                    raise
+                except Exception:
+                    self.log.exception("Error merging item %s", item)
+                    if errors is not None:
+                        errors.append(err_msg)
                 if not commit:
+                    if errors is not None:
+                        errors.append(err_msg)
                     return None
                 if files or dirs:
                     repo = self.getRepo(item['connection'], item['project'])
@@ -1122,10 +1156,7 @@ class Merger(object):
                         project=item['project'],
                         branch=item['branch'],
                         files=repo_files))
-        ret_recent = {}
-        for k, v in recent.items():
-            ret_recent[k] = v.hexsha
-        return commit.hexsha, read_files, repo_state, ret_recent, orig_commit
+        return commit.hexsha, read_files, repo_state, recent, orig_commit
 
     def setRepoState(self, connection_name, project_name, repo_state,
                      zuul_event_id=None, process_worker=None):
