@@ -14,7 +14,7 @@ import time
 from abc import ABCMeta
 from configparser import ConfigParser
 from threading import Thread
-from typing import Optional, List, Callable
+from typing import List, Callable
 
 from kazoo.client import KazooClient
 from kazoo.exceptions import NoNodeError
@@ -23,6 +23,7 @@ from kazoo.protocol.states import KazooState
 
 from zuul.lib.config import get_default
 from zuul.zk.exceptions import NoClientException
+from zuul.zk.handler import PoolSequentialThreadingHandler
 
 
 class ZooKeeperClient(object):
@@ -33,12 +34,12 @@ class ZooKeeperClient(object):
 
     def __init__(
         self,
-        hosts: str,
-        read_only: bool = False,
-        timeout: float = 10.0,
-        tls_cert: Optional[str] = None,
-        tls_key: Optional[str] = None,
-        tls_ca: Optional[str] = None,
+        hosts,
+        read_only=False,
+        timeout=10.0,
+        tls_cert=None,
+        tls_key=None,
+        tls_ca=None,
     ):
         """
         Initialize the ZooKeeper base client object.
@@ -60,7 +61,15 @@ class ZooKeeperClient(object):
         self.tls_ca = tls_ca
         self.was_lost = False
 
-        self.client: Optional[KazooClient] = None
+        self.client = None
+
+        # Verify that we can read the cert files (Kazoo doesn't
+        # provide useful error messages).
+        for fn in (tls_cert, tls_key, tls_ca):
+            if fn:
+                with open(fn):
+                    pass
+
         self._last_retry_log: int = 0
         self.on_connect_listeners: List[Callable[[], None]] = []
         self.on_disconnect_listeners: List[Callable[[], None]] = []
@@ -118,6 +127,7 @@ class ZooKeeperClient(object):
                 hosts=self.hosts,
                 read_only=self.read_only,
                 timeout=self.timeout,
+                handler=PoolSequentialThreadingHandler(),
             )
             if self.tls_key:
                 args['use_ssl'] = True
