@@ -11,19 +11,42 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import sys
 import json
 import logging
+import threading
 from collections import defaultdict
 
 from kazoo.exceptions import NoNodeError
 from kazoo.protocol.states import EventType
 
 from zuul.zk import ZooKeeperBase
+from zuul.model_api import MODEL_API
 
 
 COMPONENTS_ROOT = "/zuul/components"
 
 
+class GlobalRegistry:
+    def __init__(self):
+        self.registry = None
+
+    def create(self, zk_client):
+        if not self.registry:
+            self.registry = ComponentRegistry(zk_client)
+        return self.registry
+
+    def clearRegistry(self):
+        self.registry = None
+
+    @property
+    def model_api(self):
+        return self.registry.model_api
+
+
+COMPONENT_REGISTRY = GlobalRegistry()
+
+
 class BaseComponent(ZooKeeperBase):
     """
     Read/write component object.
@@ -43,6 +66,7 @@ class BaseComponent(ZooKeeperBase):
     """
 
     # Component states
+    INITIALIZING = "initializing"
     RUNNING = "running"
     PAUSED = "paused"
     STOPPED = "stopped"
@@ -50,18 +74,21 @@ class BaseComponent(ZooKeeperBase):
     log = logging.getLogger("zuul.Component")
     kind = "base"
 
-    def __init__(self, client, hostname):
+    def __init__(self, client, hostname, version=None):
         # Ensure that the content is available before setting any other
         # attribute, because our __setattr__ implementation is relying on it.
         self.__dict__["content"] = {
             "hostname": hostname,
             "state": self.STOPPED,
             "kind": self.kind,
+            "version": version,
+            "model_api": 0,
         }
         super().__init__(client)
 
         self.path = None
         self._zstat = None
+        self.register_lock = threading.Lock()
 
     def __getattr__(self, name):
         try:
@@ -78,36 +105,39 @@ class BaseComponent(ZooKeeperBase):
         # Set the value in the local content dict
         self.content[name] = value
 
-        if not self.path:
-            self.log.error(
-                "Path is not set on this component, did you forget to call "
-                "register()?"
-            )
-            return
+        with self.register_lock:
+            if not self.path:
+                self.log.error(
+                    "Path is not set on this component, did you forget "
+                    "to call register()?"
+                )
+                return
 
-        # Update the ZooKeeper node
-        content = json.dumps(self.content).encode("utf-8")
-        try:
-            zstat = self.kazoo_client.set(
-                self.path, content, version=self._zstat.version
-            )
-            self._zstat = zstat
-        except NoNodeError:
-            self.log.error("Could not update %s in ZooKeeper", self)
+            # Update the ZooKeeper node
+            content = json.dumps(self.content, sort_keys=True).encode("utf-8")
+            try:
+                zstat = self.kazoo_client.set(
+                    self.path, content, version=self._zstat.version
+                )
+                self._zstat = zstat
+            except NoNodeError:
+                self.log.error("Could not update %s in ZooKeeper", self)
 
-    def register(self):
-        path = "/".join([COMPONENTS_ROOT, self.kind, self.hostname])
-        self.log.info("Registering component in ZooKeeper %s", path)
-        self.path, self._zstat = self.kazoo_client.create(
-            path,
-            json.dumps(self.content).encode("utf-8"),
-            makepath=True,
-            ephemeral=True,
-            sequence=True,
-            # Also return the zstat, which is necessary to successfully update
-            # the component.
-            include_data=True,
-        )
+    def register(self, model_api=MODEL_API):
+        self.content['model_api'] = model_api
+        with self.register_lock:
+            path = "/".join([COMPONENTS_ROOT, self.kind, self.hostname])
+            self.log.info("Registering component in ZooKeeper %s", path)
+            self.path, self._zstat = self.kazoo_client.create(
+                path,
+                json.dumps(self.content, sort_keys=True).encode("utf-8"),
+                makepath=True,
+                ephemeral=True,
+                sequence=True,
+                # Also return the zstat, which is necessary to successfully
+                # update the component.
+                include_data=True,
+            )
 
     def _onReconnect(self):
         self.register()
@@ -132,8 +162,8 @@ class SchedulerComponent(BaseComponent):
 class ExecutorComponent(BaseComponent):
     kind = "executor"
 
-    def __init__(self, client, hostname):
-        super().__init__(client, hostname)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         self.initial_state = {
             "accepting_work": False,
             "zone": None,
@@ -150,6 +180,15 @@ class MergerComponent(BaseComponent):
 class FingerGatewayComponent(BaseComponent):
     kind = "fingergw"
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.initial_state = {
+            "zone": None,
+            "public_port": None,
+            "use_ssl": False,
+        }
+        self.content.update(self.initial_state)
+
 
 class WebComponent(BaseComponent):
     kind = "web"
@@ -191,6 +230,9 @@ class ComponentRegistry(ZooKeeperBase):
         # kind -> hostname -> component
         self._cached_components = defaultdict(dict)
 
+        self.model_api = None
+        # Have we initialized enough to trust the model_api
+        self._init = False
         # If we are already connected when the class is instantiated, directly
         # call the onConnect callback.
         if self.client.connected:
@@ -207,9 +249,9 @@ class ComponentRegistry(ZooKeeperBase):
             root = self._getComponentRoot(kind)
             self.kazoo_client.ensure_path(root)
             self.kazoo_client.ChildrenWatch(
-                root, self._makeComponentRootWatcher(kind),
-                send_event=True
-            )
+                root, self._makeComponentRootWatcher(kind))
+        self._init = True
+        self._updateMinimumModelApi()
 
     def _makeComponentRootWatcher(self, kind):
         def watch(children, event=None):
@@ -223,9 +265,7 @@ class ComponentRegistry(ZooKeeperBase):
                 self.log.info("Noticed new %s component %s", kind, hostname)
                 root = self._getComponentPath(kind, hostname)
                 self.kazoo_client.DataWatch(
-                    root, self._makeComponentWatcher(kind, hostname),
-                    send_event=True
-                )
+                    root, self._makeComponentWatcher(kind, hostname))
 
     def _makeComponentWatcher(self, kind, hostname):
         def watch(data, stat, event=None):
@@ -240,10 +280,8 @@ class ComponentRegistry(ZooKeeperBase):
         self.log.debug(
             "Registry %s got event %s for %s %s",
             self, etype, kind, hostname)
-        if etype in (None, EventType.CHANGED, EventType.CREATED):
-            # Ignore events without data
-            if not data:
-                return
+        if (etype in (None, EventType.CHANGED, EventType.CREATED) and
+            data is not None):
 
             # Perform an in-place update of the cached component (if any)
             component = self._cached_components.get(kind, {}).get(hostname)
@@ -271,7 +309,8 @@ class ComponentRegistry(ZooKeeperBase):
                 component._zstat = stat
 
             self._cached_components[kind][hostname] = component
-        elif etype == EventType.DELETED:
+            self._updateMinimumModelApi()
+        elif (etype == EventType.DELETED or data is None):
             self.log.info(
                 "Noticed %s component %s disappeared",
                 kind, hostname)
@@ -280,6 +319,7 @@ class ComponentRegistry(ZooKeeperBase):
             except KeyError:
                 # If it's already gone, don't care
                 pass
+            self._updateMinimumModelApi()
             # Return False to stop the datawatch
             return False
 
@@ -299,3 +339,53 @@ class ComponentRegistry(ZooKeeperBase):
 
         # Filter the cached components for the given kind
         return self._cached_components.get(kind, {}).values()
+
+    def getMinimumModelApi(self):
+        """Get the minimum model API version of all currently connected
+        components"""
+
+        # Start with our own version in case we're the only component
+        # and we haven't registered.
+        version = MODEL_API
+        for kind, components in self.all():
+            for component in components:
+                version = min(version, component.model_api)
+        return version
+
+    def _updateMinimumModelApi(self):
+        if not self._init:
+            return
+        version = self.getMinimumModelApi()
+        if version != self.model_api:
+            self.log.info(f"System minimum data model version {version}; "
+                          f"this component {MODEL_API}")
+        if self.model_api is None:
+            if version < MODEL_API:
+                self.log.info("The data model version of this component is "
+                              "newer than the rest of the system; this "
+                              "component will operate in compatability mode "
+                              "until the system is upgraded")
+            elif version > MODEL_API:
+                self.log.error("The data model version of this component is "
+                               "older than the rest of the system; "
+                               "exiting to prevent data corruption")
+                sys.exit(1)
+        else:
+            if version > self.model_api:
+                if version > MODEL_API:
+                    self.log.info("The data model version of this component "
+                                  "is older than other components in the "
+                                  "system, so other components will operate "
+                                  "in a compability mode; upgrade this "
+                                  "component as soon as possible to complete "
+                                  "the system upgrade")
+                elif version == MODEL_API:
+                    self.log.info("The rest of the system has been upgraded "
+                                  "to the data model version of this "
+                                  "component")
+            elif version < self.model_api:
+                self.log.error("A component with a data model version older "
+                               "than the rest of the system has been started; "
+                               "data corruption is very likely to occur.")
+                # Should we exit here as well?
+        self.model_api = version
