@@ -25,11 +25,11 @@ from contextlib import suppress
 
 from kazoo.exceptions import NoNodeError
 from kazoo.protocol.states import EventType
-from kazoo.recipe.election import Election
 
 from zuul import model
 from zuul.lib.collections import DefaultKeyDict
 from zuul.zk import ZooKeeperSimpleBase, sharding
+from zuul.zk.election import SessionAwareElection
 
 RESULT_EVENT_TYPE_MAP = {
     "BuildCompletedEvent": model.BuildCompletedEvent,
@@ -52,24 +52,38 @@ MANAGEMENT_EVENT_TYPE_MAP = {
 # /zuul/events/tenant   TENANT_ROOT
 #   /{tenant}           TENANT_NAME_ROOT
 #     /management       TENANT_MANAGEMENT_ROOT
+#       /queue          TENANT_MANAGEMENT_QUEUE
+#       /data           [side channel data]
 #     /trigger          TENANT_TRIGGER_ROOT
+#       /queue          TENANT_TRIGGER_QUEUE
+#       /data           [side channel data]
 #     /pipelines        PIPELINE_ROOT
 #       /{pipeline}     PIPELINE_NAME_ROOT
 #         /management   PIPELINE_MANAGEMENT_ROOT
+#           /queue      PIPELINE_MANAGEMENT_QUEUE
+#           /data       [side channel data]
 #         /trigger      PIPELINE_TRIGGER_ROOT
+#           /queue      PIPELINE_TRIGGER_QUEUE
+#           /data       [side channel data]
 #         /result       PIPELINE_RESULT_ROOT
+#           /queue      PIPELINE_RESULT_QUEUE
+#           /data       [side channel data]
 
 TENANT_ROOT = "/zuul/events/tenant"
 TENANT_NAME_ROOT = TENANT_ROOT + "/{tenant}"
 TENANT_MANAGEMENT_ROOT = TENANT_NAME_ROOT + "/management"
+TENANT_MANAGEMENT_QUEUE = TENANT_MANAGEMENT_ROOT + "/queue"
 TENANT_TRIGGER_ROOT = TENANT_NAME_ROOT + "/trigger"
+TENANT_TRIGGER_QUEUE = TENANT_TRIGGER_ROOT + "/queue"
 PIPELINE_ROOT = TENANT_NAME_ROOT + "/pipelines"
 PIPELINE_NAME_ROOT = PIPELINE_ROOT + "/{pipeline}"
 PIPELINE_MANAGEMENT_ROOT = PIPELINE_NAME_ROOT + "/management"
+PIPELINE_MANAGEMENT_QUEUE = PIPELINE_MANAGEMENT_ROOT + "/queue"
 PIPELINE_TRIGGER_ROOT = PIPELINE_NAME_ROOT + "/trigger"
+PIPELINE_TRIGGER_QUEUE = PIPELINE_TRIGGER_ROOT + "/queue"
 PIPELINE_RESULT_ROOT = PIPELINE_NAME_ROOT + "/result"
+PIPELINE_RESULT_QUEUE = PIPELINE_RESULT_ROOT + "/queue"
 
-EVENT_DATA_ROOT = "/zuul/events/data"
 CONNECTION_ROOT = "/zuul/events/connection"
 
 # This is the path to the serialized from of the event in ZK (along
@@ -111,8 +125,8 @@ class EventWatcher(ZooKeeperSimpleBase):
             if tenant_name in self.watched_tenants:
                 continue
 
-            for path in (TENANT_MANAGEMENT_ROOT,
-                         TENANT_TRIGGER_ROOT):
+            for path in (TENANT_MANAGEMENT_QUEUE,
+                         TENANT_TRIGGER_QUEUE):
                 path = path.format(tenant=tenant_name)
                 self.kazoo_client.ensure_path(path)
                 self.kazoo_client.ChildrenWatch(
@@ -131,9 +145,9 @@ class EventWatcher(ZooKeeperSimpleBase):
             if key in self.watched_pipelines:
                 continue
 
-            for path in (PIPELINE_MANAGEMENT_ROOT,
-                         PIPELINE_TRIGGER_ROOT,
-                         PIPELINE_RESULT_ROOT):
+            for path in (PIPELINE_MANAGEMENT_QUEUE,
+                         PIPELINE_TRIGGER_QUEUE,
+                         PIPELINE_RESULT_QUEUE):
                 path = path.format(tenant=tenant_name,
                                    pipeline=pipeline_name)
 
@@ -182,11 +196,13 @@ class ZooKeeperEventQueue(ZooKeeperSimpleBase, Iterable):
 
     log = logging.getLogger("zuul.zk.event_queues.ZooKeeperEventQueue")
 
-    def __init__(self, client, event_root):
+    def __init__(self, client, queue_root):
         super().__init__(client)
-        self.event_root = event_root
+        self.queue_root = queue_root
+        self.event_root = f'{queue_root}/queue'
+        self.data_root = f'{queue_root}/data'
         self.kazoo_client.ensure_path(self.event_root)
-        self.kazoo_client.ensure_path(EVENT_DATA_ROOT)
+        self.kazoo_client.ensure_path(self.data_root)
 
     def _listEvents(self):
         return self.kazoo_client.get_children(self.event_root)
@@ -209,61 +225,89 @@ class ZooKeeperEventQueue(ZooKeeperSimpleBase, Iterable):
         if not self._remove(event.ack_ref.path, event.ack_ref.version):
             self.log.warning("Event %s was already acknowledged", event)
 
-    def _put(self, data):
-        # Within a transaction, we need something after the / in order
-        # to cause the sequence node to be created under the node
-        # before the /.  So we use "seq" for that.  This will produce
-        # paths like event_root/seq-0000.
-        event_path = f"{self.event_root}/seq"
-
+    def _put(self, data, updater=None):
         # Event data can be large, so we want to shard it.  But events
         # also need to be atomic (we don't want an event listener to
         # start processing a half-stored event).  A natural solution
-        # is to use a ZK transaction to write the sharded data along
-        # with the event.  However, our events are sequence nodes in
-        # order to maintain ordering, and we can't use our sharding
-        # helper to write shards underneath a sequence node inside the
-        # transaction because we don't know the path of the sequence
-        # node within the transaction.  To resolve this, we store the
-        # event data in two places: the event itself and associated
-        # metadata are in the event queue as a single sequence node.
-        # The event data are stored in a separate tree under a uuid.
-        # The event metadata includes the UUID of the data.  We call
-        # the separated data "side channel data" to indicate it's
-        # stored outside the main event queue.
+        # is to write the sharded data along with the event.  However,
+        # our events are sequence nodes in order to maintain ordering,
+        # and we can't write shard nodes under the event without
+        # creating the event first.  We can't use transactions because
+        # they are subject to the same size limit.  To resolve this,
+        # we store the event data in two places: the event itself and
+        # associated metadata are in the event queue as a single
+        # sequence node.  The event data are stored in a separate tree
+        # under a UUID.  We have to use a UUID because we don't know
+        # the sequence node when we write it.  The event metadata
+        # includes the UUID of the data.  We call the separated data
+        # "side channel data" to indicate it's stored outside the main
+        # event queue.
         #
         # To make the API simpler to work with, we assume "event_data"
         # contains the bulk of the data.  We extract it here into the
         # side channel data, then in _iterEvents we re-constitute it
         # into the dictionary.
+        #
+        # If `updater` is supplied, it is a RequestUpdater instance, and
+        # we are engaged in a cooperative transaction with a job
+        # request.
 
+        # Add a trailing /q for our sequence nodes; transaction
+        # sequence nodes need a character after / otherwise they drop
+        # the /.
+        event_path = f"{self.event_root}/q"
+
+        if updater and not updater.preRun():
+            # Don't even try the update
+            return
+
+        # Write the side channel data here under the assumption that
+        # the transaction will proceed.  If the transaction fails, it
+        # will be cleaned up after about 5 minutes.
         side_channel_data = None
-        encoded_data = json.dumps(data).encode("utf-8")
-        if (len(encoded_data) > sharding.NODE_BYTE_SIZE_LIMIT
+        size_limit = sharding.NODE_BYTE_SIZE_LIMIT
+        if updater:
+            # If we are in a transaction, leave enough room to share.
+            size_limit /= 2
+        encoded_data = json.dumps(data, sort_keys=True).encode("utf-8")
+        if (len(encoded_data) > size_limit
             and 'event_data' in data):
             # Get a unique data node
             data_id = str(uuid.uuid4())
-            data_root = f'{EVENT_DATA_ROOT}/{data_id}'
-            data_path = f'{data_root}/seq'
-            side_channel_data = json.dumps(data['event_data']).encode("utf-8")
+            data_root = f'{self.data_root}/{data_id}'
+            side_channel_data = json.dumps(data['event_data'],
+                                           sort_keys=True).encode("utf-8")
             data = data.copy()
             del data['event_data']
             data['event_data_path'] = data_root
-            encoded_data = json.dumps(data).encode("utf-8")
+            encoded_data = json.dumps(data, sort_keys=True).encode("utf-8")
 
-            tr = self.kazoo_client.transaction()
-            tr.create(data_root)
-
-            with sharding.BufferedShardWriter(tr, data_path) as stream:
+            with sharding.BufferedShardWriter(
+                    self.kazoo_client, data_root) as stream:
                 stream.write(side_channel_data)
 
-            tr.create(event_path, encoded_data, sequence=True)
-            resp = self.client.commitTransaction(tr)
-            return resp[-1]
-        else:
+        if updater is None:
             return self.kazoo_client.create(
                 event_path, encoded_data, sequence=True)
 
+        # The rest of the method is the updater case.  Start a
+        # transaction.
+        transaction = self.kazoo_client.transaction()
+
+        # Add transaction tasks.
+        transaction.create(event_path, encoded_data, sequence=True)
+        updater.run(transaction)
+
+        # Commit the transaction and process the results.  `results`
+        # is an array of either exceptions or return values
+        # corresponding to the operations in order.
+        result = transaction.commit()
+        if isinstance(result[0], Exception):
+            raise result[0]
+
+        updater.postRun(result[1])
+        return result[0]
+
     def _iterEvents(self):
         try:
             # We need to sort this ourself, since Kazoo doesn't guarantee any
@@ -332,17 +376,67 @@ class ZooKeeperEventQueue(ZooKeeperSimpleBase, Iterable):
         except NoNodeError:
             return False
 
+    def _findLostSideChannelData(self, age):
+        # Get data nodes which are older than the specified age (we
+        # don't want to delete nodes which are just being written
+        # slowly).
+        # Convert to MS
+        now = int(time.time() * 1000)
+        age = age * 1000
+        data_nodes = set()
+        for data_id in self.kazoo_client.get_children(self.data_root):
+            data_path = f'{self.data_root}/{data_id}'
+            data_zstat = self.kazoo_client.exists(data_path)
+            if now - data_zstat.mtime > age:
+                data_nodes.add(data_path)
 
-class ManagementEventResultFuture(ZooKeeperSimpleBase):
-    """Returned when a management event is put into a queue."""
+        # If there are no candidate data nodes, we don't need to
+        # filter them by known events.
+        if not data_nodes:
+            return data_nodes
 
-    log = logging.getLogger("zuul.zk.event_queues.MangementEventResultFuture")
+        events = set(self._listEvents())
+
+        for event_id in events:
+            # Load the event metadata
+            path = "/".join((self.event_root, event_id))
+            data, zstat = self.kazoo_client.get(path)
+            try:
+                event = json.loads(data)
+            except json.JSONDecodeError:
+                self.log.exception("Malformed event data in %s", path)
+                self._remove(path)
+                continue
+
+            side_channel_path = event.get('event_data_path')
+            if side_channel_path:
+                data_nodes.discard(side_channel_path)
+        return data_nodes
+
+    def cleanup(self, age=300):
+        try:
+            for path in self._findLostSideChannelData(age):
+                try:
+                    self.log.error("Removing side channel data: %s", path)
+                    self.kazoo_client.delete(path, recursive=True)
+                except Exception:
+                    self.log.execption(
+                        "Unable to delete side channel data %s", path)
+        except Exception:
+            self.log.exception("Error cleaning up event queue %s", self)
+
+
+class EventResultFuture(ZooKeeperSimpleBase):
+    """Base class for result futures for different events."""
+
+    log = logging.getLogger("zuul.zk.event_queues.EventResultFuture")
 
     def __init__(self, client, result_path):
         super().__init__(client)
         self._result_path = result_path
         self._wait_event = threading.Event()
         self.kazoo_client.DataWatch(self._result_path, self._resultCallback)
+        self.data = {}
 
     def _resultCallback(self, data=None, stat=None):
         if data is None:
@@ -352,6 +446,15 @@ class ManagementEventResultFuture(ZooKeeperSimpleBase):
         # Stop the watch if we got a result
         return False
 
+    def _read(self):
+        # Internal method to read the result data; may be overridden
+        # by subclasses.
+        return self.kazoo_client.get(self._result_path)[0]
+
+    def _delete(self):
+        with suppress(NoNodeError):
+            self.kazoo_client.delete(self._result_path, recursive=True)
+
     def wait(self, timeout=None):
         """Wait until the result for this event has been written."""
 
@@ -363,21 +466,86 @@ class ManagementEventResultFuture(ZooKeeperSimpleBase):
             return False
         try:
             try:
-                data, _ = self.kazoo_client.get(self._result_path)
-                result = json.loads(data.decode("utf-8"))
+                data = self._read()
+                self.data = json.loads(data.decode("utf-8"))
             except json.JSONDecodeError:
                 self.log.exception(
                     "Malformed result data in %s", self._result_path
                 )
                 raise
-            tb = result.get("traceback")
-            if tb is not None:
-                # TODO: raise some kind of ManagementEventException here
-                raise RuntimeError(tb)
+        finally:
+            self._delete()
+        return True
+
+
+class JobResultFuture(EventResultFuture):
+
+    log = logging.getLogger("zuul.JobResultFuture")
+
+    def __init__(self, client, request_path, result_path, waiter_path):
+        super().__init__(client, result_path)
+
+        self.request_path = request_path
+        self._waiter_path = waiter_path
+        self._result_data_path = None
+        self.merged = None
+        self.updated = None
+        self.commit = None
+        self.files = None
+        self.repo_state = None
+        self.item_in_branches = None
+
+    def _read(self):
+        result_node = self.kazoo_client.get(self._result_path)[0]
+        result = json.loads(result_node)
+        self._result_data_path = result['result_data_path']
+        with sharding.BufferedShardReader(
+                self.kazoo_client, self._result_data_path) as stream:
+            return stream.read()
+
+    def _delete(self):
+        super()._delete()
+        if self._result_data_path:
+            with suppress(NoNodeError):
+                self.kazoo_client.delete(self._result_data_path,
+                                         recursive=True)
+
+    def wait(self, timeout=None):
+        try:
+            res = super().wait(timeout)
         finally:
             with suppress(NoNodeError):
-                self.kazoo_client.delete(self._result_path)
-        return True
+                self.kazoo_client.delete(self._waiter_path)
+        self.merged = self.data.get("merged", False)
+        self.updated = self.data.get("updated", False)
+        self.commit = self.data.get("commit")
+        self.files = self.data.get("files", {})
+        self.repo_state = self.data.get("repo_state", {})
+        self.item_in_branches = self.data.get("item_in_branches", [])
+        return res
+
+    def cancel(self):
+        # Remove our waiter node so that if a result is ever reported,
+        # it will be garbage collected.
+        with suppress(NoNodeError):
+            self.kazoo_client.delete(self._waiter_path)
+
+
+class ManagementEventResultFuture(EventResultFuture):
+    """Returned when a management event is put into a queue."""
+
+    log = logging.getLogger("zuul.zk.event_queues.ManagementEventResultFuture")
+
+    def __init__(self, client, result_path: str):
+        super().__init__(client, result_path)
+
+    def wait(self, timeout=None):
+        res = super().wait(timeout)
+        tb = self.data.get("traceback")
+        if tb is not None:
+            # TODO: raise some kind of ManagementEventException here
+            raise RuntimeError(tb)
+        return res
 
 
 class ManagementEventQueue(ZooKeeperEventQueue):
@@ -428,8 +596,11 @@ class ManagementEventQueue(ZooKeeperEventQueue):
             if event.zuul_event_ltime is None:
                 event.zuul_event_ltime = zstat.creation_transaction_id
 
-            with suppress(ValueError):
+            try:
                 other_event = event_list[event_list.index(event)]
+            except ValueError:
+                other_event = None
+            if other_event:
                 if isinstance(other_event, model.TenantReconfigureEvent):
                     other_event.merge(event)
                     continue
@@ -455,11 +626,11 @@ class ManagementEventQueue(ZooKeeperEventQueue):
             return
 
         result_data = {"traceback": event.traceback,
-                       "timestamp": time.monotonic()}
+                       "timestamp": time.time()}
         try:
             self.kazoo_client.set(
                 event.result_ref,
-                json.dumps(result_data).encode("utf-8"),
+                json.dumps(result_data, sort_keys=True).encode("utf-8"),
             )
         except NoNodeError:
             self.log.warning(f"No result node found for {event}; "
@@ -472,10 +643,10 @@ class PipelineManagementEventQueue(ManagementEventQueue):
     )
 
     def __init__(self, client, tenant_name, pipeline_name):
-        event_root = PIPELINE_MANAGEMENT_ROOT.format(
+        queue_root = PIPELINE_MANAGEMENT_ROOT.format(
             tenant=tenant_name,
             pipeline=pipeline_name)
-        super().__init__(client, event_root)
+        super().__init__(client, queue_root)
 
     @classmethod
     def createRegistry(cls, client):
@@ -500,9 +671,9 @@ class TenantManagementEventQueue(ManagementEventQueue):
     log = logging.getLogger("zuul.zk.event_queues.TenantManagementEventQueue")
 
     def __init__(self, client, tenant_name):
-        event_root = TENANT_MANAGEMENT_ROOT.format(
+        queue_root = TENANT_MANAGEMENT_ROOT.format(
             tenant=tenant_name)
-        super().__init__(client, event_root)
+        super().__init__(client, queue_root)
 
     def ackWithoutResult(self, event):
         """
@@ -533,10 +704,10 @@ class PipelineResultEventQueue(ZooKeeperEventQueue):
     log = logging.getLogger("zuul.zk.event_queues.PipelineResultEventQueue")
 
     def __init__(self, client, tenant_name, pipeline_name):
-        event_root = PIPELINE_RESULT_ROOT.format(
+        queue_root = PIPELINE_RESULT_ROOT.format(
             tenant=tenant_name,
             pipeline=pipeline_name)
-        super().__init__(client, event_root)
+        super().__init__(client, queue_root)
 
     @classmethod
     def createRegistry(cls, client):
@@ -556,12 +727,12 @@ class PipelineResultEventQueue(ZooKeeperEventQueue):
     def _createRegistry(cls, client, tenant_name):
         return DefaultKeyDict(lambda p: cls(client, tenant_name, p))
 
-    def put(self, event):
+    def put(self, event, updater=None):
         data = {
             "event_type": type(event).__name__,
             "event_data": event.toDict(),
         }
-        self._put(data)
+        self._put(data, updater)
 
     def __iter__(self):
         for data, ack_ref, _ in self._iterEvents():
@@ -582,9 +753,9 @@ class TriggerEventQueue(ZooKeeperEventQueue):
 
     log = logging.getLogger("zuul.zk.event_queues.TriggerEventQueue")
 
-    def __init__(self, client, event_root, connections):
+    def __init__(self, client, queue_root, connections):
         self.connections = connections
-        super().__init__(client, event_root)
+        super().__init__(client, queue_root)
 
     def put(self, driver_name, event):
         data = {
@@ -593,12 +764,24 @@ class TriggerEventQueue(ZooKeeperEventQueue):
         }
         self._put(data)
 
+    def put_supercede(self, event):
+        data = {
+            "event_type": "SupercedeEvent",
+            "event_data": event.toDict(),
+            "driver_name": None,
+        }
+        self._put(data)
+
     def __iter__(self):
         for data, ack_ref, _ in self._iterEvents():
             try:
-                event_class = self.connections.getTriggerEventClass(
-                    data["driver_name"]
-                )
+                if (data["driver_name"] is None and
+                        data["event_type"] == "SupercedeEvent"):
+                    event_class = model.SupercedeEvent
+                else:
+                    event_class = self.connections.getTriggerEventClass(
+                        data["driver_name"]
+                    )
                 event_data = data["event_data"]
             except KeyError:
                 self.log.warning("Malformed event found: %s", data)
@@ -614,9 +797,9 @@ class TenantTriggerEventQueue(TriggerEventQueue):
     log = logging.getLogger("zuul.zk.event_queues.TenantTriggerEventQueue")
 
     def __init__(self, client, connections, tenant_name):
-        event_root = TENANT_TRIGGER_ROOT.format(
+        queue_root = TENANT_TRIGGER_ROOT.format(
             tenant=tenant_name)
-        super().__init__(client, event_root, connections)
+        super().__init__(client, queue_root, connections)
 
     @classmethod
     def createRegistry(cls, client, connections):
@@ -636,10 +819,10 @@ class PipelineTriggerEventQueue(TriggerEventQueue):
     log = logging.getLogger("zuul.zk.event_queues.PipelineTriggerEventQueue")
 
     def __init__(self, client, tenant_name, pipeline_name, connections):
-        event_root = PIPELINE_TRIGGER_ROOT.format(
+        queue_root = PIPELINE_TRIGGER_ROOT.format(
             tenant=tenant_name,
             pipeline=pipeline_name)
-        super().__init__(client, event_root, connections)
+        super().__init__(client, queue_root, connections)
 
     @classmethod
     def createRegistry(cls, client, connections):
@@ -671,13 +854,14 @@ class ConnectionEventQueue(ZooKeeperEventQueue):
     log = logging.getLogger("zuul.zk.event_queues.ConnectionEventQueue")
 
     def __init__(self, client, connection_name):
-        event_root = "/".join((CONNECTION_ROOT, connection_name, "events"))
-        super().__init__(client, event_root)
+        queue_root = "/".join((CONNECTION_ROOT, connection_name, "events"))
+        super().__init__(client, queue_root)
         self.election_root = "/".join(
             (CONNECTION_ROOT, connection_name, "election")
         )
         self.kazoo_client.ensure_path(self.election_root)
-        self.election = self.kazoo_client.Election(self.election_root)
+        self.election = SessionAwareElection(
+            self.kazoo_client, self.election_root)
 
     def _eventWatch(self, callback, event_list):
         if event_list:
@@ -706,7 +890,7 @@ class ConnectionEventQueue(ZooKeeperEventQueue):
             yield event
 
 
-class EventReceiverElection(Election):
+class EventReceiverElection(SessionAwareElection):
     """Election for a singleton event receiver."""
 
     def __init__(self, client, connection_name, receiver_name):
@@ -714,3 +898,11 @@ class EventReceiverElection(Election):
             (CONNECTION_ROOT, connection_name, f"election-{receiver_name}")
         )
         super().__init__(client.client, self.election_root)
+
+
+class NodepoolEventElection(SessionAwareElection):
+    """Election for the nodepool completion event processor."""
+
+    def __init__(self, client):
+        self.election_root = "/zuul/nodepool/election"
+        super().__init__(client.client, self.election_root)
