@@ -27,5 +27,5 @@ class NoClientException(ZuulZooKeeperException):
         super().__init__("No zookeeper client!")
 
 
-class BuildRequestNotFound(ZuulZooKeeperException):
+class JobRequestNotFound(ZuulZooKeeperException):
     pass
