@@ -13,19 +13,31 @@
 import json
 import logging
 import time
-from typing import Dict, Optional, List
+from enum import Enum
+from typing import Optional, List
+import threading
 
 from kazoo.exceptions import NoNodeError, LockTimeout
-from kazoo.recipe.cache import TreeCache
-from kazoo.recipe.cache import TreeEvent
+from kazoo.recipe.cache import TreeCache, TreeEvent
 from kazoo.recipe.lock import Lock
 
 import zuul.model
-from zuul.model import HoldRequest, Node, NodeRequest, NodeSet
-from zuul.zk import ZooKeeperClient, ZooKeeperBase
+from zuul.lib.jsonutil import json_dumps
+from zuul.model import HoldRequest, NodeRequest, Node
+from zuul.zk import ZooKeeperBase
 from zuul.zk.exceptions import LockException
 
 
+class NodeRequestEvent(Enum):
+    COMPLETED = 0  # FULFILLED | FAILED
+    DELETED = 2
+
+
+class NodeEvent(Enum):
+    CHANGED = 0
+    DELETED = 2
+
+
 class ZooKeeperNodepool(ZooKeeperBase):
     """
     Class implementing Nodepool related ZooKeeper interface.
@@ -38,32 +50,50 @@ class ZooKeeperNodepool(ZooKeeperBase):
 
     log = logging.getLogger("zuul.zk.nodepool.ZooKeeperNodepool")
 
-    def __init__(self, client: ZooKeeperClient, enable_cache: bool = True):
+    def __init__(self, client,
+                 enable_node_request_cache=False,
+                 node_request_event_callback=None,
+                 enable_node_cache=False):
         super().__init__(client)
-        self.enable_cache = enable_cache  # type: bool
+        self.enable_node_request_cache = enable_node_request_cache
+        self.node_request_event_callback = node_request_event_callback
+        self.enable_node_cache = enable_node_cache
         # The caching model we use is designed around handing out model
         # data as objects. To do this, we use two caches: one is a TreeCache
         # which contains raw znode data (among other details), and one for
         # storing that data serialized as objects. This allows us to return
         # objects from the APIs, and avoids calling the methods to serialize
         # the data into objects more than once.
-        self._hold_request_tree: Optional[TreeCache] = None
-        self._cached_hold_requests: Optional[Dict[str, HoldRequest]] = {}
+        self._node_request_tree = None
+        self._node_request_cache = {}
+        self._node_tree = None
+        self._node_cache = {}
+        self._callback_lock = threading.Lock()
+
         if self.client.connected:
             self._onConnect()
 
     def _onConnect(self):
-        if self.enable_cache:
-            self._hold_request_tree = TreeCache(self.kazoo_client,
-                                                self.HOLD_REQUEST_ROOT)
-            self._hold_request_tree.listen_fault(self._cacheFaultListener)
-            self._hold_request_tree.listen(self._holdRequestCacheListener)
-            self._hold_request_tree.start()
+        if self.enable_node_request_cache:
+            self._node_request_tree = TreeCache(self.kazoo_client,
+                                                self.REQUEST_ROOT)
+            self._node_request_tree.listen_fault(self._cacheFaultListener)
+            self._node_request_tree.listen(self.requestCacheListener)
+            self._node_request_tree.start()
+        if self.enable_node_cache:
+            self._node_tree = TreeCache(self.kazoo_client,
+                                        self.NODES_ROOT)
+            self._node_tree.listen_fault(self._cacheFaultListener)
+            self._node_tree.listen(self.nodeCacheListener)
+            self._node_tree.start()
 
     def _onDisconnect(self):
-        if self._hold_request_tree is not None:
-            self._hold_request_tree.close()
-            self._hold_request_tree = None
+        if self._node_request_tree is not None:
+            self._node_request_tree.close()
+            self._node_request_tree = None
+        if self._node_tree is not None:
+            self._node_tree.close()
+            self._node_tree = None
 
     def _launcherPath(self, launcher):
         return "%s/%s" % (self.LAUNCHER_ROOT, launcher)
@@ -74,6 +104,9 @@ class ZooKeeperNodepool(ZooKeeperBase):
     def _cacheFaultListener(self, e):
         self.log.exception(e)
 
+    def register(self):
+        self.kazoo_client.ensure_path(self.REQUEST_ROOT)
+
     def getRegisteredLaunchers(self):
         """
         Get a list of all launchers that have registered with ZooKeeper.
@@ -98,18 +131,23 @@ class ZooKeeperNodepool(ZooKeeperBase):
             objs.append(Launcher.fromDict(json.loads(data.decode('utf8'))))
         return objs
 
-    def getNodes(self):
+    def getNodes(self, cached=False):
         """
-        Get the current list of all nodes.
+        Get the current list of all node ids.
 
-        :returns: A list of nodes.
+        :param bool cached: Whether to use the internal cache to get the list
+            of ids.
+        :returns: A list of node ids.
         """
-        try:
-            return self.kazoo_client.get_children(self.NODES_ROOT)
-        except NoNodeError:
-            return []
+        if cached and self.enable_node_cache:
+            return list(self._node_cache.keys())
+        else:
+            try:
+                return self.kazoo_client.get_children(self.NODES_ROOT)
+            except NoNodeError:
+                return []
 
-    def _getNode(self, node):
+    def _getNodeData(self, node):
         """
         Get the data for a specific node.
 
@@ -129,12 +167,38 @@ class ZooKeeperNodepool(ZooKeeperBase):
         d['id'] = node
         return d
 
-    def nodeIterator(self):
+    def getNode(self, node_id):
+        """
+        Get a Node object for a specific node.
+
+        :param str node_id: The node ID.
+
+        :returns: The Node, or None if the node was not found.
+        """
+        node = self._node_cache.get(node_id)
+        if node:
+            return node
+
+        path = self._nodePath(node_id)
+        try:
+            data, stat = self.kazoo_client.get(path)
+        except NoNodeError:
+            return None
+        if not data:
+            return None
+
+        node = Node(None, None)
+        node.updateFromDict(data)
+        node.id = node_id
+        node.stat = stat
+        return node
+
+    def nodeIterator(self, cached=False):
         """
         Utility generator method for iterating through all nodes.
         """
-        for node_id in self.getNodes():
-            node = self._getNode(node_id)
+        for node_id in self.getNodes(cached):
+            node = self.getNode(node_id)
             if node:
                 yield node
 
@@ -219,7 +283,7 @@ class ZooKeeperNodepool(ZooKeeperBase):
 
         failure = False
         for node_id in getHeldNodeIDs(request):
-            node = self._getNode(node_id)
+            node = self._getNodeData(node_id)
             if not node or node['state'] == zuul.model.STATE_USED:
                 continue
 
@@ -316,99 +380,121 @@ class ZooKeeperNodepool(ZooKeeperBase):
         request.lock.release()
         request.lock = None
 
-    def _holdRequestCacheListener(self, event):
-        """
-        Keep the hold request object cache in sync with the TreeCache.
-        """
+    def requestCacheListener(self, event):
         try:
-            if hasattr(event.event_data, 'path'):
-                # Ignore root node
-                path = event.event_data.path
-                if path == self.HOLD_REQUEST_ROOT:
-                    return
-
-            if event.event_type not in (TreeEvent.NODE_ADDED,
-                                        TreeEvent.NODE_UPDATED,
-                                        TreeEvent.NODE_REMOVED):
-                return
-
-            path = event.event_data.path
-            request_id = path.rsplit('/', 1)[1]
-
-            if event.event_type in (
-                    TreeEvent.NODE_ADDED, TreeEvent.NODE_UPDATED):
-                # Requests with no data are invalid
-                if not event.event_data.data:
-                    return
-
-                # Perform an in-place update of the already cached request
-                d = json.loads(event.event_data.data.decode('utf8'))
-                old_request = self._cached_hold_requests.get(request_id)
-                if old_request:
-                    if event.event_data.stat.version <= old_request.stat\
-                            .version:
-                        # Don't update to older data
-                        return
-                    old_request.updateFromDict(d)
-                    old_request.stat = event.event_data.stat
-                else:
-                    request = HoldRequest.fromDict(d)
-                    request.id = request_id
-                    request.stat = event.event_data.stat
-                    self._cached_hold_requests[request_id] = request
-
-            elif event.event_type == TreeEvent.NODE_REMOVED:
-                try:
-                    del self._cached_hold_requests[request_id]
-                except KeyError:
-                    pass
+            # This lock is only needed for tests.  It wraps updating
+            # our internal cache and performing the callback which
+            # will generate node provisioned events in a critical
+            # section.  This way the tests know if a request is
+            # outstanding because either our cache is out of date or
+            # there is an event on the queue.
+            with self._callback_lock:
+                self._requestCacheListener(event)
         except Exception:
             self.log.exception(
-                "Exception in hold request cache update for event: %s", event)
+                "Exception in request cache update for event: %s",
+                event)
 
-    def submitNodeRequest(self, node_request, priority, watcher):
+    def _requestCacheListener(self, event):
+        if hasattr(event.event_data, 'path'):
+            # Ignore root node
+            path = event.event_data.path
+            if path == self.REQUEST_ROOT:
+                return
+
+            # Ignore lock nodes
+            if '/lock' in path:
+                return
+
+        # Ignore any non-node related events such as connection events here
+        if event.event_type not in (TreeEvent.NODE_ADDED,
+                                    TreeEvent.NODE_UPDATED,
+                                    TreeEvent.NODE_REMOVED):
+            return
+
+        path = event.event_data.path
+        request_id = path.rsplit('/', 1)[1]
+
+        if event.event_type in (TreeEvent.NODE_ADDED, TreeEvent.NODE_UPDATED):
+            # Requests with empty data are invalid so skip add or update these.
+            if not event.event_data.data:
+                return
+
+            # Perform an in-place update of the cached request if possible
+            d = self._bytesToDict(event.event_data.data)
+            request = self._node_request_cache.get(request_id)
+            if request:
+                if event.event_data.stat.version <= request.stat.version:
+                    # Don't update to older data
+                    return
+                old_state = request.state
+                request.updateFromDict(d)
+                request.stat = event.event_data.stat
+            else:
+                old_state = None
+                request = NodeRequest.fromDict(d)
+                request.id = request_id
+                request.stat = event.event_data.stat
+                self._node_request_cache[request_id] = request
+
+            if (request.state in {zuul.model.STATE_FULFILLED,
+                                  zuul.model.STATE_FAILED}
+                and request.state != old_state
+                and self.node_request_event_callback):
+                self.node_request_event_callback(
+                    request, NodeRequestEvent.COMPLETED)
+
+        elif event.event_type == TreeEvent.NODE_REMOVED:
+            request = self._node_request_cache.pop(request_id)
+            if self.node_request_event_callback:
+                self.node_request_event_callback(
+                    request, NodeRequestEvent.DELETED)
+
+    def submitNodeRequest(self, node_request, priority):
         """
         Submit a request for nodes to Nodepool.
 
         :param NodeRequest node_request: A NodeRequest with the
             contents of the request.
-
-        :param callable watcher: A callable object that will be
-            invoked each time the request is updated.  It is called
-            with two arguments: (node_request, deleted) where
-            node_request is the same argument passed to this method,
-            and deleted is a boolean which is True if the node no
-            longer exists (notably, this will happen on disconnection
-            from ZooKeeper).  The watcher should return False when
-            further updates are no longer necessary.
         """
         node_request.created_time = time.time()
         data = node_request.toDict()
 
         path = '{}/{:0>3}-'.format(self.REQUEST_ROOT, priority)
         path = self.kazoo_client.create(path, json.dumps(data).encode('utf8'),
-                                        makepath=True, sequence=True,
-                                        ephemeral=True)
+                                        makepath=True, sequence=True)
         reqid = path.split("/")[-1]
         node_request.id = reqid
 
-        def callback(value, _):
-            if value:
-                self.updateNodeRequest(node_request, value)
-            deleted = (value is None)  # data *are* none
-            return watcher(node_request, priority, deleted)
+    def getNodeRequests(self, cached=False):
+        '''
+        Get the current list of all node request IDs in priority sorted order.
 
-        self.kazoo_client.DataWatch(path, callback)
+        :param bool cached: Whether to use the internal cache to get the list
+            of ids.
+        :returns: A list of request ids.
+        '''
+        if cached and self.enable_node_request_cache:
+            requests = self._node_request_cache.keys()
+        else:
+            try:
+                requests = self.kazoo_client.get_children(self.REQUEST_ROOT)
+            except NoNodeError:
+                return []
 
-    def getNodeRequest(self, node_request_id, nodeset=None):
+        return sorted(requests)
+
+    def getNodeRequest(self, node_request_id, cached=False):
         """
         Retrieve a NodeRequest from a given path in ZooKeeper.
 
-        The serialized version of the NodeRequest doesn't contain a NodeSet, so
-        we have to add this to the request manually. The nodeset provided to
-        this method will be set on the NodeRequest before updating it. This
-        will ensure that all nodes are updated as well.
+        :param str node_request_id: The ID of the node request to retrieve.
+        :param bool cached: Whether to use the cache.
         """
+        if cached:
+            req = self._node_request_cache.get(node_request_id)
+            if req:
+                return req
 
         path = f"{self.REQUEST_ROOT}/{node_request_id}"
         try:
@@ -422,44 +508,24 @@ class ZooKeeperNodepool(ZooKeeperBase):
         json_data = json.loads(data.decode("utf-8"))
 
         obj = NodeRequest.fromDict(json_data)
-        if nodeset is None:
-            # If no NodeSet is provided, we create one on-the-fly based on the
-            # list of labels (node_types) stored in the NodeRequest's znode
-            # data.
-            # This is necessary as the logic in the updateNodeRequest() method
-            # below will update each node "in-place" an thus, we have to ensure
-            # that the NodeRequest has a valid NodeSet with all nodes available
-            # in advance.
-            # This is only used to return the nodes to nodepool in case
-            # something went wrong and the original NodeRequest and/or NodeSet
-            # objects are not available anymore.
-            nodeset = NodeSet()
-            for i, node_type in enumerate(json_data["node_types"]):
-                node = Node(name=f"{node_type}-{i}", label=node_type)
-                nodeset.addNode(node)
-
-        obj.nodeset = nodeset
-        # Using updateNodeRequest() here will ensure that the nodes are also
-        # updated. Doing the update in here directly rather than calling it
-        # from the outside afterwards, saves us one call to ZooKeeper as the
-        # data we just retrieved can directly be reused.
-        self.updateNodeRequest(obj, data)
         obj.id = node_request_id
         obj.stat = stat
         return obj
 
-    def deleteNodeRequest(self, node_request):
+    def deleteNodeRequest(self, node_request_id):
         """
         Delete a request for nodes.
 
-        :param NodeRequest node_request: A NodeRequest with the
-            contents of the request.
+        :param str node_request_id: The node request ID.
+
+        :returns: True if the request was deleted, False if it did not exist.
         """
-        path = '%s/%s' % (self.REQUEST_ROOT, node_request.id)
+        path = '%s/%s' % (self.REQUEST_ROOT, node_request_id)
         try:
             self.kazoo_client.delete(path)
+            return True
         except NoNodeError:
-            pass
+            return False
 
     def nodeRequestExists(self, node_request):
         """
@@ -485,7 +551,8 @@ class ZooKeeperNodepool(ZooKeeperBase):
         """
         path = '%s/%s' % (self.REQUEST_ROOT, node_request.id)
         self.kazoo_client.set(
-            path, json.dumps(node_request.toDict()).encode('utf8'))
+            path, json.dumps(node_request.toDict()).encode('utf8'),
+            version=node_request.stat.version)
 
     def updateNodeRequest(self, node_request, data=None):
         """
@@ -498,12 +565,68 @@ class ZooKeeperNodepool(ZooKeeperBase):
             path = '%s/%s' % (self.REQUEST_ROOT, node_request.id)
             data, stat = self.kazoo_client.get(path)
         data = json.loads(data.decode('utf8'))
-        request_nodes = list(node_request.nodeset.getNodes())
-        for i, nodeid in enumerate(data.get('nodes', [])):
-            request_nodes[i].id = nodeid
-            self._updateNode(request_nodes[i])
         node_request.updateFromDict(data)
 
+    def nodeCacheListener(self, event):
+        try:
+            self._nodeCacheListener(event)
+        except Exception:
+            self.log.exception(
+                "Exception in node cache update for event: %s",
+                event)
+
+    def _nodeCacheListener(self, event):
+        # Ignore events without data
+        if event.event_data is None:
+            # This seems to be the case for some NODE_REMOVED events.
+            # Without the data we are not able to pop() the correct node
+            # from our cache dict. As we can't do much else, ignore the
+            # event.
+            return
+
+        if hasattr(event.event_data, 'path'):
+            # Ignore root node
+            path = event.event_data.path
+            if path == self.NODES_ROOT:
+                return
+
+            # Ignore lock nodes
+            if '/lock' in path:
+                return
+
+        # Ignore any non-node related events such as connection events here
+        if event.event_type not in (TreeEvent.NODE_ADDED,
+                                    TreeEvent.NODE_UPDATED,
+                                    TreeEvent.NODE_REMOVED):
+            return
+
+        path = event.event_data.path
+        node_id = path.rsplit('/', 1)[1]
+
+        if event.event_type in (TreeEvent.NODE_ADDED, TreeEvent.NODE_UPDATED):
+            # Nodes with empty data are invalid so skip add or update these.
+            if not event.event_data.data:
+                return
+
+            # Perform an in-place update of the cached node if possible
+            d = self._bytesToDict(event.event_data.data)
+            node = self._node_cache.get(node_id)
+            if node:
+                if event.event_data.stat.version <= node.stat.version:
+                    # Don't update to older data
+                    return
+                node.updateFromDict(d)
+                node.stat = event.event_data.stat
+            else:
+                node = Node(None, None)
+                node.updateFromDict(d)
+                node.id = node_id
+                node.stat = event.event_data.stat
+                self._node_cache[node_id] = node
+
+        elif event.event_type == TreeEvent.NODE_REMOVED:
+            node = self._node_cache.pop(node_id, None)
+
     def storeNode(self, node):
         """
         Store the node.
@@ -516,13 +639,18 @@ class ZooKeeperNodepool(ZooKeeperBase):
         path = '%s/%s' % (self.NODES_ROOT, node.id)
         self.kazoo_client.set(path, json.dumps(node.toDict()).encode('utf8'))
 
-    def _updateNode(self, node):
+    def updateNode(self, node, node_id):
         """
         Refresh an existing node.
 
         :param Node node: The node to update.
+
+        :param str node_id: The ID of the node to update.  The
+            existing node.id attribute (if any) will be ignored
+            and replaced with this.
         """
-        node_path = '%s/%s' % (self.NODES_ROOT, node.id)
+        node_path = '%s/%s' % (self.NODES_ROOT, node_id)
+        node.id = node_id
         node_data, node_stat = self.kazoo_client.get(node_path)
         node_data = json.loads(node_data.decode('utf8'))
         node.updateFromDict(node_data)
@@ -601,7 +729,6 @@ class ZooKeeperNodepool(ZooKeeperBase):
             raise LockException("Did not get lock on %s" % path)
 
         request.lock = lock
-        self.updateNodeRequest(request)
 
     def unlockNodeRequest(self, request):
         """
@@ -649,6 +776,15 @@ class ZooKeeperNodepool(ZooKeeperBase):
                 count += 1
         return count
 
+    @staticmethod
+    def _bytesToDict(data):
+        return json.loads(data.decode("utf-8"))
+
+    @staticmethod
+    def _dictToBytes(data):
+        # The custom json_dumps() will also serialize MappingProxyType objects
+        return json_dumps(data).encode("utf-8")
+
 
 class Launcher:
     """
